﻿using System;
using System.Collections.Generic;
using System.Reflection;
using System.Reflection.Emit;
using System.ComponentModel;
using Styx;
using Styx.Common;
using Styx.CommonBot.CharacterManagement;
using Styx.Helpers;
using Styx.WoWInternals.WoWObjects;
using DefaultValue = Styx.Helpers.DefaultValueAttribute;

namespace KeepOnMoving.Settings
{
    public enum QuestingPack
    {
        None = 0,
        Old,
        New
    }

    public enum PandarenFaction
    {
        NotYetChosen = 0,
        Horde,
        Alliance
    }

    public enum TravelForm
    {
        None = 0,
        GroundMountOnly,
        FlyingMountOnly,
        Both
    }

    public enum GatheringProfessions
    {
        None = 0,
        Herbalism,
        HerbalismAndMining,
        // HerbalismAndSkinning,
        Mining
        // MiningAndSkinning,
        // Skinning
    }

    public enum TrainFlying
    {
        None = 0,
        ArtisanOnly,
        ArtisanAndMaster
    }

    public enum HandleNodes
    {
        Disabled = 0,
        BotBaseOnly,
        PluginOnly,
        Both
    }

    public enum Fragments
    {
        None        = 0,
        Dwarven     = 87533,
        Draenei     = 87534,
        Fossil      = 87535,
        NightElf    = 87536,
        Nerubian    = 87537,
        Orc         = 87538,
        Tolvir      = 87539,
        Troll       = 87540,
        Vyrkul      = 87541
    }

    public enum PreferNode
    {
        Disabled = 0,
        AlwaysLowestSkill,
        LowestSkillPreDraenorOnly,
        Herb,
        Mineral
    }

    public enum HarvestMobs
    {
        Disabled = 0,
        MineralsOnly,
        HerbsOnly,
        Both
    }

    public enum TestMode
    {
        Disabled = 0,
        StarlightRoseRank3,
        FjarnskagglRank3,
        DreamleafRank3,
        FelslateRank3Deposit,
        FelslateRank3Seam,
        FelslateRank3Living
    }

    public enum ApplyRelics
    {
        Disabled = 0,
        Enabled,
        KeepAffectedTrait
    }

    public enum Treasures
    {
        Disabled = 0,
        Enabled,
        PreferTreasuresToNodes
    }

    public enum RareMobs
    {
        Disabled = 0,
        Enabled,
        PreferRareMobsToNodes
    }

    public enum MogItems
    {
        Disabled = 0,
        Green,
        Blue,
        Purple
    }

    public class KOMSettings : Styx.Helpers.Settings
    {
        private static KOMSettings _instance;
        private static LocalPlayer Me => StyxWoW.Me;
        public static KOMSettings Instance => _instance ?? (_instance = new KOMSettings());
        public static string _settingsFile => string.Format("{0}\\Settings\\KeepOnMoving\\{1}_{2}.config", 
                                                                            Utilities.AssemblyDirectory, 
                                                                            Me.RealmName.Replace(" ", ""), 
                                                                            Me.Name);
        public Dictionary<object, object> _settingsDict = new Dictionary<object, object>();

        private KOMSettings() : base(_settingsFile)
        {
        }

        [Setting]
        [DefaultValue(TestMode.Disabled)]
        [Category("- TEST Settings")]
        [DisplayName("Set Mode")]
        [Description("Sets mode being performed --- Disabled")]
        public TestMode Test_Mode { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(60)]
        [Category("- MOTIONLESS Settings")]
        [DisplayName("Timer")]
        [Description("Amount of seconds not having moved before motionless procedure begins (\"0\" to disable) [NOTE: This setting will not take effect if you are using \"Questing Profile Pack\" shipped with HB or from store] --- 60")]
        public int Motionless_Timer { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- MOTIONLESS Settings")]
        [DisplayName("Objective Timer")]
        [Description("Amount of seconds objective not having changed before motionless procedure begins (\"0\" to disable) --- 0")]
        public int Motionless_ObjectiveTimer { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- STUCK Settings")]
        [DisplayName("Force Dismount")]
        [Description("Immediately dismounts when ground mounted and stuck is detected --- False")]
        public bool Stuck_Dismount { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(5)]
        [Category("- STUCK Settings")]
        [DisplayName("Blackspot Radius")]
        [Description("Blackspots current location whenever stuck occurs (\"0\" to disable) --- 5")]
        public double Stuck_BlackspotRadius { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(30)]
        [Category("- STUCK Settings")]
        [DisplayName("Speed Tolerance")]
        [Description("When char's speed falls below this percentage of default speed, StuckCount will be increased for this tick (1 - 100) [NOTE: Decrease to slow down, increase to speed up stuck detection] --- 30")]
        public int Stuck_SpeedTolerance { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(7)]
        [Category("- STUCK Settings")]
        [DisplayName("StuckCount Threshold")]
        [Description("If StuckCount reaches this value, char is considered as being stuck and unstuck handling is triggered [NOTE: Increase to slow down, decrese to speed up stuck detection] --- 5")]
        public int Stuck_StuckCountThreshold { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- STUCK Settings")]
        [DisplayName("Disable Stuck Handling")]
        [Description("Prevents KoM from handling stucks --- False")]
        public bool Stuck_Disable { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- DIALOG Settings")]
        [DisplayName("Decline Duel Invites")]
        [Description("Declines duel invites --- False")]
        public bool Dialog_DeclineDuelInvites { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- DIALOG Settings")]
        [DisplayName("Decline Party Invites")]
        [Description("Declines party invites --- False")]
        public bool Dialog_DeclinePartyInvites { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- DIALOG Settings")]
        [DisplayName("Decline Guild Invites")]
        [Description("Declines guild invites --- False")]
        public bool Dialog_DeclineGuildInvites { get; set; }

        [Setting]
        [DefaultValue(GatheringProfessions.None)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Set Gathering Professions")]
        [Description("Handles certain tasks concerning gathering professions --- None")]
        public GatheringProfessions GatheringProfessions_Professions { get; set; }

        [Setting]
        [DefaultValue(HandleNodes.PluginOnly)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Handle Nodes")]
        [Description("Sets who is allowed to handle nodes --- PluginOnly")]
        public HandleNodes GatheringProfessions_HandleNodes { get; set; }

        [Setting]
        [DefaultValue(PreferNode.LowestSkillPreDraenorOnly)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Prefer Node")]
        [Description("In case mineral and herb node have spawned at the same time, this setting determines, which one KoM will prefer to gather --- LowestSkillPreDraenorOnly")]
        public PreferNode GatheringProfessions_PreferNode { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Gather Grey Nodes")]
        [Description("Gathers nodes although they will not increase skill --- False")]
        public bool GatheringProfessions_GatherGrey { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Enable Dungeon Quest (Felslate)")]
        [Description("Enabled dungeon quest for Felslate (Rank 2) --- True")]
        public bool GatheringProfessions_DungeonQuest { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Skill Ranks")]
        [Description("Skills ranks --- True")]
        public bool GatheringProfessions_Ranks { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Train Skill")]
        [Description("Trains skill --- True")]
        public bool GatheringProfessions_TrainSkill { get; set; }

        [Setting]
        [DefaultValue(HarvestMobs.Both)]
        [Category("- GATHERING PROFESSIONS Settings")]
        [DisplayName("Harvest Mobs")]
        [Description("Defines the type of mobs being gathered --- Both")]
        public HarvestMobs GatheringProfessions_HarvestMobs { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Go to Pandaria")]
        [Description("Goes to Pandaria when skill value becomes suitable --- False")]
        public bool Archaeology_GoToPandaria { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Go to Draenor")]
        [Description("Goes to Draenor when skill value becomes suitable --- False")]
        public bool Archaeology_GoToDraenor { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Go to Broken Isles")]
        [Description("Goes to Broken Isles when skill value becomes suitable --- False")]
        public bool Archaeology_GoToBrokenIsles { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Train Archaeology")]
        [Description("Trains at trainer [NOTE: \"Set Hearthstone\" needs to be enabled] --- True")]
        public bool Archaeology_Train { get; set; }

        [Setting]
        [DefaultValue(Fragments.None)]
        [Category("- ARCHAEOLOGY Settings")]
        [DisplayName("Buy Fragments")]
        [Description("Buys fragments from Brann Bronzebeard in Pandaria [NOTE: \"Open Items\" should be enabled as fragments couldn't be used otherwise] --- None")]
        public Fragments Archaeology_Fragments { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- JUMP Settings")]
        [DisplayName("Enable Jump")]
        [Description("Jumps randomly between minimum and maximum pause --- True")]
        public bool Jump_Enable { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(5000)]
        [Category("- JUMP Settings")]
        [DisplayName("Maximum Pause")]
        [Description("Maximum pause between jumps in milliseconds --- 3000")]
        public int Jump_MaxPause { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(1000)]
        [Category("- JUMP Settings")]
        [DisplayName("Minimum Pause")]
        [Description("Minimum pause between jumps in milliseconds --- 1000")]
        public int Jump_MinPause { get; set; }

        [Setting]
        [DefaultValue(PandarenFaction.NotYetChosen)]
        [Category("- QUEST Settings")]
        [DisplayName("Set Pandaren Faction")]
        [Description("Skip message box for choosing faction by loading subprofile instead; this takes effect for starting quests only --- NotYetChosen")]
        public PandarenFaction Quest_PandarenFaction { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- LOG Settings")]
        [DisplayName("Enable Runtime Logging")]
        [Description("Logs runtimes. Enable this setting when debugging perfomance related issues only --- False")]
        public bool Log_Runtime { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Sort Bags")]
        [Description("Sorts bags --- True")]
        public bool Bag_Sort { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Create Heirlooms")]
        [Description("Creates and equips heirlooms --- True")]
        public bool Bag_Heirlooms { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Mail Items")]
        [Description("Mails items (low level items like flasks, etc. included) --- True")]
        public bool Bag_MailItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Combine Items")]
        [Description("Combines items (like pre herbs/ores) --- True")]
        public bool Bag_CombineItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Open Items")]
        [Description("Opens items (like clams) --- True")]
        public bool Bag_OpenItems { get; set; }

        [Setting]
        [DefaultValue(ApplyRelics.Enabled)]
        [Category("- ARTIFACT Settings")]
        [DisplayName("Apply Artifact Relics")]
        [Description("Apply artifact relics to artifact weapon --- True")]
        public ApplyRelics Artifact_ApplyRelics { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- ARTIFACT Settings")]
        [DisplayName("Use Artifact Power Items")]
        [Description("Uses items increasing artifact power --- True")]
        public bool Artifact_PowerItems { get; set; }

        /*
        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- ARTIFACT Settings")]
        [DisplayName("Use Artifact Knowledge Items")]
        [Description("Uses items increasing artifact knowledge --- True")]
        public bool Artifact_KnowledgeItems { get; set; }
        */

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- ARTIFACT Settings")]
        [DisplayName("Protect Artifact Relics")]
        [Description("Protects artifact relics from being vendored --- True")]
        public bool Artifact_ProtectRelics { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Use Ancient Mana Items")]
        [Description("Uses items increasing ancient mana --- True")]
        public bool Bag_AncientManaItems { get; set; }

        [Setting]
        [DefaultValue(MogItems.Green)]
        [Category("- BAG Settings")]
        [DisplayName("Collect Transmogrification Template")]
        [Description("Maximum item quality transmogrification template is collected from --- Green")]
        public MogItems Bag_MogItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Use Reputation Items")]
        [Description("Uses items increasing reputation --- True")]
        public bool Bag_ReputationItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Learn Pets")]
        [Description("Learns pets --- True")]
        public bool Bag_LearnPets { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Learn Toys")]
        [Description("Learns toys --- True")]
        public bool Bag_LearnToys { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Remove Already Known Toy Items")]
        [Description("Removes already known toy items --- True")]
        public bool Bag_RemoveToys { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- BAG Settings")]
        [DisplayName("Remove Low Level Quest Items")]
        [Description("Removes low level quest items --- False")]
        public bool Bag_RemoveQuestItems { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Manage Containers")]
        [Description("Equips container items or exchange with better ones --- True")]
        public bool Bag_ManageContainers { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- BAG Settings")]
        [DisplayName("Get New Mails")]
        [Description("Gets new mails (attachments like items or gold included) --- True")]
        public bool Bag_GetMails { get; set; }

        /*
        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- GENERAL Settings")]
        [DisplayName("Set Favourite GroundMount")]
        [Description("set ground mount to this mount name as soon as it is available in HB settings(\"0\" to disable)\nNOTE: Mount id could be used as well\n\n                                  <Default: 0>")]
        public string G_GroundMount { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- GENERAL Settings")]
        [DisplayName("Set Favourite FlyingMount")]
        [Description("set flying mount to this mount name as sonn as it is available in HB settings(\"0\" to disable)\nNOTE: Mount id could be used as well\n\n                                  <Default: 0>")]
        public string G_FlyingMount { get; set; }


        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Set Hearthstone")]
        [Description("set hearthstone either to Orgrimmar/Stormwind or Shrine of Two Moons/Seven Stars [NOTE: This setting is crucial for some features to work properly, e.g. training skills --- True")]
        public bool Hearth_SetHearthstone { get; set; }
        */

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Class Specific: Death Knight")]
        [Description("uUe Death Knight's \"Death Gate\" --- True")]
        public bool Hearth_DeathKnight { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Class Specific: Druid")]
        [Description("Use Druid's \"Teleport: Moonglade\"/\"Dreamwalk\" --- True")]
        public bool Hearth_Druid { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Class Specific: Monk")]
        [Description("Use Monk's \"Zen Pilgrimage\" --- True")]
        public bool Hearth_Monk { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Hearthstone\"")]
        [Description("Uses \"Hearthstone\" [NOTE: Training skills for archaeology requires hearthstone to be enabled] --- True")]
        public bool Hearth_Hearthstone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Orgrimmar\"/\"Stormwind Portal Stone\"")]
        [Description("Uses \"Orgrimmar\"/\"Stormwind Portal Stone\" --- True")]
        public bool Hearth_PortalStone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Garrison Hearthstone\"")]
        [Description("Uses \"Garrison Hearthstone\" --- True")]
        public bool Hearth_GarrisonHearthstone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Dalaran Hearthstone\"")]
        [Description("Uses \"Dalaran Hearthstone\" --- True")]
        public bool Hearth_DalaranHearthstone { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- HEARTH Settings")]
        [DisplayName("Enable \"Flight Master's Whistle\"")]
        [Description("Uses \"Flight Master's Whistle\" --- True")]
        public bool Hearth_FlightmastersWhistle { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- HEARTH Settings")]
        [DisplayName("Stop Bot Instead of Hearthing")]
        [Description("Stops Bot instead of hearthing --- False")]
        public bool Hearth_PreStop { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- HEARTH Settings")]
        [DisplayName("Stop Bot After Hearthing")]
        [Description("Stops Bot after hearthing --- False")]
        public bool Hearth_PostStop { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- HEARTH Settings")]
        [DisplayName("Disable Hearthing")]
        [Description("Disables Hearthing [NOTE: Training skills for archaeology requires hearthing to be enabled] --- False")]
        public bool Hearth_Disable { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Prevent Disable Addons")]
        [Description("Prevents new questing pack from disabling game addons --- False")]
        public bool Quest_PreventDisableAddons { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Force \"KillBetweenHotspots\" to \"false\"")]
        [Description("Force \"KillBetweenHotspots\" to \"false\" although questing profile sets it to \"true\". This will speed up leveling from 12-58 dramatically if you are using heirlooms with old questing pack --- False")]
        public bool Quest_KillBetweenHotspots { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- ARTIFACT Settings")]
        [DisplayName("Train Artifact Traits")]
        [Description("Trains artifact traits whenever char is in Class Hall [NOTE: only few classes are supported yet] --- True")]
        public bool Artifact_TrainTraits { get; set; }

        [Setting]
        [DefaultValue(TravelForm.Both)]
        [Category("- GENERAL Settings")]
        [DisplayName("TravelForm As Mount")]
        [Description("Set TravelForm as Mount in HB settings as soon as spell is learnt --- Both")]
        public TravelForm General_TravelForm { get; set; }

        [Setting]
        [DefaultValue(RareMobs.Enabled)]
        [Category("- GENERAL Settings")]
        [DisplayName("Attack Rare Mobs")]
        [Description("Attack and loots rare mobs --- Enabled")]
        public RareMobs General_RareMobs { get; set; }

        [Setting]
        [DefaultValue(Treasures.Enabled)]
        [Category("- GENERAL Settings")]
        [DisplayName("Loot Treasures")]
        [Description("Loots treasure chests --- Enabled")]
        public Treasures General_Treasures { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GENERAL Settings")]
        [DisplayName("Speed Looting")]
        [Description("Loots while being in combat --- True")]
        public bool General_SpeedLoot { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- GENERAL Settings")]
        [DisplayName("Disable Flying on Broken Isles")]
        [Description("Prevent HB from flying on Broken Isles --- False")]
        public bool General_DisableLegionFlying { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- GENERAL Settings")]
        [DisplayName("Prevent Refreshing Taxi Nodes")]
        [Description("Prevents refreshing taxi nodes --- False")]
        public bool General_PreventTaxi { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- GENERAL Settings")]
        [DisplayName("Infinite Mode")]
        [Description("Restarts bot after having stopped. Can only be turned off by changing this setting or unticking plugin --- False")]
        public bool General_InfiniteMode { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Get Off Ground")]
        [Description("Prevents hovering with flying mount by trying to get back in air and switches to ground mount instead if those attempts failed (happens in zones when flying skill is already learnt but profiles are coded for running/walking) --- False")]
        public bool Quest_GetOffGround { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- GENERAL Settings")]
        [DisplayName("MountUp Distance")]
        [Description("Forces MountUp when distance to next hotspot is greater than this value (\"0\" to disable) --- 0")]
        public int General_MountUpDistance { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(180)]
        [Category("- GENERAL Settings")]
        [DisplayName("Resurrection Timer")]
        [Description("Forces resurrection at graveyard after this amount of seconds being dead (\"0\" to disable) --- 180")]
        public int General_ResurrectionTimer { get; set; }

        [Setting]
        [DefaultValue(QuestingPack.Old)]
        [Category("- QUEST Settings")]
        [DisplayName("Questing Pack")]
        [Description("Uses new or old questing pack [NOTE: Copy files of new pack to folder \"Questing Profile Pack\"] --- Old")]
        public QuestingPack Quest_QuestingPack { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- QUEST Settings")]
        [DisplayName("Prefer AutoLoader")]
        [Description("Prefers AutoLoader if using default questing profiles [NOTE: Custom loader can be set under \"Custom Loader\"] --- True")]
        public bool Quest_AutoLoader { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- QUEST Settings")]
        [DisplayName("Custom Loader")]
        [Description("Sets custom loader being used whenever profile gets reloaded (\"0\" to disable) --- 0")]
        public string Quest_CustomLoader { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- QUEST Settings")]
        [DisplayName("Abandon Low Level Quests")]
        [Description("Abandons low level quests [NOTE: As KoM will skip unnecessary parts (e.g. if you have heirlooms equipped), HB might not be able to abandon/complete them itself) --- False")]
        public bool Quest_AbandonQuests { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Low Graphic Mode")]
        [Description("Set RenderScale to 0.1, maximum FPS and maximum background FPS to 20 --- False")]
        public bool Performance_LowGraphicMode { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(100)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Default RenderScale")]
        [Description("Sets RenderScale to this value whenever HB stops/pauses (1 - 200) --- 100")]
        public double Performance_DefaultRenderScale { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Default Maximum FPS")]
        [Description("Sets maximum FPS to this value whenever HB stops/pauses (\"0\" removes limiter) --- 0")]
        public double Performance_MaxFps { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- PERFORMANCE Settings")]
        [DisplayName("Default Maximum Background FPS")]
        [Description("Sets maximum background FPS to this value whenever HB stops/pauses (\"0\" removes limiter) --- 0")]
        public double Performance_MaxFpsBk { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(true)]
        [Category("- GENERAL Settings")]
        [DisplayName("Update Sound")]
        [Description("Plays update sound OnEnable when new version is available --- True")]
        public bool General_UpdateSound { get; set; }

        [Setting]
        [DefaultValue(TrainFlying.None)]
        [Category("- QUEST Settings")]
        [DisplayName("Learn \"Artisan\"- and \"Master Riding\"")]
        [Description("Learns \"Artisan\"- (280%) and \"Master Riding\" (310%). [NOTE: Depending on your reputation with OG/SW faction you will need 4-5k gold for each] --- None")]
        public TrainFlying Quest_TrainFlying { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- QUEST Settings")]
        [DisplayName("Set Specialisation")]
        [Description("Changes specialization to defined index (\"0\" to disable) --- 0")]
        public int Quest_Specialisation { get; set; }

        /*
         * private int \u202B;
         *
        public int FollowingDistance
        {
            get
            {
                return this.\u202B‏‬‪‬⁫‮‪⁪⁬‌⁭​⁬⁮‪‏⁭⁯‏‫⁬‮‍‌⁫‏⁫‏⁯‍​⁪‬‭‏‌‌‍‬‮;
            }
            set
            {
                if (value < 5)
                    this.\u202B‏‬‪‬⁫‮‪⁪⁬‌⁭​⁬⁮‪‏⁭⁯‏‫⁬‮‍‌⁫‏⁫‏⁯‍​⁪‬‭‏‌‌‍‬‮ = 5;
                else if (value > 35)
                    this.\u202B‏‬‪‬⁫‮‪⁪⁬‌⁭​⁬⁮‪‏⁭⁯‏‫⁬‮‍‌⁫‏⁫‏⁯‍​⁪‬‭‏‌‌‍‬‮ = 35;
                else
                    this.\u202B‏‬‪‬⁫‮‪⁪⁬‌⁭​⁬⁮‪‏⁭⁯‏‫⁬‮‍‌⁫‏⁫‏⁯‍​⁪‬‭‏‌‌‍‬‮ = value;
            }
        }
        */

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- STOP Settings")]
        [DisplayName("SkillMax")]
        [Description("Stops HB when gather professions are max skilled --- False")]
        public bool Stop_SkillMax { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- STOP Settings")]
        [DisplayName("Log Message")]
        [Description("Stops HB prints this message (\"0\" to disable) [NOTE: Separate multiple messages with char you prefer] --- 0")]
        public string Stop_Message { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(0)]
        [Category("- STOP Settings")]
        [DisplayName("Level")]
        [Description("Stops HB when this level is reached (\"0\" to disable) --- 0")]
        public int Stop_Level { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- STOP Settings")]
        [DisplayName("Item")]
        [Description("Stops HB when item with this ID is received [NOTE: Separate multiple IDs with char you prefer] (\"0\" to disable) --- 0")]
        public string Stop_Item { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- STOP Settings")]
        [DisplayName("Quest")]
        [Description("Stop HBs when quest with this ID is received [NOTE: Separate multiple IDs with char you prefer] (\"0\" to disable) --- 0")]
        public string Stop_Quest { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue("0")]
        [Category("- STOP Settings")]
        [DisplayName("Action")]
        [Description("Starts this HBRelog profile when stopping point is reached (\"0\" to disable) --- 0")]
        public string Stop_Action { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(false)]
        [Category("- STOP Settings")]
        [DisplayName("Stop HBRelog Profile")]
        [Description("Stops HBRelog profile when stopping point is reached --- False")]
        public bool Stop_HBRelog { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(15)]
        [Category("- FPS STABILISER Settings")]
        [DisplayName("FPS Threshold")]
        [Description("If FPS (frames per seconds) drop below this value, TPS (ticks per seconds) decrease to minimum (\"0\" to disable) --- 15")]
        public int FpsStabiliser_FpsThreshold { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(5)]
        [Category("- FPS STABILISER Settings")]
        [DisplayName("Minimum TPS")]
        [Description("If FPS (frames per seconds) drop below threshold, TPS (ticks per seconds) increase to this value --- 5")]
        public int FpsStabiliser_MinTps { get; set; }

        [Setting]
        [Styx.Helpers.DefaultValue(15)]
        [Category("- FPS STABILISER Settings")]
        [DisplayName("Maximum TPS")]
        [Description("If FPS (frames per seconds) are above threshold, TPS (ticks per seconds) slowly increase to to this value --- 15")]
        public int FpsStabiliser_MaxTps { get; set; }
    }
}