﻿using System;
using System.Collections.Generic;
using System.Collections.ObjectModel;
using System.Linq;
using System.IO;
using System.Threading;
using System.Threading.Tasks;
using System.Diagnostics;
using System.Windows.Media;
using System.Media;
using System.Numerics;
using System.Xml;

using Buddy.Coroutines;
using CommonBehaviors.Actions;
using Bots.DungeonBuddy.Enums;
using Bots.DungeonBuddy.Helpers;
using Bots.Grind;
using Bots.ArchaeologyBuddy;

using Styx;
using Styx.TreeSharp;
using Styx.Helpers;
using Styx.Plugins;
using Styx.Pathing;
using Styx.Pathing.Avoidance;
using Styx.Common;
using Styx.Common.Helpers;
using Styx.CommonBot;
using static Styx.CommonBot.BotEvents;
using Styx.CommonBot.CharacterManagement;
using Styx.CommonBot.Coroutines;
using Styx.CommonBot.Frames;
using Styx.CommonBot.Profiles;
using Styx.CommonBot.Profiles.Quest.Order;
using Styx.CommonBot.ObjectDatabase;
using Styx.CommonBot.POI;
using Styx.WoWInternals;
using Styx.WoWInternals.WoWObjects;
using static Styx.WoWInternals.WoWMovement;
using Styx.WoWInternals.World;
using Styx.WoWInternals.DB;
using Styx.WoWInternals.DBC;

using System.Windows.Forms;
using System.Net;
using System.Reflection;
using System.Text.RegularExpressions;
using System.Runtime.CompilerServices;
using System.Security.Cryptography;
using System.Text;

namespace KeepOnMoving
{
    using Settings;

    enum Stuck
    {
        Off = 0,
        On,
        Stage1,
        Stage2,
        Stage3,
        Stage4,
        Stage5,
        Stage6, // RELOAD
        Stage7  // HEARTH
    }

    enum Motionless
    {
        Off = 0,
        On,
        Stage1, // UNSTUCK
        Stage2, // RELOAD
        Stage3  // HEARTH
    }

    enum Hearth
    {
        Off = 0,
        On,
        Capital,
        Pandaria,
        Draenor,
        Dalaran
    }

    enum Dialog
    {
        None = 0,
        Guild,
        Party,
        Duel
    }

    enum Dungeon
    {
        Off = 0,
        InQueue,
        Proposal,
        Succeeded,
        NotPorted,
        InDungeon
    }

    public class KOMWebClient : WebClient
    {
        protected override WebRequest GetWebRequest(Uri _uri)
        {
            var _webRequest = base.GetWebRequest(_uri);
            _webRequest.Timeout = 4 * 1000;
            return _webRequest;
        }
    }

    public static class KOMFlightor
    {
        private static LocalPlayer Me => StyxWoW.Me;

        public static bool CanFly
        {
            get
            {
                var _plugin = PluginManager.Plugins.FirstOrDefault(p => p.Name == "KeepOnMoving" &&
                                                                        p.Enabled);
                var _setting = new KeepOnMoving.KOMCore().Settings.General_DisableLegionFlying;
                var _onBrokenIsles = (Me.CurrentMap.ExpansionId == 6);
                if (_plugin != null && _onBrokenIsles && _setting && Me.IsAlive)
                    return false;

                var _activeMover = WoWMovement.ActiveMover;
                if (_activeMover == null)
                    return false;

                if (_activeMover.MovementInfo.CanFly)
                    return true;

                if (!_activeMover.IsMe)
                    return false;

                if (!Flightor.IsFlyableAreaOrChildOfFlyableArea())
                    return false;

                if (Mount.FlyingMounts.Count() > 0)
                    return true;

                return false;
            }
        }
    }

    static public class KOMApi
    {
        public static string _lastProfile = null;
        public static string _lastBotBaseName = null;

        public static int _availableTraits
        {
            get
            {
                return Lua.GetReturnVal<int>("return ArtifactWatchBar.numPointsAvailableToSpend", 0);
            }
        }

        public static void Log(string _text, params object[] _arguments)
        {
            var _color = Color.FromRgb(28, 117, 240);
            _text = _text
                        .Replace("@", "꙰")
                        .Replace("~", "     ");

            Bots.Professionbuddy.PBLog.Log(_color, "          ⌠ KEEP oи MOVING ⌡ ", Colors.White, _text, _arguments);
        }

        public static void DevLog(string _text, params object[] _arguments)
        {
            var _color = Color.FromRgb(125, 175, 255);
            Logging.WriteDiagnostic(_color, $"          ***** ⌠ KEEP oи MOVING ⌡ {_text}", _arguments);
        }

        public static void ChangeBotBase(string _botBaseName, string _profile = null)
        {
            var _botBases = BotManager.Instance.Bots;
            if (!_botBases.ContainsKey(_botBaseName))
                return;

            var _botBase = _botBases[_botBaseName];

            ThreadPool.QueueUserWorkItem(t =>
            {
                var KOMCore = new KOMCore();
                var KOMSettings = KOMCore.Settings;
                var _preSetting = KOMSettings.General_InfiniteMode;
                KOMSettings.General_InfiniteMode = false;

                while (KOMSettings.General_InfiniteMode)
                    Thread.Sleep(50);

                TreeRoot.Stop("Changing bot base");
                TreeRoot.WaitForStop();

                BotManager.Instance.SetCurrent(_botBase);

                if(_profile != null)
                    ProfileManager.LoadNew(_profile);

                if(_botBaseName == "Dungeonbuddy")
                {
                    KOMCore.SetDungeonBuddySettings();
                    KOMCore.DisablePugWarning();
                    Thread.Sleep(2 * 1000);
                }

                TreeRoot.Start();

                KOMSettings.General_InfiniteMode = _preSetting;
            });
        }

        public static void SwitchBack()
        {
            DevLog("SwitchBack");
            if (_lastProfile == null || _lastBotBaseName == null)
            {
                DevLog("Profile == null: {0}", _lastProfile == null);
                DevLog("BotBaseName == null: {0}", _lastBotBaseName == null);
                return;
            }

            Log("Switching back");
            ChangeBotBase(_lastBotBaseName, _lastProfile);
        }
    }

    public class KOMCore : HBPlugin
    {
        public override string Name => "KeepOnMoving";
        public override string Author => "mh66";
        public override Version Version => new Version(3, 2, 79);
        public override bool WantButton => true;
        public override string ButtonText => "Settings";
        public KOMSettings Settings => KOMSettings.Instance;
        private Form _gui;

        private void Log(string _text, params object[] _arguments)
        {
            KOMApi.Log(_text, _arguments);
        }

        public void DevLog(string _text, params object[] _arguments)
        {
            KOMApi.DevLog(_text, _arguments);
        }

        /*
         *      TEST TWO DIFFERENT CODE RUNTIMES
         *
            ClearLog();
            var _d = DateTime.Now;
            var _runs = 1000;

            for(int i=1; i <= _runs; i++)
            {
	            // FIRST CODE
            }
            var _firstRuntime = DateTime.Now.Subtract(_d).TotalMilliseconds / _runs;

            _d = DateTime.Now;
            for(int i=1; i <= _runs; i++)
            {
	            // SECOND CODE
            }
            var _secondRuntime = DateTime.Now.Subtract(_d).TotalMilliseconds / _runs;

            Log("1 : {0} ({1}% of 2) \n2 : {2} ({3}% of 1)",
		            _firstRuntime,
		            Math.Round(_firstRuntime / _secondRuntime * 100),
		            _secondRuntime,
		            Math.Round(_secondRuntime / _firstRuntime * 100));
        */

        private void RuntimeLog(string _option = null, [CallerLineNumber] int _lineNumber = 0)
        {
            if (!Settings.Log_Runtime && _spawnNpc == null)
                return;

            var _colorGrey = Color.FromRgb(100, 100, 100);
            var _colorGreen = Color.FromRgb(0, 255, 100);

            if (_option == "Start")
            {
                Logging.WriteDiagnostic(_colorGrey, "~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~");
                _timePerformanceRunSum = DateTime.Now;
            }
            else if (_option == "Sum")
            {
                var _totalMs = Math.Round(DeltaTime(_timePerformanceRunSum, "ms"));
                var _tps = TreeRoot.TicksPerSecond;
                var _msPerTick = Math.Round(1000.0 / TreeRoot.TicksPerSecond);
                var _deltaMs = Math.Round(_msPerTick - _totalMs);

                if (_deltaMs < 0 && Settings.FpsStabiliser_FpsThreshold != 0)
                {
                    DevLog("DeltaMS < 0: {0}", _deltaMs);
                    TreeRoot.TicksPerSecond = (byte)Settings.FpsStabiliser_MinTps;
                }

                Logging.WriteDiagnostic(_colorGrey, "                          {0} ({1})", _totalMs,
                                                                                           (_deltaMs > 0)
                                                                                                ? $"+ {_deltaMs}"
                                                                                                : $"{_deltaMs}");
                Logging.WriteDiagnostic(_colorGrey, "                              {0} TPS", _tps);
                Logging.WriteDiagnostic(_colorGrey, "                              {0} FPS", StyxWoW.WoWClient.Fps);
                Logging.WriteDiagnostic(_colorGrey, "                              {0} ms", _msPerTick);
            }
            else
            {
                if (!_lineRuntimeSum.ContainsKey(_lineNumber))
                    _lineRuntimeSum.Add(_lineNumber, DeltaTime(_timePerformanceRun, "ms"));
                else
                {
                    if (_option == "##")
                        _lineRuntimeSum[_lineNumber] += Math.Round(DeltaTime(_timePerformanceRun, "ms") / TreeRoot.TicksPerSecond);
                    else
                        _lineRuntimeSum[_lineNumber] += DeltaTime(_timePerformanceRun, "ms");
                }

                var _sortedDictionary = from _entry in _lineRuntimeSum
                                        orderby _entry.Value ascending
                                        select _entry;

                var _index = _sortedDictionary.ToList().IndexOf(new KeyValuePair<int, double>(_lineNumber, _lineRuntimeSum[_lineNumber])) + 1;
                float _count = _sortedDictionary.Count();

                var _highlightedEntries = 6;
                var _factor = _index / _count;
                _factor = (_index < _count - _highlightedEntries - 1)
                                ? Math.Max(0, _factor - 0.4f)
                                : _factor;

                var _newR = _colorGrey.R + (_colorGreen.R - _colorGrey.R) * _factor;
                var _newG = _colorGrey.G + (_colorGreen.G - _colorGrey.G) * _factor;
                var _newB = _colorGrey.B + (_colorGreen.B - _colorGrey.B) * _factor;

                var _colorPercentage = Color.FromRgb(Convert.ToByte(_newR),
                                                     Convert.ToByte(_newG),
                                                     Convert.ToByte(_newB));

                var _pre = (_option == null)
                                ? null
                                : _option.Replace(">", "   ")
                                         .Replace("#", null);

                Logging.WriteDiagnostic(_colorPercentage, _pre + "{0}: [{1}] {2}", _lineNumber,
                                                                                   Math.Round(_lineRuntimeSum[_lineNumber]),
                                                                                   DeltaTime(_timePerformanceRun, "ms"));
            }
            _timePerformanceRun = DateTime.Now;
        }

        private static LocalPlayer Me => StyxWoW.Me;

        private Vector3 _lastLocation = Vector3.Zero;
        private Vector3 _lastStuckLocation = Vector3.Zero;
        private double _stuckCount = 0;
        private double _lastStuckCount = 0;
        private double _intervalLastFlightorLog = 0;
        private int _stableStuckCount = 0;
        private int _disableAddonsSetting = -1;
        private WoWGuid _lastRareGuid = WoWGuid.Empty;
        private string _lastRareName = null;
        private uint _lastRareId = 0;

        private string _lastFrameText = null;
        private string _lastObjective = null;
        private string _lastLogEntry = null;
        private string _mountErrorMessage = Lua.GetReturnVal<string>("return SPELL_FAILED_NO_MOUNTS_ALLOWED", 0);
        private string _startsQuestText = Lua.GetReturnVal<string>("return ITEM_STARTS_QUEST", 0);
        private Dialog DialogState = Dialog.None;
        private Stuck StuckState = Stuck.Off;
        private Hearth HearthState = Hearth.Off;
        private Motionless MotionlessState = Motionless.Off;
        private Dungeon DungeonState = Dungeon.Off;

        private DateTime _timeTpsChanged = DateTime.MinValue;
        private DateTime _timePreMotionless = DateTime.Now;
        private DateTime _timeDied = DateTime.MinValue;
        private DateTime _timeUnghost = DateTime.MinValue;
        private DateTime _timeGarrisonCacheInteracted = DateTime.MinValue;
        private DateTime _timeRun = DateTime.MinValue;
        private DateTime _timeLastJump = DateTime.MinValue;
        private DateTime _timePerformanceRun = DateTime.MinValue;
        private DateTime _timePerformanceRunSum = DateTime.MinValue;
        private DateTime _timeFrameChange = DateTime.MinValue;
        private DateTime _timeStartFlyingMountRun = DateTime.MinValue;
        private DateTime _timeAtTransportBugLocation = DateTime.MinValue;
        private DateTime _timeMovingInto = DateTime.MinValue;
        private DateTime _timeMountError = DateTime.MinValue;
        private DateTime _timeFlightorFailed = DateTime.MinValue;
        private DateTime _timeLastLocation = DateTime.MinValue;
        private DateTime _timeLastTurn = DateTime.MinValue;
        private DateTime _timeLastMoved = DateTime.MinValue;
        private DateTime _timeLastObjectiveChange = DateTime.MinValue;
        private DateTime _timeReload = DateTime.MinValue;
        private DateTime _timeTransportDismount = DateTime.MinValue;
        private DateTime _timeInterrupted = DateTime.MinValue;
        private DateTime _timeSucceeded = DateTime.MinValue;
        private DateTime _timeMountUp = DateTime.MinValue;
        private DateTime _timeDialogOpened = DateTime.MinValue;
        private DateTime _timeDialogClosed = DateTime.MinValue;
        private DateTime _timePathDistancesRequestCount = DateTime.MinValue;
        private DateTime _timeEnterVehicle = DateTime.MinValue;
        private DateTime _timeExitVehicle = DateTime.MinValue;
        private DateTime _timeLastCTM = DateTime.MinValue;
        private DateTime _timeMapChanged = DateTime.MinValue;
        private DateTime _timeLastFlightorLog = DateTime.MinValue;
        private DateTime _timeGettingOff = DateTime.MinValue;
        private DateTime _timePreventStuck = DateTime.MinValue;
        private DateTime _timeStartUpdateQuests = DateTime.MinValue;
        private DateTime _timeExceptionThrown = DateTime.MinValue;
        private DateTime _timeStartRareAttack = DateTime.MinValue;
        private DateTime _timeProfileLoaded = DateTime.MinValue;
        private DateTime _timeCheckedItems = DateTime.MinValue;
        private DateTime _timeAutoEquip = DateTime.MinValue;

        private double _deltaNextJump = 0;
        private double _failedReviveCount = 0;
        private double _failedMountUpCount = 0;
        private double _failedPathDistancesRequestCount = 0;
        private double _failedPreventStuckCount = 0;

        private double _accountExpansionLevel = Lua.GetReturnVal<int>("return GetAccountExpansionLevel()", 0);
        private double _renderScale = Lua.GetReturnVal<double>("return GetCVar('RenderScale')", 0);
        private double _maxFps = Lua.GetReturnVal<double>("return GetCVar('MaxFPS')", 0);
        private double _maxFpsBk = Lua.GetReturnVal<double>("return GetCVar('maxFPSBk')", 0);
        private double _autoLoot = Lua.GetReturnVal<double>("return GetCVar('AutoLootDefault')", 0);
        private double _spec = Lua.GetReturnVal<int>("return GetSpecialization()", 0);
        private bool _declineGuildInvite = Lua.GetReturnVal<bool>("return GetAutoDeclineGuildInvites()", 0);
        private bool _init = false;

        private bool _checkedRelics = false;
        private bool _checkedItems = false;
        private bool _checkedQuests = false;
        private bool _checkedMails = false;
        private bool _checkedTraits = false;
        private bool _checkedHeirlooms = false;
        private bool _checkedGatheringSkills = false;
        private bool _checkedArchaeologySkill = false;
        private bool _meleeDismount = false;
        private bool _reloadProfile = false;
        private bool _stopMessage = false;

        private SkillLine _lowestNodeSkill = 0;
        private List<uint> _completedQuests = new List<uint>(QuestLog.Instance.GetCompletedQuests());
        private List<PlayerQuest> _allQuests = QuestLog.Instance.GetAllQuests();
        private Dictionary<WoWGuid, double> _ignoredNodeGuids = new Dictionary<WoWGuid, double>();
        private Dictionary<WoWGuid, DateTime> _ignoredGeneralGuids = new Dictionary<WoWGuid, DateTime>();
        private Dictionary<int, DateTime> _ignoredQuests = new Dictionary<int, DateTime>();
        private Dictionary<int, double> _lineRuntimeSum = new Dictionary<int, double>();
        private Dictionary<int, bool> _toys = new Dictionary<int, bool>();
        private Dictionary<string, int> _pets = new Dictionary<string, int>();
        private List<object[]> _stuckLocations = new List<object[]>();
        private List<object[]> _trainerData = new List<object[]>();
        private List<BagItem> _bagItems = new List<BagItem>();

        private Random _rnd = new Random();
        private Composite _KOM;
        private WaitTimer _infoTimer = new WaitTimer(TimeSpan.FromSeconds(5));
        private WaitTimer _logTimer = new WaitTimer(TimeSpan.FromSeconds(10));
        private WaitTimer _taxiWarningTimer = new WaitTimer(TimeSpan.FromSeconds(10));
        private WaitTimer _nodeRefreshTimer = new WaitTimer(TimeSpan.FromMilliseconds(1000));

        private readonly SoundPlayer _soundPlayer = new SoundPlayer();

        public override void OnButtonPress()
        {
            if (_gui == null || _gui.IsDisposed || _gui.Disposing)
                _gui = new GUI();
            if (_gui != null || _gui.IsDisposed)
                _gui.ShowDialog();
        }

        private async Task<bool> Sleep(int _maxSleep = 1000, int _minSleep = 250)
        {
            await Coroutine.Sleep(_rnd.Next(_minSleep, _maxSleep));
            return true;
        }

        private async Task<bool> ClickButton(string _buttonName, int _sleep = 1500)
        {
            await Sleep(_sleep);
            if (IsFrameVisible($"{_buttonName}"))
                Lua.DoString($"{_buttonName}:Click()");

            return true;
        }

        private List<string> _defaultFrames = new List<string>
        {
            "QuestFrame",
            "CinematicFrame",
            "GossipFrame",
            "TaxiFrame",
            "MailFrame",
            "LootFrame",
            "TrainerFrame",
            "FlightMapFrame"
        };

        private bool IsFrameVisible(string _frameName, bool _ignoreTimeLimit = false)
        {
            if (_defaultFrames.Contains(_frameName) &&
                !_ignoreTimeLimit &&
                DeltaTime(_timeDialogOpened, "s") > 10)
                return false;

            switch (_frameName)
            {
                case "CinematicFrame":
                    return StyxWoW.WoWClient.IsCinematicPlaying;
                    break;
                case "QuestFrame":
                    return QuestFrame.Instance != null && QuestFrame.Instance.IsVisible;
                    break;
                case "GossipFrame":
                    return GossipFrame.Instance != null && GossipFrame.Instance.IsVisible;
                    break;
                case "MailFrame":
                    return MailFrame.Instance != null && MailFrame.Instance.IsVisible;
                    break;
                case "LootFrame":
                    return LootFrame.Instance != null && LootFrame.Instance.IsVisible;
                    break;
                case "TrainerFrame":
                    return TrainerFrame.Instance != null && TrainerFrame.Instance.IsVisible;
                    break;
                case "TaxiFrame":
                    return TaxiFrame.Instance != null && TaxiFrame.Instance.IsVisible;
                    break;
                case "FlightMapFrame":
                    return FlightMapFrame.Instance != null && FlightMapFrame.Instance.IsVisible;
                    break;
                default:
                    return new Frame(_frameName).IsVisible;
                    break;
            }
        }

        /*
        Assembly _assembly = typeof(Styx.CommonBot.Frames.Frame).Assembly;
        Type _type = _assembly.GetType("Styx.CommonBot.Frames." + _frameName);

        if(_type != null)
        {
            dynamic _obj = Activator.CreateInstance(_type);
            return _obj != null && _obj.IsVisible;
        */

        private string _mapPosition
        {
            get
            {
                var _mapCoordinates = Lua.GetReturnValues("return GetPlayerMapPosition('player')");
                if (_mapCoordinates.Count == 0)
                {
                    DevLog("IsInInstance: {0}", Me.IsInInstance);
                    return " -/- ";
                }

                var _x = Math.Round(Convert.ToDouble(_mapCoordinates[0]) * 100f, 1);
                var _y = Math.Round(Convert.ToDouble(_mapCoordinates[1]) * 100f, 1);

                return $"{_x}, {_y}";
            }
        }

        private double DeltaTime(DateTime _time, string _unit)
        {
            var _delta = DateTime.Now.Subtract(_time);

            switch (_unit)
            {
                case "ms":
                    return _delta.TotalMilliseconds;
                    break;
                case "s":
                    return _delta.TotalSeconds;
                    break;
                case "m":
                    return _delta.TotalMinutes;
                    break;
                case "h":
                    return _delta.TotalHours;
                    break;
            }

            return 0;
        }

        private WoWGameObject _deathGate
        {
            get
            {
                return ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                            .FirstOrDefault(o => o.Entry == 190942 &&
                                                 o.IsValid &&
                                                 o.CreatedByGuid == Me.Guid);
            }
        }

        private WoWGameObject _garrisonCache
        {
            get
            {
                return ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                            .FirstOrDefault(o => ((Me.IsHorde)
                                                        ? o.Entry == 237191 || o.Entry == 237724
                                                        : o.Entry == 236916 || o.Entry == 237722) &&
                                                 o.IsValid);
            }
        }

        private void Reset(bool _fullReset = false)
        {
            DevLog("Reset{0}", (_fullReset)
                                    ? " > FullReset"
                                    : "");

            if (_fullReset)
            {
                _init = false;

                _ignoredNodeGuids.Clear();
                _ignoredGeneralGuids.Clear();
                _stuckLocations.Clear();

                _timeDied = DateTime.MinValue;

                _checkedHeirlooms = false;
                _checkedTraits = false;

                if (StyxWoW.IsInWorld)
                {
                    _renderScale = Lua.GetReturnVal<double>("return GetCVar('RenderScale')", 0);
                    _maxFps = Lua.GetReturnVal<double>("return GetCVar('MaxFPS')", 0);
                    _maxFpsBk = Lua.GetReturnVal<double>("return GetCVar('maxFPSBk')", 0);
                    _spec = GetSpecIndex();
                }

                UpdateQuests();
            }

            HearthState = Hearth.Off;
            StuckState = Stuck.Off;
            MotionlessState = Motionless.Off;

            DialogState = Dialog.None;

            _checkedItems = false;
            _checkedQuests = false;
            _checkedMails = false;
            _checkedRelics = false;
            _checkedGatheringSkills = false;
            _stopMessage = false;

            _stuckCount = 0;
            _lastStuckCount = 0;
            _stableStuckCount = 0;

            _failedMountUpCount = 0;
            _failedPathDistancesRequestCount = 0;

            _timeAtTransportBugLocation = DateTime.MinValue;
            _timeFlightorFailed = DateTime.MinValue;
            _timeLastMoved = DateTime.MinValue;
            _timeLastObjectiveChange = DateTime.MinValue;
            _timeDialogOpened = DateTime.MinValue;
            _timeDialogClosed = DateTime.MinValue;
        }

        private bool breakOut
        {
            get
            {
                return  !Me.IsAlive ||
                        !_init ||
                        (
                            Me.Combat &&
                            Me.HealthPercent <= 75 &&
                            !Me.Mounted
                        ) ||
                        DungeonState == Dungeon.Proposal ||
                        _spawnNpc != null ||
                        IsFrameVisible("CinematicFrame", true);
            }
        }

        private WoWUnit GetNpc(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                var _unit = ObjectManager.GetObjectsOfType<WoWUnit>()
                                .Where(u => u.IsValid &&
                                            u.Entry == _id &&
                                            u.IsAlive)
                                .OrderBy(u => u.Distance)
                                .FirstOrDefault();

                return _unit;
            }

            return null;
        }

        private WoWGameObject GetObject(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                var _object = ObjectManager.GetObjectsOfType<WoWGameObject>()
                                    .Where(o => o.IsValid &&
                                                o.Entry == _id &&
                                                o.CanUseNow())
                                    .OrderBy(o => o.Distance)
                                    .FirstOrDefault();

                return _object;
            }

            return null;
        }

        private WoWItem GetItem(int _id)
        {
            var _item = Me.BagItems
                            .Where(i => i.IsValid &&
                                        i.Entry == _id)
                            .OrderBy(i => i.StackCount)
                            .FirstOrDefault();

            return _item;
        }

        private int GetItemCount(int _id, List<WoWItem> _list = null)
        {
            _list = _list ?? Me.BagItems;

            return (int)_list.Where(i => i.Entry == _id &&
                                         i.IsValid)
                             .Sum(i => i.StackCount);
        }

        private int _garrisonResourcesCount
        {
            get
            {
                return (int)WoWCurrency.GetCurrencyByType(WoWCurrencyType.GarrisonResources).Amount;
            }
        }

        private int _gatheringItemCount
        {
            get
            {
                return (int)Me.BagItems
                    .Where(i => i.IsValid &&
                                (i.ItemInfo.TradeGoodsClass == WoWItemTradeGoodsClass.MetalStone ||
                                 i.ItemInfo.TradeGoodsClass == WoWItemTradeGoodsClass.Herb))
                    .Sum(i => i.StackCount);
            }
        }

        private WoWGameObject _mailbox
        {
            get
            {
                IEnumerable<WoWGameObject> _objects =
                    (from o in ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                     where
                        o.SubType == WoWGameObjectType.Mailbox &&
                        o.IsValid &&
                        o.CanUse()
                     orderby
                        o.Distance
                     select o);

                return (_objects.Count() > 0)
                            ? _objects.First()
                            : null;
                /*
                var _factionId = (_first != null)
                                        ? _first.FactionTemplate.Faction.Id
                                        : 0;
                var _moreThanNeutral = (_factionId != 0)
                                            ? Me.GetReputationLevelWith(_factionId) >= WoWUnitReaction.Friendly
                                            : false;
                var _isMyFaction = (Me.IsHorde)
                                        ? _factionId == 66 || _moreThanNeutral
                                        : _factionId == 189 || _moreThanNeutral;
                return (_first != null && _isMyFaction)
                            ? _first
                            : null;
                */
            }
        }

        private MailFrame.InboxMailItem _newMail
        {
            get
            {
                IEnumerable<MailFrame.InboxMailItem> _mails =
                    (from m in MailFrame.Instance.GetAllMails()
                     where
                        (!m.WasRead ||
                        m.Copper > 0 ||
                        m.ItemCount > 0) &&
                        m.ItemCount < Me.FreeBagSlots
                     select m);

                return (_mails.Count() > 0)
                                ? _mails.First()
                                : null;
            }
        }

        private WoWUnit _lootableMob
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                        u.CanLoot &&
                        u.WithinInteractRange &&
                        u.Lootable &&
                        !u.Looting &&
                        u.IsValid &&
                        (
                            !_ignoredGeneralGuids.ContainsKey(u.Guid) ||
                            DeltaTime(_ignoredGeneralGuids[u.Guid], "m") >= 60
                        )
                     orderby
                        u.Distance
                     select u);

                return (_units.Count() > 0)
                                ? _units.First()
                                : null;
            }
        }

        private bool HasCorrectSkinType(WoWUnit _unit)
        {
            var _skinType = _unit.SkinType;
            var _harvestMobs = (int)Settings.GatheringProfessions_HarvestMobs;
            var _professions = Settings.GatheringProfessions_Professions.ToString();

            if(_skinType == WoWCreatureSkinType.Herb)
            {
                if (_harvestMobs != 1)
                    return true;
            }
            else if(_skinType == WoWCreatureSkinType.Rock)
            {
                if (_harvestMobs != 2)
                    return true;
            }

            return false;
        }

        private WoWUnit _interactableMob
        {
            get
            {
                var _unit = ObjectManager.GetObjectsOfType<WoWUnit>()
                                    .Where( u =>    u.IsValid &&
                                                    !u.CanLoot &&
                                                    u.CanSkin &&
                                                    HasCorrectSkinType(u) &&
                                                    u.CanInteractNow &&
                                                    !u.Looting &&
                                                    !u.Lootable &&
                                                    u.Distance <= 60 &&
                                                    (Me.IsOutdoors == u.IsOutdoors) &&
                                                    (
                                                        !_ignoredGeneralGuids.ContainsKey(u.Guid) ||
                                                        DeltaTime(_ignoredGeneralGuids[u.Guid], "m") >= 60
                                                    ))
                                    .OrderBy(u => u.Distance)
                                    .FirstOrDefault();
                return _unit;
            }
        }

        private float GetPathDistance(WoWObject _object)
        {
            int i = 1;
            float _pathDistance = 0;
            while (i <= 5)
            {
                _pathDistance = Navigator.LookupPathInfo(_object).Distance;
                if (_pathDistance != 0)
                    return _pathDistance;
                i++;
            }

            return 0;
        }

        private List<uint> _brimstoneObjectIds = new List<uint>()
        {
            247965,
            247956,
            247970,
            247959,
            247964,
            247967,
            247961,
            247962,
            247969,
            247958,
            247957,
            247963,
            247966
        };

        private List<uint> _felwortObjectIds = new List<uint>()
        {
            252404,
            247999,
            248002,
            248011,
            248007,
            248008,
            248003,
            248001,
            244786,
            248005,
            248004,
            248010,
            248000,
            248009,
            248006,
            248012
        };

        /* profession WQ
         *
        "Felwort",              41225, new Vector3()
        "Felwort",              41511, new Vector3()
        "Felwort",              41512, new Vector3(3912.737f, 5348.055f, 623.2257f)
        "Felwort",              41513, new Vector3()
        "Felwort",              41514, new Vector3(535.6406f, 5325.36f, 47.72783f)
        "Felwort",              41515, new Vector3()
        "Felwort",              41516, new Vector3()
        "Felwort",              41517, new Vector3()
        "Felwort",              41518, new Vector3()
        "Felwort",              41519, new Vector3()
        "Felwort",              41520, new Vector3()
        "Felwort",              41521, new Vector3()
        "Felwort",              41521, new Vector3()
        "Felwort",              41522, new Vector3()
        "Felwort",              41523, new Vector3()
        "Felwort",              41524, new Vector3()

        "Infernal Brimstone",   41208, new Vector3()
        "Infernal Brimstone",   41209, new Vector3()
        "Infernal Brimstone",   41210, new Vector3(3728.291f, 3772.924f, 946.6287f)
        "Infernal Brimstone",   41481, new Vector3()
        "Infernal Brimstone",   41482, new Vector3(487.0125f, 5609.89f, 66.89051f)
        "Infernal Brimstone",   41483, new Vector3()
        "Infernal Brimstone",   41484, new Vector3()
        "Infernal Brimstone",   41486, new Vector3()
        "Infernal Brimstone",   41487, new Vector3()
        "Infernal Brimstone",   41488, new Vector3()
        "Infernal Brimstone",   41489, new Vector3()
        "Infernal Brimstone",   41490, new Vector3()
        "Infernal Brimstone",   41491, new Vector3()
        "Infernal Brimstone",   41492, new Vector3()

         */


        private WoWGameObject _node
        {
            get
            {
                RuntimeLog(">>>");

                _lowestNodeSkill = (_lowestNodeSkill == 0)
                                        ? GetLowestNodeSkill()
                                        : _lowestNodeSkill;

                if (_lowestNodeSkill == 0)
                    return null;

                RuntimeLog(">>>");

                var _nodes = ObjectManager.GetObjectsOfTypeFast<WoWGameObject>()
                                .Where(o => o.CanLoot &&
                                            !BlackspotManager.IsBlackspotted(o.Location) &&
                                            (
                                                _brimstoneObjectIds.Contains(o.Entry) ||
                                                _felwortObjectIds.Contains(o.Entry) ||
                                                (
                                                    ( o.IsMineral || o.IsHerb ) &&
                                                    o.SubType == WoWGameObjectType.GatheringNode &&
                                                    (
                                                        !_ignoredNodeGuids.ContainsKey(o.Guid) ||
                                                        _ignoredNodeGuids[o.Guid] > 0
                                                    )
                                                )
                                            ) );

                RuntimeLog(">>>");

                if (_nodes.Count() == 0)
                    return null;

                /*
                foreach (var _node in _nodes)
                {
                    if (!IsNodeValuable(_node))
                        _ignoredNodeGuids[_node.Guid] = 0;
                }
                */

                RuntimeLog(">>>");

                _nodes = _nodes.Where(o => IsNodeValuable(o));
                if (_nodes.Count() == 0)
                    return null;

                RuntimeLog(">>>");

                _nodes = (Me.IsFlying || (Flightor.CanFly && Me.IsOutdoors))
                                ? _nodes.OrderBy(o => o.Distance)
                                : _nodes.Where(o => GetPathDistance(o) > 0 &&
                                                    GetPathDistance(o) < 150)
                                        .OrderBy(o => GetPathDistance(o));

                RuntimeLog(">>>");

                if (_nodes.Count() == 0)
                    return null;

                return GetPreferredNode(_nodes.ToList());
            }
        }

        private WoWGameObject GetPreferredNode(List<WoWGameObject> _nodes)
        {
            var _felwortObject = _nodes.FirstOrDefault(o => _felwortObjectIds.Contains(o.Entry));
            if (_felwortObject != null)
                return _felwortObject;

            var _brimstoneObject = _nodes.FirstOrDefault(o => _brimstoneObjectIds.Contains(o.Entry));
            if (_brimstoneObject != null)
                return _brimstoneObject;

            var _setting = (int)Settings.GatheringProfessions_PreferNode;
            if (_setting == 0)
                return _nodes.FirstOrDefault();

            var _herbs = _nodes.Where(o => o.IsHerb);
            var _minerals = _nodes.Where(o => o.IsMineral);
            if (_herbs.Count() == 0 || _minerals.Count() == 0)
                return _nodes.FirstOrDefault();

            WoWGameObject _tempNode = null;
            if(_setting >= 3)
            {
                _tempNode = (_setting == 3)
                                    ? _herbs.FirstOrDefault()
                                    : _minerals.FirstOrDefault();

                return (_tempNode == null)
                            ? _nodes.FirstOrDefault()
                            : _tempNode;
            }

            if (_setting == 2 && Me.CurrentMap.ExpansionId >= 5)
                return _nodes.FirstOrDefault();

            _tempNode = (_lowestNodeSkill == SkillLine.Herbalism)
                                ? _herbs.FirstOrDefault()
                                : _minerals.FirstOrDefault();
            return (_tempNode == null)
                        ? _nodes.FirstOrDefault()
                        : _tempNode;
        }

        private async Task<bool> DruidForm()
        {
            RuntimeLog(">>>");

            if (Me.Class != WoWClass.Druid)
                return false;

            RuntimeLog(">>>");

            if (Flightor.CanFly && _preventFlying)
                return false;

            RuntimeLog(">>>");

            if (!Mount.CanMount() && (Me.Shapeshift == ShapeshiftForm.Cat ||
                                      !SpellManager.HasSpell(768)))
                return false;

            RuntimeLog(">>>");

            if (Me.Shapeshift == ShapeshiftForm.Aqua)
                return true;

            if (Mount.CanMount() && (Me.Shapeshift == ShapeshiftForm.Travel ||
                                     !SpellManager.HasSpell(783)))
                return false;

            RuntimeLog(">>>");

            var _flyingMountSpellId = CharacterSettings.Instance.FlyingMountSpellId;
            if (Mount.CanMount() && Flightor.CanFly && _flyingMountSpellId != 783)
                return false;

            RuntimeLog(">>>");

            var _groundMountSpellId = CharacterSettings.Instance.GroundMountSpellId;
            if (Mount.CanMount() && !Flightor.CanFly && _groundMountSpellId != 783)
                return false;

            RuntimeLog(">>>");

            if (Settings.General_DisableLegionFlying && Me.CurrentMap.ExpansionId == 6)
                return false;

            // await CommonCoroutines.StopMoving();
            if (Mount.CanMount())
            {
                DevLog("DruidForm > TravelForm");
                await Coroutine.Wait(5000, () => SpellManager.CanCast(783));
                SpellManager.CastSpellById(783);
                await Coroutine.Wait(2000, () => Me.HasAura(783));

                if (Flightor.CanFly)
                    await Move("JF", 250, true);

                return true;
            }
            else
            {
                DevLog("DruidForm > CatForm");
                await Coroutine.Wait(5000, () => SpellManager.CanCast(768));
                SpellManager.CastSpellById(768);
                await Coroutine.Wait(2000, () => Me.Shapeshift == ShapeshiftForm.Cat);

                return true;
            }
        }

        private bool _preventFlying
        {
            get
            {
                return DeltaTime(_timeFlightorFailed, "s") <= 30;
            }
        }

        private bool _preventMount
        {
            get
            {
                return DeltaTime(_timeMovingInto, "s") < 5 ||
                       DeltaTime(_timeMountError, "s") < 10 ||
                       (_failedMountUpCount >= 2 && DeltaTime(_timeMountUp, "ms") < 6000) ||
                       (
                            DeltaTime(_timeTransportDismount, "s") < 30 &&
                            !Me.IsOnTransport
                       );
            }
        }

        private async Task<bool> MountUp(Nullable<Vector3> _location = null, double _distanceTreshold = 20, bool _forceGroundMount = false)
        {
            RuntimeLog(">>");

            if (_preventMount)
                return false;

            RuntimeLog(">>");

            if (Me.Mounted && !Mount.Current.IsFlyingMount && _forceGroundMount)
                return false;

            if (Me.Mounted && !Flightor.CanFly && !Mount.Current.IsFlyingMount)
                return false;

            RuntimeLog(">>");

            if (Me.Mounted && Flightor.CanFly && Mount.Current.IsFlyingMount)
            {
                if (_forceGroundMount)
                    await CommonCoroutines.Dismount(null, false);
                return false;
            }

            RuntimeLog(">>");

            if (Me.Class == WoWClass.Druid && !Mount.CanMount() &&
                (
                    Me.Shapeshift == ShapeshiftForm.Cat ||
                    !SpellManager.CanCast(768)
                ))
                return false;

            RuntimeLog(">>");

            if (Me.Shapeshift == ShapeshiftForm.Aqua)
                return true;

            if (Me.Class == WoWClass.Druid && Me.Shapeshift == ShapeshiftForm.Travel)
                return false;

            RuntimeLog(">>");

            if (Me.Class != WoWClass.Druid && (Me.Combat || !Mount.CanMount()))
                return false;

            RuntimeLog(">>");

            if (_location != null && ((Vector3)_location).Distance(Me.Location) < _distanceTreshold)
                return false;

            RuntimeLog(">>");

            if (!await DruidForm() || _forceGroundMount)
            {
                if (Flightor.CanFly && !_preventFlying && !_forceGroundMount)
                {
                    DevLog("MountUp > FlyingMount");
                    while ((!Me.Mounted || !Mount.Current.IsFlyingMount) && !breakOut)
                    {
                        if (_preventMount)
                        {
                            DevLog("PreventMount: {0}", _preventMount);
                            return false;
                        }

                        DevLog("WHILE: MountUp > FlyingMount");
                        await CommonCoroutines.StopMoving();
                        await CommonCoroutines.SummonFlyingMount();
                        await Coroutine.Wait(2000, () => Me.Mounted || _preventMount);
                        await Coroutine.Yield();
                    }
                    await Move("J", 200);

                    return true;
                }
                else
                {
                    DevLog("MountUp > GroundMount");
                    while (!Me.Mounted && Mount.CanMount() && !breakOut)
                    {
                        if (_preventMount)
                        {
                            DevLog("PreventMount: {0}", _preventMount);
                            return false;
                        }

                        DevLog("WHILE: MountUp > GroundMount");
                        await CommonCoroutines.StopMoving();
                        await CommonCoroutines.SummonGroundMount();
                        await Coroutine.Wait(2000, () => Me.Mounted || _preventMount);
                        await Coroutine.Yield();
                    }

                    return true;
                }
            }

            _stuckCount = 0;
            return true;
        }

        private List<uint> _rankQuestItemsIds = new List<uint>()
        {
            // FJARNSKAGGL - RANK 2
	            138111, // HOOK
                129201,
                129200,
                129202,
            // BRIMSTONE - RANK 2
	            128392,
                128396,
                128405,
            // STARLIGHT ROSE - RANK 1
                129121,
            // AETHRIL - RANK 2
	            129153,
            // FELWORT - RANK 2
	            129212,
                129213,
                129214,
            // LEYSTONE - RANK 2
	            128767,
                128769,
            // FELSLATE - RANK 2
	            126940
        };

        private bool IsLowLevelQuestItem(WoWItem _item)
        {
            if (_item == null)
                return false;

            if (!_item.IsValid)
                return false;

            var _isRankQuestItem = _rankQuestItemsIds.Contains(_item.Entry);
            if (_isRankQuestItem)
                return false;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null)
                return false;

            var _questId = _itemInfo.BeginQuestId;
            if (_questId != 0)
            {
                var _quest = Quest.FromId((uint)_questId);
                if (_quest != null &&
                    _quest.Level > 0 &&
                    _quest.Level <= Me.Level - 10)
                    return true;
            }

            var _itemClass = _itemInfo.ItemClass;
            if (_itemClass == null)
                return false;

            if (_itemClass != WoWItemClass.Quest)
                return false;

            if (_itemInfo.Level == 1)
                return false;

            if (_itemInfo.Level > Me.Level - 10)
                return false;

            /*

            if (_itemClass != WoWItemClass.Quest &&
                _itemClass != WoWItemClass.Miscellaneous)
                return false;

            if (_itemClass == WoWItemClass.Miscellaneous &&
                !StartsQuest(_item))
                return false;
            */

            return true;
        }

        private bool IncreasesAncientMana(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _effects = _item.Effects;
            if (_effects == null || _effects.Count() == 0)
                return false;

            var _increasesAncientMana = _effects.Any(e => e != null &&
                                                          e.Spell != null &&
                                                          e.Spell.SpellEffects.Count() > 0 &&
                                                          e.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.GiveCurrency &&
                                                                                         se.MiscValueA == _ancientManaCurrencyId));

            if (_increasesAncientMana)
                return true;

            return false;
        }

        private bool IncreasesArtifactPower(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _effects = _item.Effects;
            if (_effects == null || _effects.Count() == 0)
                return false;

            var _increasesArtifactPower = _effects.Any(e => e != null &&
                                                            e.Spell != null &&
                                                            e.Spell.SpellEffects.Count() > 0 &&
                                                            e.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.GiveArtifactPower));
            if (_increasesArtifactPower)
                return true;

            return false;
        }

        private int _ancientManaCurrencyId
        {
            get
            {
                return 1155;
            }
        }

        private int _ancientMana
        {
            get
            {
                var _ancientManaCurrency = WoWCurrency.GetCurrencyById((uint)_ancientManaCurrencyId);
                return (int)_ancientManaCurrency.Amount;
            }
        }

        private int _artifactKnowledgeLevel
        {
            get
            {
                var _knowledgeLevelCurreny = WoWCurrency.GetCurrencyById(1171);
                return (int)_knowledgeLevelCurreny.Amount;
            }
        }

        public int _artifactPower
        {
            get
            {
                return Lua.GetReturnVal<int>("return ArtifactWatchBar.xp", 0);
            }
        }

        /*
        private bool IncreasesArtifactKnowledge(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null)
                return false;

            var _requiredLevel = _itemInfo.RequiredLevel;
            if (_requiredLevel > Me.Level)
                return false;

            var _effects = _item.Effects;
            if (_effects == null || _effects.Count() == 0)
                return false;

            var _increasesArtifactKnowledge = _effects.Any(e => e != null &&
                                                                e.Spell != null &&
                                                                e.Spell.Name == "Increase Artifact Knowledge" &&
                                                                e.Spell.SpellEffects.Count() > 0 &&
                                                                e.Spell.SpellEffects.Any(se => se.BasePoints > _artifactKnowledgeLevel));
            if (_increasesArtifactKnowledge)
                return true;

            return false;
        }

        private bool ExceedsMaximum(WoWItem _item)
        {
            return false;

            Func<SpellEffect, bool> _giveArtifactPower =
                    se => se.EffectType == WoWSpellEffectType.GiveArtifactPower;

            var _spellEffect = _item.Effects
                                    .FirstOrDefault(e => e.Spell.SpellEffects.Any(_giveArtifactPower));
                                    .Spell.SpellEffects
                                    .FirstOrDefault(_giveArtifactPower);
            var _basePoints = _spellEffect.BasePoints;

            var _artifactKnowledge = WoWCurrency.GetCurrencyById(1171).Amount;

            Lua.DoString("ArtifactFrame:Show()");
            var _multiplier = Lua.GetReturnVal<double>("return C_ArtifactUI.GetArtifactKnowledgeMultiplier()", 0);
            Lua.DoString("ArtifactFrame:Hide()");
            var _givesValue = _basePoints;
            var _powerLeft = Lua.GetReturnVal<int>("return ArtifactWatchBar.xpForNextPoint", 0);

            return _givesValue > _powerLeft;
        }
        */

        private List<uint> _blacklistOpenIds = new List<uint>()
        {
            131926,
            35792       // QuestItem that needs to be opened by questing profile
        };

        private List<uint> _openableItemIds = new List<uint>()
        {
            141410,
            126947
        };

        private bool IsOpenable(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            if (_blacklistOpenIds.Contains(_item.Entry))
                return false;

            if (_openableItemIds.Contains(_item.Entry))
                return true;

            if (_item.IsOpenable)
                return true;

            var _effects = _item.Effects;
            if (_effects == null || _effects.Count() == 0)
                return false;

            var _isOpenable = _effects.Any(e => e != null &&
                                                e.Spell != null &&
                                                e.Spell.SpellEffects.Count() > 0 &&
                                                e.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.CreateRandomItem ||
                                                                               se.EffectType == WoWSpellEffectType.Loot));
            if (_isOpenable)
                return true;

            return false;
        }

        private bool IsCombinable(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _effect = _item.Effects.FirstOrDefault(e => e.TriggerType == ItemEffectTriggerType.OnUse);
            if (_effect == null)
                return false;

            var _spell = _effect.Spell;
            if (_spell == null || _spell.CreatesItemId == 0)
                return false;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null || _itemInfo.MaxStackSize != 200)
                return false;

            var _createdItemInfo = ItemInfo.FromId(_spell.CreatesItemId);
            if (_createdItemInfo == null || !_createdItemInfo.IsCraftingReagent)
                return false;

            var _reagents = _spell.InternalInfo.SpellReagents;
            if (_reagents == null || !_reagents.HasValue)
                return false;

            var _requiredReagentCount = (uint)_reagents.Value.ReagentCount[0] + 1;
            var _itemCount = Me.BagItems
                                .Where(i => i.Entry == _item.Entry)
                                .Sum(i => i.StackCount);
            if (_itemCount >= _requiredReagentCount)
                return true;

            return false;
        }

        /*
        private bool _startNewExpansion
        {
            get
            {
                return ProfileManager.XmlLocation != null && ProfileManager.XmlLocation.ToLower().Contains("questing profile pack") &&
                       ((Me.Level == 58 && (Me.CurrentMap.ExpansionId != 1 && !_inCapital)) ||
                        (Me.Level == 68 && Me.CurrentMap.ExpansionId == 1) ||
                        (Me.Level == 80 && Me.CurrentMap.ExpansionId == 2) ||
                        (Me.Level == 85 && !_inCapital && ((Me.IsHorde)
                                                                ? !IsQuestCompleted(29611)
                                                                : !IsQuestCompleted(29547))) ||
                        (Me.Level == 90 && Me.CurrentMap.ExpansionId == 4) ||
                        (Me.Level == 100 && Me.CurrentMap.ExpansionId == 5 && _accountExpansionLevel >= 6));
            }
        }
        */

        private void SetTps()
        {
            if (Settings.FpsStabiliser_FpsThreshold == 0)
                return;

            if (DeltaTime(_timeTpsChanged, "ms") < 100)
                return;

            if (StyxWoW.WoWClient.Fps <= Settings.FpsStabiliser_FpsThreshold)
            {
                if (TreeRoot.TicksPerSecond <= Settings.FpsStabiliser_MinTps)
                    return;

                TreeRoot.TicksPerSecond = Convert.ToByte(Settings.FpsStabiliser_MinTps);
                _timeTpsChanged = DateTime.Now;
            }
            else
            {
                if (TreeRoot.TicksPerSecond >= Settings.FpsStabiliser_MaxTps)
                    return;

                if (DeltaTime(_timeTpsChanged, "ms") < 500)
                    return;

                TreeRoot.TicksPerSecond += 1;
                _timeTpsChanged = DateTime.Now;
            }
        }

        private async Task<bool> HandleTps()
        {
            SetTps();

            if (DeltaTime(_timeRun, "ms") < 1000)
                return false;

            return true;
        }

        private async Task<bool> MoveTo(Vector3 _location, bool _useTaxi = false, bool _preventBlackspot = false)
        {
            await HandleLegionFlying();

            if (Me.HasAura("StagForm") && Flightor.CanFly)
                Me.GetAuraByName("Travel Form").TryCancelAura();

            await HandleTravelForm();
            await MountUp(_location);
            await HandleJump();
            await HandleTps();
            await HandleStuck();

            if (_preventBlackspot)
                await HandleNavigationBlackspots();

            // Prevent Motionless
            _timeLastMoved = DateTime.Now;
            _timeLastObjectiveChange = DateTime.Now;

            /*
            if (_useTaxi)
                await TakeTaxi(_location);
                */

            var _moveToParameters = new MoveToParameters(_location);
            _moveToParameters.AllowFlightPaths = _useTaxi;

            var _flyToParameters = new FlyToParameters(_location);
            _flyToParameters.CheckIndoors = true;
            _flyToParameters.MinHeight = 20;
            _flyToParameters.GroundNavParameters.AllowFlightPaths = _useTaxi;

            /*
            var _moveResult = (Flightor.CanFly && Me.IsOutdoors && !Me.Combat)
                                        ? Flightor.MoveTo(_flyToParameters)
                                        : Navigator.MoveTo(_location);

            if (_moveResult == MoveResult.PathGenerationFailed)
                return false;
            */

            if (Flightor.CanFly && DeltaTime(_timeFlightorFailed, "s") > 30)
            {
                if (Flightor.MoveTo(_flyToParameters) == MoveResult.PathGenerationFailed)
                {
                    _timeFlightorFailed = DateTime.Now;
                    await CommonCoroutines.LandAndDismount(null, true);
                }
            }
            else
            {
                if (Navigator.MoveTo(_moveToParameters) == MoveResult.PathGenerationFailed)
                {
                    DevLog("PathGenerationFailed");
                    return false;
                }
            }

            return true;
        }

        private async Task<bool> PreventTaxiRefreshing()
        {
            if (!Settings.General_PreventTaxi)
                return false;

            if (KnownFlightNodesManager.ElapsedSinceUpdate.TotalHours > 1)
                KnownFlightNodesManager.UpdateKnownFlightNodes(new byte[0]);

            return true;
        }

        private int _knownNodesCount
        {
            get
            {
                var _knownNodes = KnownFlightNodesManager.EnumerateKnownFlightNodes()
                                            .Where(n => Map.FromId((uint)n.MapId).ExpansionId == Me.CurrentMap.ExpansionId);
                return _knownNodes.Count();
            }
        }

        private async Task<bool> HandleTaxiNodes()
        {
            if (await PreventTaxiRefreshing())
                return false;

            if (_flightmaster == null)
                return false;

            if (_flightmaster.InteractType == WoWInteractType.TaxiNotEligible)
                return false;

            if (IsHelperProfile())
                return false;

            var _elapsedTime = KnownFlightNodesManager.ElapsedSinceUpdate;
            if (_knownNodesCount > 0 || Flightor.CanFly)
            {
                if (_ignoredGeneralGuids.ContainsKey(_flightmaster.Guid) &&
                    DeltaTime(_ignoredGeneralGuids[_flightmaster.Guid], "m") < 60)
                    return false;

                if (_flightmaster.InteractType != WoWInteractType.TaxiPathAvailable &&
                    _elapsedTime.TotalHours < 23)
                    return false;

                if (Me.IsFlying && GameWorld.TraceLine(Me.Location, _flightmaster.Location, TraceLineHitFlags.Collision))
                    return false;

                if (_inShrine)
                    return false;
            }

            if (_flightmaster.InteractType == WoWInteractType.TaxiPathAvailable)
                Log("Getting taxi node");
            else
            {
                Log("Refreshing taxi nodes");
                DevLog("ElapsedTime: {0}h", Math.Round(_elapsedTime.TotalHours));
                DevLog("InteractType: {0}", _flightmaster.InteractType);
            }

            var _flightmasterLocation = _flightmaster.Location;
            var _flightmasterEntry = _flightmaster.Entry;
            var _flightmasterName = _flightmaster.Name;
            var _flightmasterGuid = _flightmaster.Guid;
            var _startTime = DateTime.Now;
            var _maxTime = GetMaxTime(GetPathDistance(_flightmaster));

            while (!IsFrameVisible("TaxiFrame") && !IsFrameVisible("FlightMapFrame") && !breakOut)
            {
                DevLog("WHILE: HandleTaxiNodes");

                if (DeltaTime(_startTime, "s") > _maxTime &&
                    (_flightmaster == null || !_flightmaster.WithinInteractRange))
                {
                    DevLog("     MaxTime reached");
                    DevLog("     Blacklisting for 1 hour");
                    DevLog("     Name: {0} (Id: {1})", _flightmasterName, _flightmasterEntry);
                    if (!_ignoredGeneralGuids.ContainsKey(_flightmasterGuid))
                        _ignoredGeneralGuids.Add(_flightmasterGuid, DateTime.Now);
                    else
                        _ignoredGeneralGuids[_flightmasterGuid] = DateTime.Now;
                    return false;
                }

                if (_flightmaster == null || !_flightmaster.WithinInteractRange)
                {
                    if (!await MoveTo(_flightmasterLocation))
                    {
                        DevLog("     PathGenerationFailed");
                        DevLog("     Blacklisting for 1 hour");
                        DevLog("     Name: {0} (Id: {1})", _flightmasterName, _flightmasterEntry);
                        if (!_ignoredGeneralGuids.ContainsKey(_flightmasterGuid))
                            _ignoredGeneralGuids.Add(_flightmasterGuid, DateTime.Now);
                        else
                            _ignoredGeneralGuids[_flightmasterGuid] = DateTime.Now;
                        return false;
                    }
                }
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _flightmaster.Interact();

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _taxiEntry = GossipFrame.Instance.GossipOptionEntries.FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Taxi);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_taxiEntry.Index);
                        await Coroutine.Wait(5000, () => IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"));
                    }
                }
                await Coroutine.Yield();
            }

            if (IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"))
            {
                var _mask = WoWTaxi.GetKnownFlightNodesMask();
                KnownFlightNodesManager.UpdateKnownFlightNodes(_mask);
            }

            return true;
        }

        private async Task<bool> HandleKillBetweenHotspots()
        {
            if (!Settings.Quest_KillBetweenHotspots || !GlobalSettings.Instance.KillBetweenHotspots)
                return false;

            DevLog("KillBetweenHotspots");
            GlobalSettings.Instance.KillBetweenHotspots = false;
            GlobalSettings.Instance.Save();

            return true;
        }

        private bool HasNewMail(bool _ignorePerformance = false)
        {
            if (_checkedMails)
                return false;

            return IsFrameVisible("MiniMapMailFrame");
        }

        private async Task<bool> HandleNewMails()
        {
            RuntimeLog(">");

            if (!Settings.Bag_GetMails)
                return false;

            RuntimeLog(">");

            if (_mailbox == null)
                return false;

            RuntimeLog(">");

            if (!HasNewMail() && _newMail == null)  // MiniMapMailFrame
                return false;

            RuntimeLog(">");

            if (IsFrameVisible("MailFrame"))
                return false;

            RuntimeLog(">");

            if (Me.FreeBagSlots <= 5)
                return false;

            Log("Getting new mails");
            var _mailboxLoc = _mailbox.Location;

            while (!IsFrameVisible("MailFrame", true) && !breakOut)
            {
                DevLog("WHILE: HandleNewMails > GoTo");

                if (_mailbox == null || !_mailbox.WithinInteractRange)
                    await MoveTo(_mailboxLoc);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _mailbox.Interact();
                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("MailFrame", true));
                }

                await Coroutine.Yield();
            }
            await Coroutine.Wait(2000, () => _newMail != null);

            var _autoEquip = CharacterSettings.Instance.AutoEquip;
            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = false;

            while (_newMail != null && IsFrameVisible("MailFrame", true) && !breakOut)
            {
                DevLog("WHILE: HandleNewMails > Read");

                var _index = _newMail.Index;

                await Sleep();
                await MailFrame.Instance.GetMailAttachmentsCoroutine(_index);
                if (_newMail != null && _newMail.Index == _index)
                {
                    while (!IsFrameVisible("MailItem" + _index + "Button"))
                    {
                        await ClickButton("InboxNextPageButton");
                        await Coroutine.Yield();
                    }
                    await ClickButton($"MailItem{_index}Button");
                    await Coroutine.Wait(2 * 1000, () => IsFrameVisible("OpenMailFrame"));
                    Lua.DoString("OpenMailFrame:Hide()");
                }

                await Coroutine.Yield();
            }
            Lua.DoString("MailFrame:Hide()");

            if (!await Coroutine.Wait(2 * 1000, () => !HasNewMail()) )
            {
                Lua.DoString("ReloadUI()");
                await Coroutine.Wait(3000, () => !StyxWoW.IsInWorld);
                await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);
            }

            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = true;

            _checkedMails = true;
            return true;
        }

        private async Task<bool> Move(string _direction, int _duration = 0, bool _waitUntilFinished = false)
        {
            WoWMovement.MovementDirection _movementDirection = WoWMovement.MovementDirection.None;

            if (_direction.Contains("A"))
                _movementDirection |= WoWMovement.MovementDirection.AutoRun;
            if (_direction.Contains("B"))
                _movementDirection |= WoWMovement.MovementDirection.Backwards;
            if (_direction.Contains("D"))
                _movementDirection |= WoWMovement.MovementDirection.Descend;
            if (_direction.Contains("F"))
                _movementDirection |= WoWMovement.MovementDirection.Forward;
            if (_direction.Contains("J"))
                _movementDirection |= WoWMovement.MovementDirection.JumpAscend;
            if (_direction.Contains("L"))
                _movementDirection |= WoWMovement.MovementDirection.StrafeLeft;
            if (_direction.Contains("l"))
                _movementDirection |= WoWMovement.MovementDirection.TurnLeft;
            if (_direction.Contains("R"))
                _movementDirection |= WoWMovement.MovementDirection.StrafeRight;
            if (_direction.Contains("r"))
                _movementDirection |= WoWMovement.MovementDirection.TurnRight;

            if (_duration > 0)
                WoWMovement.Move(_movementDirection, _duration);
            else
                WoWMovement.Move(_movementDirection);

            if (_waitUntilFinished)
                await Coroutine.Sleep(_duration);

            return true;
        }

        private async Task<bool> Turn(string _direction, int _duration, int _turnPause)
        {
            if (DeltaTime(_timeLastTurn, "ms") < _turnPause)
                return false;

            _timeLastTurn = DateTime.Now;
            await Move($"{_direction.ToLower()}", _duration);

            return true;
        }

        private double _defaultSpeed
        {
            get
            {
                double _factor = 1;

                if (Me.Mounted)
                {
                    if (Me.IsFlying)
                        switch (Me.GetSkill(SkillLine.Riding).CurrentValue)
                        {
                            case 225:
                                _factor = 2.5;
                                break;
                            case 300:
                                _factor = 3.8;
                                break;
                            case 375:
                                _factor = 4.1;
                                break;
                        }
                    else
                        switch (Me.GetSkill(SkillLine.Riding).CurrentValue)
                        {
                            case 75:
                                _factor = 1.6;
                                break;
                            default:
                                _factor = 2;
                                break;
                        }
                }

                if (Me.IsSwimming)
                {
                    _factor = (Me.Class == WoWClass.Druid && Me.Shapeshift == ShapeshiftForm.Aqua)
                                        ? 1.3
                                        : 0.5;
                }

                _factor *= GetSpeedDecreasingFactor();

                /*
                var _decreasingSpeedAuraIds = new List<int>()
                {
                    193563,         // A Heavy Burden (QuestId: 40078)
                    191382,         // You Lift Brul (QuestId: 39440)
                    189915,         // -
                    189912,         // -
                    229539          //

                };
                var _hasAura = _decreasingSpeedAuraIds.Any(id => Me.HasAura(id));
                if (_hasAura)
                    _factor *= 0.25;
                    */

                return _factor * 7;
            }
        }

        private double GetSpeedDecreasingFactor()
        {
            var _speedDecreasingAura = Me.GetAllAuras()
                                            .FirstOrDefault(a => a.Spell != null &&
                                                                 a.Spell.SpellEffects.Count() > 0 &&
                                                                 a.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.ApplyAura &&
                                                                                                se.AuraType == WoWApplyAuraType.ModDecreaseSpeed));
            if (_speedDecreasingAura == null)
                return 1;

            var _spellEffect = _speedDecreasingAura.Spell.SpellEffects
                                    .FirstOrDefault(se => se.EffectType == WoWSpellEffectType.ApplyAura &&
                                                          se.AuraType == WoWApplyAuraType.ModDecreaseSpeed);
            var _basePoints = _spellEffect.BasePoints;
            var _factor = (100 - _basePoints) / 100.0;

            return _factor;
        }

        private List<int> _stuckIntervals = new List<int>
            {
                5,
                10,
                14,
                18,
                23,
                68, // RELOAD
                    // HEARTH
            };

        private async Task<bool> JumpToHearthStage()
        {
            DevLog("Hearth disabled: {0}", Settings.Hearth_Disable);

            if (InVehicle())
            {
                DevLog("WHILE: JumpToHearthStage > VehicleExit");
                Lua.DoString("VehicleExit()");
                await Coroutine.Sleep(2000);
            }

            if (MotionlessState == Motionless.Off)
            {
                Log("     Stage 7");
                StuckState = Stuck.Off;
            }
            else
            {
                Log("     Stage 3");
                MotionlessState = Motionless.Off;
            }

            HearthState = Hearth.On;
            return true;
        }

        private double _speed
        {
            get
            {
                var _activeMover = WoWMovement.ActiveMover;
                if (_activeMover == null)
                    return _defaultSpeed;

                var _distance = _activeMover.Location.Distance(_lastLocation);
                var _time = DeltaTime(_timeLastLocation, "ms");
                var _speed = _distance / _time * 1000f;

                return Math.Round(_speed, 1);
            }

        }

        private async Task<bool> IsStuck()
        {
            if (WoWMovement.ClickToMoveInfo.IsClickMoving)
                _timeLastCTM = DateTime.Now;

            if (IsFrameVisible("CinematicFrame", true))
            {
                _stuckCount = 0;
                return false;
            }

            var _activeMover = WoWMovement.ActiveMover
                                    ?? Me;
            if (_activeMover.IsMoving && _speed < (Settings.Stuck_SpeedTolerance / 100f) * _defaultSpeed &&
                DeltaTime(_timeLastCTM, "ms") < 750 &&
                DeltaTime(_timeMountUp, "ms") > 1500)
                _stuckCount++;

            if (_lastStuckCount == _stuckCount)
            {
                _stableStuckCount++;

                if (_stableStuckCount >= 3)
                    _stuckCount = 0;
            }
            else
                _stableStuckCount = 0;
            _lastStuckCount = _stuckCount;

            if (_stuckCount < Settings.Stuck_StuckCountThreshold)
            {
                _lastLocation = _activeMover.Location;
                _timeLastLocation = DateTime.Now;

                if (_activeMover == Me &&
                    !Me.IsInInstance &&
                    Settings.Stuck_StuckCountThreshold >= 5 &&
                    _stuckCount >= Settings.Stuck_StuckCountThreshold - 2 &&
                    Me.Mounted && !Me.IsFlying)
                {
                    var _myLocation = Me.Location;
                    var _clickLocation = (WoWMovement.ClickToMoveInfo.IsClickMoving)
                                                        ? WoWMovement.ClickToMoveInfo.ClickPos
                                                        : _myLocation;
                    var _deltaHeight = _clickLocation.Z - _myLocation.Z;
                    var _deltaFrontHeight = (_myLocation.Distance2D(_clickLocation) / 3) * _deltaHeight;
                    var _inFront = WoWMathHelper.CalculatePointInFront(_myLocation + new Vector3(0, 0, 2 + _deltaFrontHeight),
                                                                       Me.Rotation, 3);
                    var _isPassage = !GameWorld.TraceLine(_myLocation, _inFront, TraceLineHitFlags.Collision);

                    var _preStuckCount = 0;
                    var _stuckLocationEntry = _stuckLocations.FirstOrDefault(l => Me.Location.Distance((Vector3)(l[0])) < 10);
                    if (_stuckLocationEntry != null)
                        _preStuckCount = (int)_stuckLocationEntry[1];

                    if (_isPassage || _preStuckCount >= 1)
                    {
                        DevLog("PreventStuck");

                        if (DeltaTime(_timePreventStuck, "ms") < 8000)
                        {
                            _failedPreventStuckCount++;
                            DevLog("FailedPreventStuckCount++ ({0})", _failedPreventStuckCount);
                        }
                        else if (_failedPreventStuckCount > 0)
                        {
                            DevLog("FailedPreventStuckCount Reset");
                            _failedPreventStuckCount = 0;
                        }
                        _timePreventStuck = DateTime.Now;

                        if (_failedPreventStuckCount >= 2)
                        {
                            _failedPreventStuckCount = 0;
                            return true;
                        }

                        PrintInfo();
                        await CommonCoroutines.LandAndDismount(null, true);
                        await Move("F", 500, true);
                    }
                }

                return false;
            }

            return true;
        }

        private string PrintMatrix(Matrix4x4 _matrix)
        {
            return "<Matrix " +
                    _matrix.ToString()
                            .Replace("{ {", "")
                            .Replace("} }", "")
                            .Replace("} {", " ")
                            .Replace(":", "=\"")
                            .Replace(" M", "\" M") +
                   "\" />";
        }

        private Vector3 GetLastStuckPoint()
        {
            return (Vector3)((_stuckLocations[_stuckLocations.Count - 2])[0]);
        }

        private void PrintInfo(bool _stuck = true, int _preStuckCount = 0, double _deltaLastUnstuck = 0)
        {
            var _logString
                        = "\n\n\n" +
                           "----- KOM-LOG ----- \n" +
                           "\n" +
                          $"Location: {Me.Location} ({_mapPosition}) \n";

            if (_stuck && _stuckLocations.Count > 1)
                _logString += $"LastStuckLocation: {GetLastStuckPoint()} (distance {Math.Round(Me.Location.Distance(GetLastStuckPoint()), 1)} yards)\n";

            if (_preStuckCount > 0 || _deltaLastUnstuck > 0)
                _logString += $"PreStuckCount: {_preStuckCount}\n" +
                              $"DeltaLastUnstuck: {TimeString(_deltaLastUnstuck)}\n";


            if (_stuck)
            {
                var _closestObject = ObjectManager.GetObjectsOfType<WoWGameObject>()
                                            .OrderBy(o => o.Distance)
                                            .FirstOrDefault();

                _logString += "\n" +
                              $"ClickPos: {WoWMovement.ClickToMoveInfo.ClickPos} \n" +
                              $"Zone: {Me.RealZoneText} (ZoneId: {Me.ZoneId}) > {Me.MinimapZoneText}" + ((Me.SubZoneId != 0)
                                                                                                            ? $" (SubZoneId: {Me.SubZoneId})"
                                                                                                            : "") + "\n" +
                              $"Map: {Me.MapName} (MapId: {Me.MapId}) \n" +
                               "\n" +

                              $"We are stuck! (TPS: {TreeRoot.TicksPerSecond}, FPS: {StyxWoW.WoWClient.Fps}, Latency: {StyxWoW.WoWClient.Latency}, Map: {Me.MapName}, loc: {Me.Location})!\n" +

                              ((_closestObject != null)
                                        ? $"Nearest game object, distance {Math.Round(_closestObject.Distance, 1)} yards \n" +
                                            $"<!-- {_closestObject.Name} in {Me.RealZoneText} zone and on {Me.MapName} map -->\n" +
                                            $"<GameObject model=\"#{_closestObject.ModelFileId}\">\n" +
                                            $"   {PrintMatrix(_closestObject.GetWorldMatrix())}\n" +
                                             "</GameObject>"
                                        : "") + "\n" +
                               "\n" +
                              $"Profile: {ProfileManager.CurrentProfile.Name} \n" +
                              $"BotPoi: {Styx.CommonBot.POI.BotPoi.Current} \n" +
                              $"TreeRoot: {TreeRoot.GoalText} \n" +
                               "\n" +
                              $"DefaultSpeed: {_defaultSpeed} \n" +
                              $"     CurrentSpeed: {_speed} \n" +
                              $"     {Math.Round(_speed / _defaultSpeed * 100)} % of DefaultSpeed \n" +
                              $"FailedMountUpCount: {_failedMountUpCount} \n" +
                              $"FailedPathDistancesRequestCount: {_failedPathDistancesRequestCount} \n" +
                              $"SpeedTolerance: {Settings.Stuck_SpeedTolerance} \n" +
                              $"StuckCountThreshold: {Settings.Stuck_StuckCountThreshold} \n" +
                              $"Dismount: {Settings.Stuck_Dismount} \n" +
                               "\n" +
                               "--- Testing: \n" +
                              $"<MoveTo X=\"{Me.Location.X}\" Y=\"{Me.Location.Y}\" Z=\"{Me.Location.Z}\" /> \n" +
                              $"<Code File=\"MyCTM\" X=\"{WoWMovement.ClickToMoveInfo.ClickPos.X}\" Y=\"{WoWMovement.ClickToMoveInfo.ClickPos.Y}\" Z=\"{WoWMovement.ClickToMoveInfo.ClickPos.Z}\" /> \n" +
                               "\n" +
                               "----- KOM-LOG -----\n\n";
            }
            else
            {
                _logString += "\n" +
                              $"Zone: {Me.RealZoneText} (ZoneId: {Me.ZoneId}) > {Me.MinimapZoneText} (SubZoneId: {Me.SubZoneId})\n" +
                              $"Map: {Me.MapName} (MapId: {Me.MapId}) \n" +
                               "\n" +
                              $"Profile: {ProfileManager.CurrentProfile.Name} \n" +
                              $"BotPoi: {Styx.CommonBot.POI.BotPoi.Current} \n" +
                              $"TreeRoot: {TreeRoot.GoalText} \n" +
                               "\n" +
                              $"Timer: {Settings.Motionless_Timer} \n" +
                              $"ObjectiveTimer: {Settings.Motionless_ObjectiveTimer} \n" +
                               "\n" +
                               "----- KOM-LOG -----\n\n";
            }

            DevLog(_logString);
        }

        private async Task<bool> DoUnstuck(int _preStuckCount = 1, bool _motionless = false)
        {
            Navigator.Clear();
            Flightor.Clear();

            var _horizontalDirections = new List<string>
            {
                "L",
                "R"
            };

            var _verticalDirections = new List<string>
            {
                "J",
                "D"
            };

            var _randomNr = (_rnd.Next(2) == 0)
                                ? 0
                                : 1;
            var _horizontalDirection = _horizontalDirections[_randomNr];
            var _horizontalCounterDirection = _horizontalDirections[(_randomNr + 1) % 2];
            var _verticalDirection = "J";
            var _verticalCounterDirection = "J";

            if (Me.IsFlying ||
               (Flightor.CanFly && Me.Mounted && Mount.Current.IsFlyingMount))
            {
                var _randomNrVertical = (GameWorld.TraceLine(Me.Location, Me.Location + new Vector3(0, 0, 20f), TraceLineHitFlags.Collision))
                                                ? 1
                                                : 0;
                _verticalDirection = _verticalDirections[_randomNr];
                _verticalCounterDirection = _verticalDirections[(_randomNr + 1) % 2];
            }

            var _timeStartStuck = DateTime.Now;
            var _lastStage = Stuck.On;
            var _turnPause = 100;
            _lastStuckLocation = Me.Location;

            var _radius = Settings.Stuck_BlackspotRadius;
            if (_radius > 0)
            {
                /*
                _radius = (_radius == 1)
                                ? (_preStuckCount + 1) * _defaultSpeed
                                : _radius;
                */

                var _blackspot = new Styx.CommonBot.Profiles.Blackspot(
                                            Me.Location,
                                            (float)_radius,
                                            (float)_radius,
                                            null,
                                            (int)Me.MapId);
                BlackspotManager.AddBlackspot(_blackspot);
            }

            Log("STUCK! Unstucking...");

            while (Me.Location.Distance(_lastStuckLocation) < (_preStuckCount + 1) * _defaultSpeed)
            {
                DevLog("WHILE: DoUnstuck");

                if (!StyxWoW.IsInWorld)
                    await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);

                if (!_init)
                    return false;

                if (DungeonState != Dungeon.InDungeon &&
                    Me.IsAlive && Me.Combat && Me.HealthPercent <= 70)
                {
                    DevLog("Combat and Health <= 70");
                    return false;
                }

                var _currentStageEntry = _stuckIntervals.FirstOrDefault(s => DeltaTime(_timeStartStuck, "s") < s);
                StuckState = (Stuck)(_stuckIntervals.IndexOf(_currentStageEntry) + 2);

                if ((int)StuckState >= 5 && IsPlayerInRange)
                {
                    DevLog("PlayerInRange");

                    if(DungeonState == Dungeon.InDungeon)
                    {
                        DevLog("Leave group");
                        Lua.DoString("LeaveParty()");
                        return false;
                    }

                    await JumpToHearthStage();
                    return false;
                }

                if (StuckState != _lastStage)
                {
                    Log("     {0}", StuckState.ToString().Replace("e", "e "));
                    _turnPause = 0;
                    _lastStage = StuckState;
                }

                _turnPause = (_speed < (Settings.Stuck_SpeedTolerance / 100f) * _defaultSpeed)
                                    ? 100
                                    : _turnPause + 100;

                if ((int)StuckState <= 5)
                    await Move("A");

                switch (StuckState)
                {
                    case Stuck.Stage1:
                        await Move($"{_verticalDirection}{_horizontalDirection}", 100);
                        await Turn($"{_horizontalCounterDirection}", 50, _turnPause);
                        break;

                    case Stuck.Stage2:
                        await Move($"{_verticalCounterDirection}{_horizontalCounterDirection}", 100);
                        await Turn($"{_horizontalDirection}", 50, _turnPause);
                        break;

                    case Stuck.Stage3:
                        if (Me.Mounted && !Me.IsFlying)
                            await CommonCoroutines.LandAndDismount(null, true);

                        await Move($"{_verticalDirection}{_horizontalDirection}", 100);
                        await Turn($"{_horizontalCounterDirection}", 100, _turnPause);
                        break;

                    case Stuck.Stage4:
                        await Move($"{_verticalCounterDirection}{_horizontalCounterDirection}", 100);
                        await Turn($"{_horizontalDirection}", 100, _turnPause);
                        break;

                    case Stuck.Stage5:
                        if (MotionlessState != Motionless.Off)
                            return false;

                        await Move("B", 5000, true);
                        break;

                    case Stuck.Stage6:
                        _timeReload = DateTime.Now;
                        ReloadProfile();
                        return false;
                        break;
                }

                _lastLocation = Me.Location;
                _timeLastLocation = DateTime.Now;
                await Coroutine.Yield();
            }

            await CommonCoroutines.StopMoving();
            return true;
        }

        private bool ActuallyInCombat()
        {
            return Me.IsActuallyInCombat;

            /*
            var _attackingMobs = ObjectManager.GetObjectsOfType<WoWUnit>()
                    .Where(u => u.Combat &&
                                u.TaggedByMe &&
                                !u.IsFriendly &&
                                u.IsTargetingMeOrPet &&
                                ((Me.IsMelee)
                                    ? u.IsWithinMeleeRange
                                    : u.Distance < 45));

            if (_attackingMobs.Count() == 0)
                return false;

            var _target = Me.CurrentTarget;
            if (_target == null)
                return false;

            var _meIsTargetingMob = _attackingMobs.Contains(_target);
            if (!_meIsTargetingMob)
                return false;

            return true;
            */
        }

        private async Task<bool> HandleStuck()
        {
            var _hasAura = Me.HasAura(203740);      // QUEST #41094
            if (_hasAura)
                return false;

            if (HearthState != Hearth.Off)
                return false;

            if (Settings.Stuck_Disable)
                return false;

            Navigator.NavigationProvider.ClearStuckInfo();

            if (DeltaTime(_timeMountUp, "ms") < 1500)
                return false;

            if (Me.Stunned)
                return false;

            /*
            if (!Me.IsAlive)
                return false;

            if (ActuallyInCombat())
            {
                _stuckCount = 0;
                return false;
            }
*/
            if (Me.IsAlive && Me.Combat && Me.HealthPercent <= 70)
                return false;

            if (Me.OnTaxi || (Me.IsOnTransport && !InVehicle()) )
                return false;

            if (!await IsStuck() &&
                StuckState == Stuck.Off &&
                _failedMountUpCount < 3 &&
                _failedPathDistancesRequestCount < 5)
                return false;

            RuntimeLog(">");

            if (DungeonState != Dungeon.InDungeon &&
                Me.IsAlive && await HandleReloadStage())
            {
                await JumpToHearthStage();
                return false;
            }

            if (StuckState == Stuck.Stage6)
                return false;

            var _preStuckCount = 0;
            var _stuckLocationEntry = _stuckLocations.FirstOrDefault(l => Me.Location.Distance((Vector3)(l[0])) < 10);
            if (_stuckLocationEntry == null)
            {
                _stuckLocations.Add(new object[] { Me.Location, 0, DateTime.MinValue });
                _stuckLocationEntry = _stuckLocations.Last();

                PrintInfo();
            }
            else
            {
                var _timeLastUnstuck = (DateTime)_stuckLocationEntry[2];
                var _deltaLastUnstuck = DeltaTime(_timeLastUnstuck, "s");
                if (_deltaLastUnstuck >= 30)
                {
                    _stuckLocationEntry[1] = 0;
                    _stuckLocationEntry[2] = DateTime.MinValue;
                }

                _preStuckCount = (int)_stuckLocationEntry[1];
                PrintInfo(true, _preStuckCount, _deltaLastUnstuck);

                if (_preStuckCount >= 3)
                {
                    DevLog("PreStuckCount >= 3");
                    Log("STUCK! Unstucking...");
                    await JumpToHearthStage();
                    return false;
                }
            }

            if ((Settings.Stuck_Dismount || Me.IsInInstance) && Me.Mounted && !Me.IsFlying)
            {
                DevLog("StuckDismount enabled");
                DevLog("InInstance: {0}", Me.IsInInstance);
                await CommonCoroutines.LandAndDismount(null, true);
            }

            if (!await DoUnstuck(_preStuckCount))
                return false;

            _stuckLocationEntry[2] = DateTime.Now;
            _stuckLocationEntry[1] = (int)(_stuckLocationEntry[1]) + 1;

            Reset();
            return true;
        }

        private class ClassHearth
        {
            public ClassHearth(WoWClass _class,
                               string _spellName,
                               int _portSpellId,
                               int _returnSpellId,
                               Vector3 _portLocation)
            {
                Class = _class;
                SpellName = _spellName;
                PortSpellId = _portSpellId;
                ReturnSpellId = _returnSpellId;
                PortLocation = _portLocation;
            }

            public WoWClass Class { get; }
            public string SpellName { get; }
            public int PortSpellId { get; }
            public int ReturnSpellId { get; }
            public Vector3 PortLocation { get; }
        }

        private List<ClassHearth> _classHearthData = new List<ClassHearth>
        {
            new ClassHearth( WoWClass.Monk,           "Zen Pilgrimage",       126892, 126896, new Vector3(3826.87f, 1795.74f, 950.3f) ),
            new ClassHearth( WoWClass.Druid,          "Teleport: Moonglade",  18960,  18960,  new Vector3(7995.56f, -2679.75f, 512.49f) ),
            new ClassHearth( WoWClass.Druid,          "Dreamwalk",            193753, 193753, new Vector3(1633.01f, 1582.1f, 5.755894f) ),
            new ClassHearth( WoWClass.DeathKnight,    "Death Gate",           50977,  50977,  new Vector3(2359.64f, -5662.41f, 382.2605f) )
        };

        private async Task<bool> CastClassHearth()
        {
            if ((int)HearthState >= 2)
                return false;

            var _classHearth = _classHearthData.FirstOrDefault(ch => Me.Class == ch.Class &&
                                                                     SpellManager.HasSpell(ch.PortSpellId) &&
                                                                     SpellManager.CanCast(ch.PortSpellId) &&
                                                                     SpellManager.HasSpell(ch.ReturnSpellId) &&
                                                                     SpellManager.CanCast(ch.PortSpellId) &&
                                                                     GetCondition(ch.Class.ToString(), true));
            if (_classHearth == null)
                return false;

            var _spellName = _classHearth.SpellName;
            var _portSpellId = _classHearth.PortSpellId;
            var _returnSpellId = _classHearth.ReturnSpellId;
            var _portLocation = _classHearth.PortLocation;
            var _preMapId = Me.CurrentMap.ExpansionId;

            Log("HEARTH! Taking me home...");
            Log("     {0}", _spellName);

            await CommonCoroutines.LandAndDismount(null, true);
            await CommonCoroutines.StopMoving();

            while (StyxWoW.IsInWorld && Me.Location.Distance(_portLocation) > 60 && !breakOut)
            {
                DevLog("WHILE: HandleClassHearth > Port");
                if (Me.Class == WoWClass.DeathKnight
                            ? _deathGate == null
                            : true)
                {
                    if (!Me.IsCasting && SpellManager.CanCast(_portSpellId))
                    {
                        SpellManager.CastSpellById(_portSpellId);
                        await Coroutine.Wait(1000, () => Me.IsCasting);
                        await Coroutine.Wait(10 * 1000, () => !Me.IsCasting);

                        await Coroutine.Wait(2000, () => (Me.Class == WoWClass.DeathKnight)
                                                                    ? _deathGate != null
                                                                    : !StyxWoW.IsInWorld);
                        if (StyxWoW.IsInWorld && (Me.Class == WoWClass.DeathKnight
                                                            ? _deathGate == null
                                                            : true))
                            await Move("B", 200, true);
                    }
                }
                else
                {
                    await Sleep();
                    _deathGate.Interact();
                    await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                }
                await Coroutine.Yield();
            }
            await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld || !_init);

            if (_preMapId != Me.CurrentMap.ExpansionId)
            {
                await WaitForMapChange();
                _preMapId = Me.CurrentMap.ExpansionId;
            }

            if (_spellName == "Death Gate" || _spellName == "Dreamwalk")
                Log("     Waiting for \"{0}\" cooldown...", _spellName);

            while (StyxWoW.IsInWorld && Me.Location.Distance(_portLocation) < 60 && !breakOut)
            {
                DevLog("WHILE: HandleClassHearth > Return");
                if (Me.Class != WoWClass.DeathKnight || _deathGate == null)
                {
                    if (!Me.IsCasting && SpellManager.CanCast(_returnSpellId))
                    {
                        SpellManager.CastSpellById(_returnSpellId);

                        if(await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                            if(await Coroutine.Wait(10 * 1000, () => !Me.IsCasting))
                                if(await Coroutine.Wait(2 * 1000, () => (Me.Class == WoWClass.DeathKnight)
                                                                            ? _deathGate != null
                                                                            : !StyxWoW.IsInWorld))

                        if (StyxWoW.IsInWorld && (Me.Class != WoWClass.DeathKnight || _deathGate == null) )
                            await Move("B", 200, true);
                    }
                }
                else
                {
                    await Sleep();
                    _deathGate.Interact();
                    await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                }
                await Coroutine.Yield();
            }
            await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld || !_init);

            if (_preMapId != Me.CurrentMap.ExpansionId)
                await WaitForMapChange();

            return true;
        }

        private async Task<bool> UseHearthItem()
        {
            var _hearthItemEntries = _hearthItemData.Where(e => Me.BagItems.Any(i => i.Entry == (int)(e[1])) &&
                                                                GetCondition((string)(e[0])) &&
                                                                GetCondition((string)(e[0]), true));
            var _hearthItemEntry = _hearthItemEntries.FirstOrDefault();
            if (_hearthItemEntry == null)
                return false;

            if (_hearthItemEntries.Count() > 1)
                _hearthItemEntry = _hearthItemEntries.FirstOrDefault(e => GetItem( (int)(e[1]) ).CooldownTimeLeft.TotalSeconds < 30);

            if (_hearthItemEntry == null)
                _hearthItemEntry = _hearthItemEntries.FirstOrDefault();

            var _itemName = (string)_hearthItemEntry[0];
            var _itemId = (int)_hearthItemEntry[1];
            var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                        i.Entry == _itemId);

            if (HearthState == Hearth.Capital)
                Log("Going to {0}", (Me.IsHorde)
                                        ? "Orgrimmar"
                                        : "Stormwind");
            else if ((int)HearthState >= 3)
                Log("Going to {0}", HearthState.ToString());

            Log("HEARTH! Taking me home...");
            if (_item.CooldownTimeLeft.TotalSeconds > 0)
            {
                Log("     Waiting for \"{0}\" cooldown...", _itemName);

                var _seconds = _item.CooldownTimeLeft.TotalSeconds;
                Log("          {0} left", TimeString(_seconds));

                Func<bool> _waitForCooldown = () =>
                {
                    if (!_init)
                        return true;

                    if (breakOut)
                        return true;

                    var _timeLeft = Math.Round(_item.CooldownTimeLeft.TotalSeconds);
                    if (_timeLeft % 10 == 0 && _logTimer.IsFinished)
                    {
                        Log("          {0} left", TimeString(_timeLeft));
                        _logTimer.Reset();
                    }

                    return _item.CooldownTimeLeft.TotalSeconds == 0;
                };
                await Coroutine.Wait(30 * 60 * 1000, _waitForCooldown);
            }

            await CommonCoroutines.LandAndDismount(null, true);
            await CommonCoroutines.StopMoving();

            Log("     Using \"{0}\"", _itemName);
            var _portLocation = Me.Location;
            var _preZone = Me.ZoneId;
            while (StyxWoW.IsInWorld && Me.Location.Distance(_portLocation) < 60 && !breakOut)
            {
                DevLog("WHILE: UseHearthItem");

                if (_item.CooldownTimeLeft.TotalSeconds > 0)
                    return false;

                if (InHearthArea())
                {
                    DevLog("     InHearthArea");
                    return true;
                }

                if (Me.IsActuallyInCombat && MotionlessState == Motionless.Off)  // +++
                    return false;

                await CommonCoroutines.StopMoving();
                if(await UseItem(_item))
                    if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                        if (await Coroutine.Wait(10 * 1000, () => !Me.IsCasting))
                            await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                if (StyxWoW.IsInWorld && !Me.IsCasting)
                    await Move("B", 400, true);

                await Coroutine.Yield();
            }

            if (!breakOut)
            {
                await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld || !_init);
                await Coroutine.Wait(6 * 1000, () => _preZone != Me.ZoneId);
            }

            return true;
        }

        private bool GetCondition(string _optionName, bool _returnSetting = false)
        {
            if (_returnSetting)
            {
                switch (_optionName)
                {
                    case "Monk":
                        return Settings.Hearth_Monk;
                        break;
                    case "Druid":
                        return Settings.Hearth_Druid;
                        break;
                    case "DeathKnight":
                        return Settings.Hearth_DeathKnight;
                        break;
                    case "Flightmaster's Whistle":
                        return Settings.Hearth_FlightmastersWhistle;
                        break;
                    case "Dalaran Hearthstone":
                        return Settings.Hearth_DalaranHearthstone;
                        break;
                    case "Garrison Hearthstone":
                        return Settings.Hearth_GarrisonHearthstone;
                        break;
                    case "Stormwind Portal Stone":
                        return Settings.Hearth_PortalStone;
                    case "Orgrimmar Portal Stone":
                        return Settings.Hearth_PortalStone;
                    case "Hearthstone":
                        return Settings.Hearth_Hearthstone;
                        break;
                    case "Pandaria":
                        return Settings.Archaeology_GoToPandaria;
                        break;
                    case "Draenor":
                        return Settings.Archaeology_GoToDraenor;
                        break;
                    case "BrokenIsles":
                        return Settings.Archaeology_GoToBrokenIsles;
                        break;
                }
            }
            else
            {
                switch (_optionName)
                {
                    case "Flightmaster's Whistle":
                        return Me.MapId == 1220 &&
                                Me.ZoneId != 7502 &&
                                GetItem(141605).Usable &&
                                HearthState != Hearth.Capital &&
                                HearthState != Hearth.Pandaria &&
                                HearthState != Hearth.Draenor &&
                                HearthState != Hearth.Dalaran;
                        break;

                    case "Dalaran Hearthstone":
                        return (
                                    HearthState == Hearth.Dalaran ||
                                    Me.CurrentMap.ExpansionId == 6
                               ) &&
                               HearthState != Hearth.Capital &&
                               HearthState != Hearth.Pandaria &&
                               HearthState != Hearth.Draenor;
                        break;

                    case "Garrison Hearthstone":
                        return (
                                   HearthState == Hearth.Draenor ||
                                   (
                                       Me.CurrentMap.ExpansionId == 5 &&
                                       (
                                           (
                                               ProfileManager.XmlLocation != null &&
                                               ProfileManager.XmlLocation.ToLower().Contains("questing profile pack") &&
                                               Me.Level != 100
                                           ) ||
                                           (
                                               ProfileManager.XmlLocation != null &&
                                               !ProfileManager.XmlLocation.ToLower().Contains("questing profile pack")
                                           )
                                       )
                                   )
                               ) &&
                               HearthState != Hearth.Capital &&
                               HearthState != Hearth.Pandaria &&
                               HearthState != Hearth.Dalaran;
                        break;

                    case "Orgrimmar Portal Stone":
                        return (int)HearthState <= 3;
                        break;

                    case "Stormwind Portal Stone":
                        return (int)HearthState <= 3;
                        break;

                    case "Hearthstone":
                        if (Me.CurrentMap.ExpansionId >= 5 &&
                            Me.MapId != HearthstoneMapId()) /* &&
                            UnfavourableAreaSet()) */
                            return false;

                        if ((int)HearthState >= 3 &&
                            (int)HearthState + 1 != HearthstoneMapId())
                            return false;

                        return true;
                        break;

                    case "40039":   // STARLIGHT ROSE - RANK 3
                        if (Settings.Test_Mode.ToString() != "StarlightRoseRank3")
                            return false;

                        var _starlightRoseCount = GetItemCount(129158);
                        if (_starlightRoseCount < 20)    // STARLIGHT ROSEDUST
                        {
                            DevLog("Not enough starlight rosedust for quest 40039 ({0})", _starlightRoseCount);
                            return false;
                        }

                        return true;
                        break;

                    case "40033":   // FJARNSKAGGL - RANK 3
                        if (Settings.Test_Mode.ToString() != "FjarnskagglRank3")
                            return false;

                        var _fjarnskagglCount = GetItemCount(124104);
                        if (_fjarnskagglCount < 20)    // FJARNSKAGGL
                        {
                            DevLog("Not enough fjarnskaggl for quest 40033 ({0})", _fjarnskagglCount);
                            return false;
                        }

                        return true;
                        break;

                    case "40023":   // DREAMLEAF - RANK 3
                        if (Settings.Test_Mode.ToString() != "DreamleafRank3")
                            return false;

                        return true;
                        break;

                    case "38803":   // FELSLATE - RANK 3 (DEPOSIT)
                        if (Settings.Test_Mode.ToString() != "FelslateRank3Deposit")
                            return false;

                        return true;
                        break;

                    case "38804":   // FELSLATE - RANK 3 (SEAM)
                        if (Settings.Test_Mode.ToString() != "FelslateRank3Seam")
                            return false;

                        return true;
                        break;

                    case "38805":   // FELSLATE - RANK 3 (LIVING)
                        if (Settings.Test_Mode.ToString() != "FelslateRank3Living")
                            return false;

                        return true;
                        break;

                    // EASTERN KINGDOMS

                    case "EbonHold_FlyWayOut": // EBON HOLD: FLY WAY OUT
                        return Me.Location.Distance(new Vector3(2416.43f, -5623.803f, 461.1342f)) <= 35;
                        break;

                    case "SwampOfSorrows_Tower": // SWAMP OF SORROWS: TOWER
                        return Me.Location.Distance(new Vector3(-10406.78f, -2614.18f, 21.88813f)) <= 2;
                        break;

                    // KALIMDOR

                    case "Orgrimmar_CleftOfShadows": // ORGRIMMAR: CLEFT OF SHADOWS
                        return Me.Location.Distance(new Vector3(1806.51f, -4335.822f, 42.4572f)) <= 20;
                        break;


                    case "Uldum_ChamberOfTheSun": // ULDUM: CHAMBER OF THE SUN
                        return Me.Location.Distance(new Vector3(-10427.58f, -304.6555f, 304.4986f)) <= 60;
                        break;

                    // OUTLAND

                    case "Shattrath_TeraceOfLight": // SHATTRATH : TERACE OF LIGHT
                        return Me.Location.Distance(new Vector3(-1864.536f, 5431.456f, 13.34797f)) <= 20;
                        break;

                    case "HellfirePeninsula_ElitesAtPortal": // HELLFIRE PENINSULA: ELITES AT PORTAL
                        return Me.Location.Distance(new Vector3(-250.8762f, 1090.677f, 47.98906f)) <= 30;
                        break;

                    // NORTHREND

                    case "GrizzlyHills_Grizzlemaw": // GRIZZLY HILLS: GRIZZLEMAW
                        return Me.Location.Distance(new Vector3(4019.972f, -3690.464f, 351.2816f)) <= 60;
                        break;

                    case "StormPeak_Mine": // STORM PEAK: MINE
                        return Me.Location.Distance(new Vector3(6902.337f, -1155.754f, 832.1863f)) <= 25;
                        break;

                    case "OldDalaran_FlyingBelow": // OLD DALARAN: FLYING BELOW
                        return FlyingBelowDalaran(new Vector3(5808.426f, 638.0281f, 647.6355f), 300);
                        break;

                    // PANDARIA

                    case "ValleyOfFourWinds_Cliff": // VALLEY OF FOUR WINDS: CLIFF
                        return DistanceToPath(Me.Location,
                                        new Vector3[] {
                                            new Vector3(-684.1757f, -309.7902f, 116.1328f) ,
                                            new Vector3(-686.4291f, -252.9519f, 132.2081f) ,
                                            new Vector3(-672.5904f, -301.6006f, 116.7788f) ,
                                            new Vector3(-721.488f, -110.343f, 102.8925f) ,
                                            new Vector3(-717.047f, 40.55526f, 87.42505f) ,
                                            new Vector3(-905.7255f, 243.4491f, 82.85305f) ,
                                            new Vector3(-950.6279f, 971.4322f, 79.7547f) ,
                                            new Vector3(-1008.135f, 1185.015f, 69.11998f) ,
                                            new Vector3(-1144.735f, 1497.213f, 59.05566f) ,
                                            new Vector3(-1092.256f, 1731.296f, 73.07876f) ,
                                            new Vector3(-1136.38f, 1954.926f, 78.24606f)
                                        }) <= 30;
                        break;

                    // BROKEN ISLES

                    case "NewDalaran_HerbalismTrainer": // NEW DALARAN: HERABLISM TRAINER
                        return Me.Location.Distance(new Vector3(-771.8003f, 4521.507f, 740.1326f)) <= 5;
                        break;

                    case "NewDalaran_FlyingBelow": // NEW DALARAN: FLYING BELOW
                        return FlyingBelowDalaran(new Vector3(-844.2977f, 4472.829f, 735.8796f), 300);
                        break;
                }
            }

            return false;
        }

        private List<object[]> _hearthItemData = new List<object[]>
        {
            new object[] {"Flightmaster's Whistle", 141605 },
            new object[] {"Dalaran Hearthstone",    140192 },
            new object[] {"Garrison Hearthstone",   110560 },
            new object[] {"Orgrimmar Portal Stone", 132119 },
            new object[] {"Stormwind Portal Stone", 132120 },
            new object[] {"Hearthstone",            6948 },
        };

        /*
        private bool UnfavourableAreaSet()
        {
            var _area = AreaTable.FromId(Me.HearthstoneAreaId);
            if (_area == null)
                return true;

            if (Me.HearthstoneAreaId != _pandariaHearthstoneAreaId &&
               ((Me.IsHorde)
                    ? _area.MapId != 1
                    : _area.MapId != 0))
                return true;

            return false;
        }
        */

        private int HearthstoneMapId()
        {
            var _area = AreaTable.FromId(Me.HearthstoneAreaId);
            if (_area == null)
                return 0;

            return (int)_area.MapId;
        }

        private bool InHearthArea()
        {
            var _area = AreaTable.FromId(Me.HearthstoneAreaId);
            if (_area == null)
                return false;

            return Me.SubZoneId == _area.AreaId;
        }

        private async Task<bool> HandleHearth()
        {
            if (Settings.Hearth_Disable)
                return false;

            if (!Me.IsAlive)
                return false;

            if (Me.IsActuallyInCombat)
                return false;

            /*
            if (Me.Combat && Me.HealthPercent <= 75 && !Me.Mounted)  // +++
                return false;
            */

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
                return false;

            RuntimeLog(">");

            /*
            if (await HandleHearthstoneArea(true))
                return true;

            RuntimeLog(">");

            if (Settings.Hearth_Hearthstone &&
               Settings.Hearth_SetHearthstone)
            {
                if (UnfavourableAreaSet())
                {
                    Log("Hearthstone is set to a highly unfavourable area. Place char in {0} and restart bot.",
                        (Me.IsHorde)
                              ? "Orgrimmar"
                              : "Stormwind");
                    Settings.General_InfiniteMode = false;
                    Log("     STOPPING bot...");
                    DevLog("HearthstoneAreaId: {0}", Me.HearthstoneAreaId);

                    var _area = AreaTable.FromId(Me.HearthstoneAreaId);
                    if (_area == null)
                        DevLog("Hearthstone not set");
                    else
                    {
                        DevLog("AreaName: {0}", _area.AreaName);
                        DevLog("MapId: {0}", _area.MapId);
                    }
                    TreeRoot.Stop("KeepOnMoving");
                }

                if (_startNewExpansion)
                {
                    if (await HandleCompletedQuests(true))
                        return false;

                    DevLog("Hearth.Capital: StartNewExpansion");
                    Log("Preparing for starting new expansion");
                    HearthState = Hearth.Capital;
                }
            }
            */

            if (HearthState == Hearth.Off &&
                _failedMountUpCount < 10)
                return false;

            if (Settings.Hearth_PreStop)
            {
                Reset(true);
                Settings.General_InfiniteMode = false;
                Log("HEARTH! Taking me home...");
                Log("     STOPPING bot...");
                TreeRoot.Stop("KeepOnMoving");
            }

            var _preMapId = Me.CurrentMap.ExpansionId;
            if (!await CastClassHearth())
            {
                if (!await UseHearthItem())
                {
                    //
                    // HEARTH AGAIN WHEN INTERRUPTED WHILE TRYING TO HEARTH
                    //
                    if (Me.IsActuallyInCombat)
                        return false;

                    Log("HEARTH! Taking me home...");
                    Log("     Couldn't do my job as no hearth option seems to be available");
                    Log("     STOPPING bot...");

                    foreach (var _classHearth in _classHearthData)
                    {
                        DevLog("{0}", _classHearth.SpellName);
                        DevLog("     AppropriateClass: {0}", Me.Class == _classHearth.Class);
                        DevLog("     SettingEnabled: {0}", GetCondition(_classHearth.Class.ToString(), true));
                        DevLog("     HasPortSpell: {0}", SpellManager.HasSpell(_classHearth.PortSpellId));
                        DevLog("     CanCastSpell: {0}", SpellManager.CanCast(_classHearth.PortSpellId));
                        DevLog("     HasReturnSpell: {0}", SpellManager.HasSpell(_classHearth.ReturnSpellId));
                    }

                    foreach (var _entry in _hearthItemData)
                    {
                        DevLog("{0}", (string)_entry[0]);
                        DevLog("     HasItem: {0}", GetItem((int)_entry[1]) != null);
                        DevLog("     SettingEnabled: {0}", GetCondition((string)_entry[0]), true);
                        DevLog("     ConditionsMeet: {0}", GetCondition((string)_entry[0]));
                    }

                    // Settings.General_InfiniteMode = false;
                    await CommonCoroutines.StopMoving();
                    TreeRoot.Stop("KeepOnMoving");
                }
            }

            /*
            if (HearthState == Hearth.Capital && !_inCapital)
            {
                if (_inShrine)
                {
                    var _portalId = (Me.IsHorde)
                                        ? 215127
                                        : 215119;
                    var _portal = GetObject(_portalId);
                    var _portalLocation = (Me.IsHorde)
                                                ? new Vector3(1735.073f, 873.5676f, 487.4114f)
                                                : new Vector3(828.2266f, 176.9051f, 519.9392f);

                    while (!_inCapital && !breakOut)
                    {
                        DevLog("WHILE: HandleHearth");

                        await TakePortal(_portalId, _portalLocation);
                        await Coroutine.Yield();
                    }
                }
                else if ((Me.IsHorde)
                            ? Me.MapId == 1
                            : Me.MapId == 0)
                    await HandleHearthstoneArea(true, true);
            }
            */

            if (!Me.IsActuallyInCombat)
            {
                if (_preMapId != Me.CurrentMap.ExpansionId)
                    await WaitForMapChange();

                Reset();
                if (Settings.Hearth_PostStop)
                {
                    Settings.General_InfiniteMode = false;
                    Log("     STOPPING bot...");
                    TreeRoot.Stop("KeepOnMoving");
                }

                ReloadProfile();
            }
            return true;
        }

        private async Task<bool> HandleGarrisonResources()
        {
            if (Me.IsHorde
                    ? Me.ZoneId != 7004
                    : Me.ZoneId != 7078)
                return false;

            if (_garrisonCache == null)
                return false;

            if (DeltaTime(_timeGarrisonCacheInteracted, "m") < 60)
                return false;

            Log("Getting garrison resources");
            var _garrisonCacheLocation = _garrisonCache.Location;
            var _preGarrisonResourcesCount = _garrisonResourcesCount;
            while (_preGarrisonResourcesCount == _garrisonResourcesCount && _garrisonCache != null && !breakOut)
            {
                DevLog("WHILE: GetGarrisonResources");

                if (_garrisonCache == null || !_garrisonCache.WithinInteractRange)
                    await MoveTo(_garrisonCacheLocation);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _garrisonCache.Interact();
                }
                await Coroutine.Yield();
            }

            _timeGarrisonCacheInteracted = DateTime.Now;
            return true;
        }

        private async Task<bool> HandleTrainFlying()
        {
            if (Settings.Quest_TrainFlying.ToString() == "None")
                return false;

            if (Me.Level < 70)
                return false;

            if (Me.GetSkill(SkillLine.Riding).CurrentValue < 225 ||
                Me.GetSkill(SkillLine.Riding).CurrentValue >= 375)
                return false;

            if (Me.GetSkill(SkillLine.Riding).CurrentValue == 300 &&
                !Settings.Quest_TrainFlying.ToString().Contains("Master"))
                return false;

            if (Me.GetSkill(SkillLine.Riding).CurrentValue == 300 &&
                Settings.Quest_TrainFlying.ToString().Contains("Master") &&
                Me.Level < 80)
                return false;

            var _trainerDataEntry = GetClosestTrainerEntry(SkillLine.Riding);
            if (_trainerDataEntry == null)
                return false;

            var _factionId = (uint)_trainerDataEntry[5];
            var _neededGold = GetFlyingGold(_factionId);
            if (Me.Gold < _neededGold)
                return false;

            /*
            if (!_inCapital)
            {
                if (!Settings.Hearth_Hearthstone ||
                    !Settings.Hearth_SetHearthstone ||
                    UnfavourableAreaSet())
                    return false;

                DevLog("Hearth.Capital: TrainFlying");
                Log("Preparing for training flying");
                HearthState = Hearth.Capital;
                return false;
            }
            */

            if (Me.GetSkill(SkillLine.Riding).CurrentValue == 225)
                Log("Learning \"Artisan Riding\"");
            else
                Log("Learning \"Master Riding\"");

            /*
            var _npcId = (Me.IsHorde)
                                ? 44919
                                : 43769;
            var _npc = GetNpc(_npcId);
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            */

            var _npcId = (int)_trainerDataEntry[0];
            var _npc = GetNpc(_npcId);
            var _npcLocation = (Vector3)_trainerDataEntry[4];

            var _preSkill = Me.GetSkill(SkillLine.Riding).CurrentValue;
            while (Me.GetSkill(SkillLine.Riding).CurrentValue == _preSkill && !breakOut)
            {
                DevLog("WHILE: TrainFlying");

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;

                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();

                    await Coroutine.Wait(5000, () => IsFrameVisible("ClassTrainerFrame"));
                    await ClickButton("ClassTrainerTrainButton");
                    await Coroutine.Wait(5000, () => Me.GetSkill(SkillLine.Riding).CurrentValue != _preSkill);
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private int GetSpecIndex()
        {
            var _classProfiles = Styx.CommonBot.CharacterManagement.CharacterManager.ClassProfiles
                        .Where(p => p.Class == Me.Class);
            var _currentSpec = Me.Specialization;
            var _currentClassProfile = _classProfiles.FirstOrDefault(p => p.GetSpec() == _currentSpec);
            return _classProfiles.ToList().IndexOf(_currentClassProfile) + 1;

        }

        private async Task<bool> HandleSpec()
        {
            if (Me.IsFlying)
                return false;

            if (Me.Mounted)
                return false;

            if (!Me.IsAlive)
                return false;

            var _setting = Settings.Quest_Specialisation;
            if (_setting == 0)
                return false;

            if (Me.Level < 10)
                return false;

            if (_spec == _setting)
                return false;

            Log("Changing specialization");
            while (GetSpecIndex() != _setting && !breakOut)
            {
                DevLog("WHILE: ChangeSpec");

                if (!Me.IsCasting)
                    Lua.DoString("SetSpecialization(" + _setting + ")");
                await Coroutine.Wait(12000, () => !Me.IsCasting);
                await Sleep();
                await Coroutine.Yield();
            }

            var _currentSpec = Me.Specialization;
            var _currentClassProfileSpec = CharacterManager.CurrentClassProfile.GetSpec();
            var _newClassProfile = CharacterManager.ClassProfiles
                                                        .FirstOrDefault(p => p.GetSpec() == _currentSpec);

            if (_currentClassProfileSpec != _currentSpec)
                CharacterManager.SetClassProfile(_newClassProfile);

            _spec = _setting;
            _checkedHeirlooms = false;
            return true;
        }

        private bool IsRankQuest(PlayerQuest _quest)
        {
            var _isRankQuest = _legionRankQuestData.Any(e => (uint)(int)e[4] == _quest.Id);

            return _isRankQuest;
        }

        private async Task<bool> HandleAbandonQuest()
        {
            if (!Settings.Quest_AbandonQuests)
                return false;

            var _questCount = QuestLog.Instance.GetAllQuests().Count();
            if (_questCount < 22)
                return false;

            var _levelThreshold = (Settings.Quest_QuestingPack.ToString() == "New")
                                            ? 3
                                            : 5;

            var _lowLevelQuest = QuestLog.Instance.GetAllQuests().FirstOrDefault(q => q.Level > 0 &&
                                                                                      q.Level <= Me.Level - _levelThreshold &&
                                                                                      !IsRankQuest(q));
            if (_lowLevelQuest == null)
                return false;

            Log("Abandoning low level quest");
            while (_lowLevelQuest != null && !breakOut)
            {
                DevLog("WHILE: HandleAbandonQuest");
                Log("     {0} (Id: {1})", _lowLevelQuest.Name, _lowLevelQuest.Id);

                QuestLog.Instance.AbandonQuestById(_lowLevelQuest.Id);
                await Coroutine.Wait(3 * 1000, () => !QuestLog.Instance.ContainsQuest(_lowLevelQuest.Id));

                _lowLevelQuest = QuestLog.Instance.GetAllQuests().FirstOrDefault(q => q.Level > 0 &&
                                                                                      q.Level <= Me.Level - _levelThreshold);
                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleSorting()
        {
            if (!Settings.Bag_Sort)
                return false;

            Log("Sorting bags");
            Lua.DoString("SortBags()");

            return true;
        }

        private bool HasToy(WoWItem _item)
        {
            var _itemId = (int)_item.Entry;
            if (!_toys.ContainsKey(_itemId))
                return false;

            var _hasToy = _toys[_itemId];
            if (_hasToy)
                return true;

            return false;
        }

        private bool CanLearnPet(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _name = _item.Name;
            if (!_pets.ContainsKey(_name))
                return false;

            var _amount = _pets[_name];
            return _amount < 3;
        }

        private bool IsToy(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _itemId = (int)_item.Entry;
            if (_toys.ContainsKey(_itemId))
                return true;

            return false;
        }

        private bool IsPet(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _name = _item.Name;
            if (!_pets.ContainsKey(_name))
                return false;

            var _effects = _item.Effects;
            if (_effects == null || _effects.Count() == 0)
                return false;

            var _hasSummonEffect = _effects.Any(e => e != null &&
                                                     e.Spell != null &&
                                                     e.Spell.SpellEffects.Count() > 0 &&
                                                     e.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.Summon));

            var _hasLearnSpellEffect = _effects.Any(e => e != null &&
                                                         e.Spell != null &&
                                                         e.Spell.SpellEffects.Count() > 0 &&
                                                         e.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.LearnSpell));

            if (_hasSummonEffect && _hasLearnSpellEffect)
                return true;

            return false;
        }

        private Dictionary<string, int> GetPets()
        {
            var _lua = @"
	            local data = {{}}

	            local addToTable = function(...)
		            local row = {}
		            for k, v in pairs({...}) do
			            table.insert(row, tostring(v) )
		            end
		            table.insert(data, table.concat(row, "";"") )
	            end

	            C_PetJournal.SetFilterChecked(LE_PET_JOURNAL_FILTER_COLLECTED, true)
	            C_PetJournal.SetFilterChecked(LE_PET_JOURNAL_FILTER_NOT_COLLECTED, true)
	            C_PetJournal.SetAllPetSourcesChecked(true)
	            C_PetJournal.SetAllPetTypesChecked(true)

	            for i = 1, C_PetJournal.GetNumPets() do
		            local _ , _, hasPet, _, _, _, _, name = C_PetJournal.GetPetInfoByIndex(i)
		            addToTable(name, hasPet)
	            end

	            return unpack(data)
            ";
            var _data = Lua.GetReturnValues(_lua);
            _data.RemoveAt(0);

            var _pets = new Dictionary<string, int>();
            foreach (var _entry in _data)
            {
                var _delimiter = ';';
                var _parts = _entry.Split(_delimiter);

                var _name = _parts[0];
                var _hasPet = Convert.ToBoolean(_parts[1]);

                if (!_pets.ContainsKey(_name))
                    _pets.Add(_name, 0);

                if (_hasPet)
                    _pets[_name]++;
            }

            return _pets;
        }

        private Dictionary<int, bool> GetToys()
        {
            var _lua = @"
	            local data = {{}}

	            local addToTable = function(...)
		            local row = {}
		            for k, v in pairs({...}) do
			            table.insert(row, tostring(v) )
		            end
		            table.insert(data, table.concat(row, "";"") )
	            end

	            C_ToyBox.SetUncollectedShown(true)
	            C_ToyBox.SetCollectedShown(true)
	            C_ToyBox.SetAllSourceTypeFilters(true)

	            for i = 1, C_ToyBox.GetNumToys() do
		            local itemId = C_ToyBox.GetToyFromIndex(i)
		            if itemId ~= -1 then
			            local hasToy = PlayerHasToy(itemId)
			            addToTable(itemId, hasToy)
		            end
	            end

	            return unpack(data)
            ";
            var _data = Lua.GetReturnValues(_lua);
            _data.RemoveAt(0);

            var _toys = new Dictionary<int, bool>();
            foreach (var _entry in _data)
            {
                var _delimiter = ';';
                var _parts = _entry.Split(_delimiter);

                var _id =       Convert.ToInt32(_parts[0]);
                var _hasToy =   Convert.ToBoolean(_parts[1]);
                _toys.Add(_id, _hasToy);
            }

            return _toys;
            /*
            var _maxToys = Lua.GetReturnVal<int>("return C_ToyBox.GetNumToys()", 0);
            var _showAll = "C_ToyBox.SetUncollectedShown(true);" +
                           "C_ToyBox.SetCollectedShown(true);" +
                           "C_ToyBox.SetAllSourceTypeFilters(true)";

            Lua.DoString(_showAll);
            for (int i = 1; i <= _maxToys; i++)
            {
                var _return = Lua.GetReturnVal<int>($"return C_ToyBox.GetToyFromIndex({i - 1})", 0);
                if (_return != -1)
                {
                    var _hasToy = Lua.GetReturnVal<bool>($"return PlayerHasToy({_return})", 0);
                    _toys.Add(_return, _hasToy);
                }
            }
            */
        }

        private async Task<bool> HandleToyItems()
        {
            RuntimeLog(">>");

            if (_toys.Count() == 0)
                _toys = GetToys();

            RuntimeLog(">>");

            if (!Settings.Bag_LearnToys &&
                !Settings.Bag_RemoveToys)
                return false;

            if (await LearnToyItems())
                return true;

            if (await RemoveToyItems())
                return true;

            return false;
        }

        private async Task<bool> HandlePetItems()
        {
            if (!Settings.Bag_LearnPets)
                return false;

            if(_pets.Count() == 0)
                _pets = GetPets();

            var _item = Me.BagItems.FirstOrDefault(i => IsPet(i) &&
                                                        CanLearnPet(i));
            if (_item == null)
                return false;

            Log("Learning pets");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandlePetItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                if (await UseItem(_item))
                    await Coroutine.Wait(2 * 1000, () => GetItem((int)_item.Entry) == null);
                else
                    break;

                _item = Me.BagItems.FirstOrDefault(i => IsPet(i) &&
                                                        CanLearnPet(i));
                await Coroutine.Yield();
            }
            _pets = GetPets();

            await HandleSorting();
            return true;
        }

        private async Task<bool> LearnToyItems()
        {
            if (!Settings.Bag_LearnToys)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsToy(i) &&
                                                        !HasToy(i));
            if (_item == null)
                return false;

            Log("Learning toys");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: LearnToyItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                if (await UseItem(_item))
                    await Coroutine.Wait(2000, () => GetItem((int)_item.Entry) == null);
                else
                    break;

                _item = Me.BagItems.FirstOrDefault(i => IsToy(i) &&
                                                        !HasToy(i));
                await Coroutine.Yield();
            }
            _toys = GetToys();

            await HandleSorting();
            return true;
        }

        private bool StartsQuest(WoWItem _item)
        {
            if (_item == null)
                return false;

            if (_item.ItemInfo == null)
                return false;

            var _questId = _item.ItemInfo.BeginQuestId;
            if (_questId == 0)
                return false;

            return true;

            /*
            if (GetTooltipText((int)_item.Entry).Contains(_startsQuestText))
                return true;
                */
        }

        private string GetTooltipText(WoWItem _item)
        {
            if (_item == null)
                return null;

            if (!_item.IsValid)
                return null;

            var _lua = @"
                    local frame = CreateFrame('GameTooltip', 'tooltip', UIParent, 'GameTooltipTemplate')
                    local string = """"
                    local bag = " + (_item.BagIndex + 1) + @"
                    local slot = " + (_item.BagSlot + 1) + @"

                    frame:SetOwner(UIParent, 'ANCHOR_NONE')
                    frame:SetBagItem(bag, slot)

                    for i = 1, tooltip:NumLines() do
                    	local text = _G['tooltipTextLeft'..i]:GetText()
                    	if text ~= nil then
                    		string = string .. text .. ""\\n""
                    	end
                    end

                    frame:Hide()
                    return string";
            var _text = Lua.GetReturnVal<string>(_lua, 0);

            return _text;
        }

        private async Task<bool> HandleQuestItems()
        {
            if (!Settings.Bag_RemoveQuestItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsLowLevelQuestItem(i));
            if (_item == null)
                return false;

            Log("Removing low level quest item");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleQuestItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                _item.PickUp();
                Lua.DoString("DeleteCursorItem();");
                await Coroutine.Wait(2000, () => GetItem((int)_item.Entry) == null);

                _item = Me.BagItems.FirstOrDefault(i => IsLowLevelQuestItem(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> RemoveToyItems()
        {
            if (!Settings.Bag_RemoveToys)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsToy(i) &&
                                                        HasToy(i));
            if (_item == null)
                return false;

            Log("Removing already known toy items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: RemoveToyItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                _item.PickUp();
                Lua.DoString("DeleteCursorItem();");
                await Coroutine.Wait(2000, () => GetItem((int)_item.Entry) == null);

                _item = Me.BagItems.FirstOrDefault(i => IsToy(i) &&
                                                        HasToy(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleCombinableItems()
        {
            if (!Settings.Bag_CombineItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsCombinable(i));
            if (_item == null)
                return false;

            if (Me.Mounted)
                return false;

            Log("Combining items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: CombineItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Coroutine.Wait(2000, () => GetItem((int)(_item.Entry)) == null ||
                                                         _item.CooldownTimeLeft.TotalSeconds == 0);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IsCombinable(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private bool IsArtifactRelic(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null)
                return false;

            var _gemClass = _itemInfo.GemClass;
            if (_gemClass == WoWItemGemClass.ArtifactRelic)
                return true;

            return false;
        }

        private bool CorrectAssigned(WoWItem _item)
        {
            var _isProtected = ProtectedItemsManager.Contains(_item.Entry);
            var _shouldBeProtected = Settings.Artifact_ProtectRelics;

            return _isProtected == _shouldBeProtected;
        }

        private async Task<bool> HandleArtifactRelics()
        {
            var _item = Me.BagItems.FirstOrDefault(i => IsArtifactRelic(i) &&
                                                        !CorrectAssigned(i) );
            if (_item == null)
                return false;

            var _setting = Settings.Artifact_ProtectRelics;
            Log("{0}rotecting artifact relics", (_setting)
                                                    ? "P"
                                                    : "Unp");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleArtifactRelics");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_setting)
                    ProtectedItemsManager.Add(_item.Entry);
                else
                    ProtectedItemsManager.Remove(_item.Entry);

                _item = Me.BagItems.FirstOrDefault(i => IsArtifactRelic(i) &&
                                                        !CorrectAssigned(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        /*
        private async Task<bool> HandleArtifactKnowledgeItems()
        {
            if (!Settings.Artifact_KnowledgeItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IncreasesArtifactKnowledge(i));
            if (_item == null)
                return false;

            Log("Using artifact knowledge items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleArtifactKnowledgeItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Coroutine.Wait(4 * 1000, () => GetItem((int)(_item.Entry)) == null);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IncreasesArtifactKnowledge(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }
        */

        private async Task<bool> HandleArtifactPowerItems()
        {
            if (!Settings.Artifact_PowerItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IncreasesArtifactPower(i));
            if (_item == null)
                return false;

            if (Me.Mounted)
                return false;

            Log("Using artifact power items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleArtifactPowerItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Coroutine.Wait(5 * 1000, () => GetItem((int)(_item.Entry)) == null);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IncreasesArtifactPower(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleOpenableItems()
        {
            if (!Settings.Bag_OpenItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsOpenable(i));
            if (_item == null)
                return false;

            if (Me.Mounted)
                return false;

            Log("Opening items");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleOpenableItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Loot();
                        await Coroutine.Wait(3 * 1000, () => GetItem((int)(_item.Entry)) == null ||
                                                             _item.CooldownTimeLeft.TotalSeconds == 0);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IsOpenable(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleMailItems()
        {
            if (!Settings.Bag_MailItems)
                return false;

            if (CharacterSettings.Instance.MailRecipient == "")
                return false;

            if (!IsFrameVisible("MailFrame"))
                return false;

            var _items = Me.BagItems.Where(i => i.ItemInfo.RequiredLevel < Me.Level - 10 &&
                                                ProtectedItemsManager.Contains(i.Entry) &&
                                                !i.IsSoulbound &&
                                                i.Entry != 5020);
            if (_items.Count() == 0)
                return false;

            Log("Mailing items");
            WoWItem[] _itemsToAttach = _items.Skip(0).Take(12).ToArray();
            var _preItemCount = (int)Me.BagItems.Sum(i => i.StackCount);
            var _lastItemsToAttach = new WoWItem[0];
            while (_items.Count() > 0 && !breakOut)
            {
                DevLog("WHILE: HandleMailItems");

                if (_itemsToAttach != _lastItemsToAttach)
                {
                    foreach (var _i in _itemsToAttach)
                    {
                        Log("     {0} (Id: {1})", _i.Name, _i.Entry);
                    }
                    _lastItemsToAttach = _itemsToAttach;
                }

                MailFrame.Instance.SendMail(CharacterSettings.Instance.MailRecipient, _itemsToAttach[0].Name, null, 0, _itemsToAttach);

                await Coroutine.Wait(2000, () => _preItemCount != (int)Me.BagItems.Sum(i => i.StackCount));
                await Sleep();

                _items = Me.BagItems.Where(i => i.IsValid &&
                                                i.ItemInfo.RequiredLevel < Me.Level - 10 &&
                                                ProtectedItemsManager.Contains(i.Entry) &&
                                                !i.IsSoulbound);

                _itemsToAttach = _items.Skip(0).Take(12).ToArray();
                _preItemCount = (int)Me.BagItems.Sum(i => i.StackCount);
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> UseItem(WoWItem _item, bool _alternativeMethod = false)
        {
            if (!_item.IsValid)
                return false;

            if (_item == null)
                return false;

            if (IsFrameVisible("MerchantFrame", true))
                return false;

            if (_alternativeMethod)
                _item.UseContainerItem();
            else
                _item.Use();
            return true;
        }

        private async Task<bool> HandleContainerItems()
        {
            if (!Settings.Bag_ManageContainers)
                return false;

            var _containers = Me.CarriedItems
                                .Where(c => c.IsValid &&
                                            c.ItemInfo.ContainerClass == WoWItemContainerClass.Bag)
                                .Cast<WoWContainer>();
            if (_containers.Count() == 0)
                return false;

            var _containersInBag = _containers
                                        .Where(c => c.BagSlot >= 0)
                                        .OrderByDescending(c => c.Slots);
            if (_containersInBag.Count() == 0)
                return false;

            var _equippedContainers = _containers
                                            .Where(c => c.BagSlot < 0)
                                            .OrderBy(c => c.Slots);

            if (_equippedContainers.Count() == 4 &&
                _equippedContainers.First().Slots >= _containersInBag.First().Slots)
                return false;

            var _autoEquip = CharacterSettings.Instance.AutoEquip;
            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = false;

            if (_equippedContainers.Count() < 4)
            {
                Log("Equipping container");

                var _item = _containersInBag.First();
                while (_equippedContainers.Count() < 4 && !breakOut)
                {
                    var _preEquippedContainersCount = _equippedContainers.Count();
                    DevLog("WHILE: HandleContainerItems");
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);

                    await Sleep();
                    _item.UseContainerItem();
                    if (!_item.IsSoulbound)
                    {
                        await Coroutine.Wait(2000, () => IsFrameVisible("StaticPopup1"));
                        while (IsFrameVisible("StaticPopup1"))
                        {
                            DevLog("WHILE: HandleContainerItems > Click");
                            await ClickButton("StaticPopup1Button1");
                            await Sleep();
                            await Coroutine.Yield();
                        }
                    }
                    await Coroutine.Wait(3 * 1000, () => _preEquippedContainersCount != _equippedContainers.Count());

                    if (_containersInBag.Count() == 0)
                        return false;
                    _item = _containersInBag.First();
                    await Coroutine.Yield();
                }
            }

            if (_equippedContainers.Count() == 0)
                return false;

            if (_equippedContainers.First().Slots < _containersInBag.First().Slots)
            {
                Log("Exchanging container");

                var _item = _equippedContainers.First();
                var _betterItem = _containersInBag.First();
                while (_equippedContainers.First().Slots < _containersInBag.First().Slots && !breakOut)
                {
                    DevLog("WHILE: ExchangeContainer", _equippedContainers.Count());
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    Log("          <=> {0} (Id: {1})", _betterItem.Name, _betterItem.Entry);

                    if (_betterItem.ToItem().BagIndex == _item.BagIndex)
                    {
                        DevLog("WHILE ExchangeContainer > ToBackpack");
                        _betterItem.PickUp();
                        Lua.DoString("PickupContainerItem(0, 1)");
                        await Coroutine.Wait(3 * 1000, () => _betterItem.ToItem().BagIndex != _item.BagIndex);
                    }

                    var _slot = 20 + _item.BagIndex;
                    Lua.DoString("PickupBagFromSlot(" + _slot + ")");
                    _betterItem.PickUp();

                    if (!_betterItem.IsSoulbound)
                    {
                        await Coroutine.Wait(2000, () => IsFrameVisible("StaticPopup1"));
                        while (IsFrameVisible("StaticPopup1") && !breakOut)
                        {
                            DevLog("WHILE: EquipContainer Click");
                            await ClickButton("StaticPopup1Button1");
                            await Coroutine.Yield();
                        }
                    }
                    await Coroutine.Wait(3 * 1000, () => _betterItem != _containersInBag.First() ||
                                                     _item != _equippedContainers.First());

                    _item = _equippedContainers.First();
                    _betterItem = _containersInBag.First();
                    await Coroutine.Yield();
                }
            }

            await HandleSorting();

            if (_autoEquip)
                CharacterSettings.Instance.AutoEquip = true;

            return true;
        }

        private List<uint> _draenorItemsData = new List<uint>
        {
            111349,     // Mining
		    111350,     // Herbalism
		    111364,     // First Aid
            111387,     // Cooking
            109586,     // Archaeology
            111351,     // Skinning
            111356,     // Fishing
	    };

        private async Task<bool> HandleDraenorItems()
        {
            var _item = Me.BagItems.FirstOrDefault(i => _draenorItemsData.Contains(i.Entry));
            if (_item == null)
                return false;

            Log("Learning Draenor item");
            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleDraenorItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                await Sleep();
                if (await UseItem(_item))
                {
                    if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                        if (await Coroutine.Wait(15 * 1000, () => !Me.IsCasting))
                            await Coroutine.Wait(2 * 1000, () => !HasItem((int)_item.Entry));
                }
                else
                    break;

                _item = Me.BagItems.FirstOrDefault(i => _draenorItemsData.Contains(i.Entry));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private bool IncreasesReputation(WoWItem _item)
        {
            if (!_item.IsValid)
                return false;

            var _effects = _item.Effects;
            if (_effects == null || _effects.Count() == 0)
                return false;

            var _increasesReputation = _effects.Any(e => e != null &&
                                                         e.Spell != null &&
                                                         e.Spell.SpellEffects.Count() > 0 &&
                                                         e.Spell.SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.Reputation));
            if (_increasesReputation)
                return true;

            return false;
        }

        private async Task<bool> HandleReputationItems()
        {
            if (!Settings.Bag_ReputationItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IncreasesReputation(i));
            if (_item == null)
                return false;

            Log("Using reputation items");
            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleReputationItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Coroutine.Wait(2 * 1000, () => GetItem((int)(_item.Entry)) == null ||
                                                             _item.CooldownTimeLeft.TotalSeconds == 0);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IncreasesReputation(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleAncientManaItems()
        {
            if (!Settings.Bag_AncientManaItems)
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IncreasesAncientMana(i));
            if (_item == null)
                return false;

            Log("Using ancient mana items");
            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleAncientManaItems");

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                if (_item.CooldownTimeLeft.TotalSeconds == 0)
                {
                    if (await UseItem(_item))
                    {
                        await Coroutine.Wait(2 * 1000, () => GetItem((int)(_item.Entry)) == null ||
                                                             _item.CooldownTimeLeft.TotalSeconds == 0);
                        await Sleep();
                    }
                    else
                        break;
                }

                _item = Me.BagItems.FirstOrDefault(i => IncreasesAncientMana(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        /*
        private class Armor
        {
            public Armor(WoWItemArmorClass _type,
                         List<WoWClass> _classes)
            {
                Type = _type;
                Classes = _classes;
            }

            public WoWItemArmorClass Type { get; }
            public List<WoWClass> Classes { get; }
        }

        private new List<Armor> _armorData = new List<Armor>()
        {
            new Armor(WoWItemArmorClass.Plate,      new List<WoWClass>() {
                                    WoWClass.Warrior,
                                    WoWClass.DeathKnight,
                                    WoWClass.Paladin} ),
            new Armor(WoWItemArmorClass.Mail,       new List<WoWClass>() {
                                    WoWClass.Hunter,
                                    WoWClass.Shaman } ),
            new Armor(WoWItemArmorClass.Leather,    new List<WoWClass>() {
                                    WoWClass.Druid,
                                    WoWClass.Rogue,
                                    WoWClass.Monk,
                                    WoWClass.DemonHunter } ),
            new Armor(WoWItemArmorClass.Cloth,      new List<WoWClass>() {
                                    WoWClass.Mage,
                                    WoWClass.Warlock,
                                    WoWClass.Priest } )
        };

        private WoWItemArmorClass GetArmorType()
        {
            var _armor = _armorData.FirstOrDefault(a => a.Classes.Contains(Me.Class));
            return _armor.Type;
        }
        */

        private bool IsMogItem(WoWItem _item)
        {
            if (_item == null)
                return false;

            if (!_item.IsValid)
                return false;

            if (_item.IsSoulbound)
                return false;

            var _itemQuality = (int)_item.Quality;
            if (_itemQuality < 2 || _itemQuality > 4)
                return false;

            var _maxQuality = (int)Settings.Bag_MogItems + 1;
            if (_itemQuality > _maxQuality)
                return false;

            var _itemInfo = _item.ItemInfo;
            if (_itemInfo == null)
                return false;

            var _requiredLevel = _itemInfo.RequiredLevel;
            if (Me.Level < _requiredLevel)
                return false;

            var _canEquip = Me.CanEquipItem(_item);
            if (!_canEquip)
                return false;

            /*
            var _itemArmorType = (int)(_itemInfo.ArmorClass);
            if (_itemArmorType < 1 || _itemArmorType > 4)
                return false;

            var _armorType = (int)GetArmorType();
            if(_itemArmorType > _armorType)
                return false;
            */

            return true;
        }

        private WoWItem GetItemInSlot(int _bag, int _slot)
        {
            var _result= Lua.GetReturnVal<string>($"return select(10, GetContainerItemInfo({_bag}, {_slot}))", 0);

            if (_result == "nil")
                return null;

            var _itemId = Convert.ToInt32(_result);
            var _item = GetItem(_itemId);
            return _item;
        }

        private async Task<bool> HandleMogItems()
        {
            if (Settings.Bag_MogItems.ToString() == "Disabled")
                return false;

            var _item = Me.BagItems.FirstOrDefault(i => IsMogItem(i));
            if (_item == null)
                return false;

            Log("Collecting mogrification templates");

            WoWItem _lastItem = null;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleMogItems");

                if (Me.Combat)
                    return false;

                if (_item != _lastItem)
                {
                    Log("     {0} (Id: {1})", _item.Name, _item.Entry);
                    _lastItem = _item;
                }

                var _bag = _item.BagIndex + 1;
                var _slot = _item.BagSlot + 1;

                await Sleep();
                if (await UseItem(_item, true))
                    if(await Coroutine.Wait(2 * 1000, () => GetItem((int)_item.Entry) == null || IsFrameVisible("StaticPopup1")))
                    {
                        if (IsFrameVisible("StaticPopup1"))
                            await ClickButton("StaticPopup1Button1");

                        await Sleep(1000, 750);
                        Lua.DoString($"UseContainerItem({_bag},{_slot})");
                    }
                else
                    break;

                _item = Me.BagItems.FirstOrDefault(i => IsMogItem(i));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private class Heirloom
        {
            public Heirloom(int _id,
                            string _name,
                            int _maxLevel)
            {
                Id = _id;
                Name = _name;
                MaxLevel = _maxLevel;
            }

            public int Id { get; }
            public string Name { get; }
            public int MaxLevel { get; }

            public override string ToString()
            {
                return String.Format("Id: {0}, Name: {1}, MaxLevel: {2}",
                                        Id,
                                        Name,
                                        MaxLevel);
            }
        }

        private List<Heirloom> GetHeirlooms()
        {
            var _lua = @"
					local data = {{}}

					local addToTable = function(...)
						local row = {}
						for k, v in pairs({...}) do
							table.insert(row, tostring(v) )
						end
						table.insert(data, table.concat(row, "";"") )
					end

					C_Heirloom.SetUncollectedHeirloomFilter(false)
					C_Heirloom.SetCollectedHeirloomFilter(true)

					local numSources = C_PetJournal.GetNumPetSources();
					for i = 1, numSources do
						C_Heirloom.SetHeirloomSourceFilter(i, true);
					end

					local _, _, classID = UnitClass(""player"");
					local specIndex = GetSpecialization();
					local specID = GetSpecializationInfo(specIndex);
					C_Heirloom.SetClassAndSpecFilters(classID, specID);

					for i = 1, C_Heirloom.GetNumDisplayedHeirlooms() do
						local itemID = C_Heirloom.GetHeirloomItemIDFromDisplayedIndex(i);
						local name, _, _, _, _, _, _, _, _, maxLevel = C_Heirloom.GetHeirloomInfo(itemID)
						addToTable(itemID, name, maxLevel)
					end

					return unpack(data)
				";
            var _data = Lua.GetReturnValues(_lua);
            _data.RemoveAt(0);
            var _heirloomData = new List<Heirloom>();

            foreach (var _entry in _data)
            {
                var _delimiter = ';';
                var _parts = _entry.Split(_delimiter);

                if (_parts.Count() <= 1)
                    return _heirloomData;

                var _id =       Convert.ToInt32(_parts[0]);
                var _name =     _parts[1];
                var _maxLevel = Convert.ToInt32(_parts[2]);

                _heirloomData.Add(new Heirloom(_id, _name, _maxLevel));
            }

            return _heirloomData;
        }

        private async Task<bool> DestroyHeirlooms()
        {
            var _item = Me.BagItems.FirstOrDefault(i => i.Quality == WoWItemQuality.Heirloom);
            if (_item == null)
                return false;

            if (Me.Combat || Me.IsCasting || Me.IsInArena)
            {
                _timeAutoEquip = DateTime.Now;
                return false;
            }

            if (DeltaTime(_timeAutoEquip, "ms") <= 5000)
                return false;

            Log("Removing remaining heirlooms");
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleHeirlooms");
                Log("     {0} (Id: {1})", _item.Name, _item.Entry);

                _item.PickUp();
                Lua.DoString("DeleteCursorItem()");
                await Coroutine.Wait(2000, () => !_item.IsValid || _item == null);

                _item = Me.BagItems.FirstOrDefault(i => i.Quality == WoWItemQuality.Heirloom);
                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleHeirlooms()
        {
            if (!Settings.Bag_Heirlooms)
                return false;

            if (await DestroyHeirlooms())
                return false;

            if (_checkedHeirlooms)
                return false;

            var _maxLevel = (_accountExpansionLevel >= 6)
                                    ? 110
                                    : 100;
            if (Me.Level >= _maxLevel)
                return false;

            var _heirlooms = GetHeirlooms();
            if (_heirlooms.Count == 0)
                return false;

            Log("Creating heirlooms");
            for(var i = 1; i <= _heirlooms.Count(); i++)
            {
                var _heirloom = _heirlooms[i - 1];
                var _itemId = _heirloom.Id;
                var _itemName = _heirloom.Name;

                var _runsLeft = 2;
                while (_runsLeft > 0 && !breakOut)
                {
                    DevLog("WHILE: HandleHeirlooms");

                    Log("     {0} (Id: {1})", _itemName, _itemId);
                    Lua.DoString($"C_Heirloom.CreateHeirloom({_itemId})");

                    if (await Coroutine.Wait(3 * 1000, () => Me.IsCasting))
                        if (await Coroutine.Wait(3 * 1000, () => !Me.IsCasting))
                            await Coroutine.Wait(2 * 1000, () => GetItem(_itemId) != null);

                    var _item = GetItem(_itemId);
                    var _itemInfo = _item.ItemInfo;
                    if (_itemInfo != null)
                    {
                        var _equipSlot = _itemInfo.EquipSlot;
                        if (_equipSlot != InventoryType.Trinket &&
                            _equipSlot != InventoryType.Weapon)
                            break;
                    }
                    _runsLeft--;

                    await Coroutine.Yield();
                }

                if (!_init || !Me.IsAlive)
                    return false;
            }

            CharacterSettings.Instance.AutoEquip = true;
            Log("Waiting for AutoEquip to equip heirlooms");

            _timeAutoEquip = DateTime.Now;
            _checkedHeirlooms = true;
            return true;
        }

        private async Task<bool> HandleItems()
        {
            if (await HandleMailItems())
                return true;

            RuntimeLog(">");

            if (await HandleHeirlooms())
                return true;

            RuntimeLog(">");

            if (_checkedItems)
                return false;

            RuntimeLog(">");

            if (await HandleContainerItems())
                return true;

            RuntimeLog(">");

            if (await HandleCombinableItems())
                return true;

            RuntimeLog(">");

            if (await HandleReputationItems())
                return true;

            RuntimeLog(">");

            if (await HandleAncientManaItems())
                return true;

            RuntimeLog(">");

            if (await HandleOpenableItems())
                return true;

            RuntimeLog(">");

            if (await HandleToyItems())
                return true;

            RuntimeLog(">");

            if (await HandlePetItems())
                return true;

            RuntimeLog(">");

            if (await HandleQuestItems())
                return true;

            RuntimeLog(">");

            if (await HandleDraenorItems())
                return true;

            RuntimeLog(">");

            if (await HandleReputationItems())
                return true;

            RuntimeLog(">");

            if (await HandleMogItems())
                return true;

            RuntimeLog(">");

            return false;
        }

        private int GetSkillValue(SkillLine _skill, bool _withBonus = false)
        {
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            if (!_withBonus)
                return _skillValue;

            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                    ? _skillValue + 15
                                    : _skillValue;
            return _skillValue;
        }

        private bool SkillNeedsTraining(SkillLine _skill, bool _archaeology = false)
        {
            if (!StringContains(Settings.GatheringProfessions_Professions, "OR", _skill) && !_archaeology)
                return false;

            var _skillMax = Me.GetSkill(_skill).MaxValue;
            int _stage = _skillMax / 75;
            if (_stage >= 8)
                return false;

            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            if (_skillValue == 0)
                return true;

            _skillValue = ((Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism) /*||
                           (Me.Race == WoWRace.Worgen && _skill == SkillLine.Skinning)*/)
                                    ? _skillValue + 15
                                    : _skillValue;

            if (_skillValue < (_stage - 1) * 75 + 50)
                return false;

            if (Me.Level < _trainLevelData[_stage] && !_archaeology)
                return false;

            return true;
        }

        private List<int> _trainLevelData = new List<int>
        {
            0,	    // 75
		    0,	    // 150
		    10,     // 225
		    25,     // 300 CLASSIC
		    40,     // 375 BC
		    55,     // 450 WOTLK
		    75,     // 525 CATA
		    80      // 600 MOP
	    };

        private void CreateTrainerData()
        {
            foreach (var _entry in _trainerPreData)
            {
                var _npcId = (int)_entry[0];
                var _skill = (SkillLine)_entry[1];
                var _minLevel = (_entry.Count() >= 3)
                                        ? (int)_entry[2]
                                        : 0;

                var _npcResult = Query.GetNpcById((uint)_npcId);
                var _mapId = (_entry.Count() >= 4)
                                    ? (int)_entry[3]
                                    : _npcResult.MapId;
                var _npcLocation = (_entry.Count() >= 5)
                                            ? (Vector3)_entry[4]
                                            : _npcResult.Location;
                var _factionId = (_entry.Count() >= 6)
                                        ? (uint)(int)_entry[5]
                                        : _npcResult.Faction;
                var _isHostile = (_entry.Count() >= 6)
                                        ? false
                                        : _npcResult.IsHostile;

                var _realEntry = new object[] {
                                        _npcId,
                                        _skill,
                                        _minLevel,
                                        _mapId,
                                        _npcLocation,
                                        _factionId};

                if(!_isHostile)
                    _trainerData.Add(_realEntry);
            }

            /*
            string _log = "\n";
            foreach (var _e in _trainerData)
            {
                _log += "new object[] {{";
                foreach (var _r in _e)
                    _log += $"{_r}, \t";
                // new object[] { 98222,   SkillLine.Herbalism,    80,     1,  new Vector3(-6657.887f, -2044.802f, -253.1613f),    "AH"},
                _log += "}}, \n";
            }
            DevLog(_log);
            */
        }

        /**/

        private object[] GetClosestTrainerEntry(SkillLine _skill)
        {
            var _entry = _trainerData.Where(e => _skill == (SkillLine)e[1] &&
                                                 Me.Level >= (int)e[2] &&
                                                 Me.MapId == (int)e[3] &&
                                                 (int)Me.GetReputationLevelWith((uint)e[5]) >= 3)
                                      .OrderBy(e => Me.Location.Distance((Vector3)e[4]))
                                      .FirstOrDefault();

            return _entry;
        }

        /*/

        private object[] GetClosestTrainerEntry(SkillLine _skill)
        {
            var _entry = _trainerData.Where(e => _skill == (SkillLine)e[1] &&
                                                 Me.Level >= (int)e[2] &&
                                                 Me.MapId == (int)e[3])
                                      .OrderBy(e => Me.Location.Distance((Vector3)e[4]))
                                      .FirstOrDefault();

            return _entry;
        }

        /**/

        private List<object[]> _trainerPreData = new List<object[]>()
        {
            // KALIMDOR
                // HORDE
                    // ORGRIMMAR
                        new object[] { 44919,   SkillLine.Riding},
                        new object[] { 47571,   SkillLine.Archaeology},
                        new object[] { 3357,    SkillLine.Mining},
                        new object[] { 46741,   SkillLine.Herbalism},
                    // THUNDERBLUFF
                        new object[] { 3013,    SkillLine.Herbalism},
                        new object[] { 3001,    SkillLine.Mining},
                // ALLIANCE
                    // DARKSHORE
                        new object[] { 43431,   SkillLine.Mining},
                // BOTH
                    // MOONGLADE
                        new object[] { 12025,   SkillLine.Herbalism,    80},
                    // UN'GORO CRATER
                        new object[] { 98222,   SkillLine.Herbalism,    80,     1,  new Vector3(-6657.887f, -2044.802f, -253.1613f),    35},
                    // TANARIS
                        new object[] { 8128,    SkillLine.Mining,       80},

            // EASTERN KINGDOMS
                // HORDE
                    // UNDERCITY
                        new object[] { 4598,    SkillLine.Mining},
                        new object[] { 4614,    SkillLine.Herbalism},
                // ALLIANCE
                    // STORMWIND
                        new object[] { 44238,   SkillLine.Archaeology},
                        new object[] { 43769,   SkillLine.Riding},
                        new object[] { 5513,    SkillLine.Mining},
                        new object[] { 5566,    SkillLine.Herbalism},
                    // IRONFORGE
                        new object[] { 4254,    SkillLine.Mining},
                        new object[] { 5137,    SkillLine.Herbalism},
                // BOTH
                    // THE CAPE OF STRANGLETHORN
                        new object[] { 908,     SkillLine.Herbalism},

            // OUTLAND
                // HORDE
                    // HELLFIRE PENINSULA
                        new object[] { 18747,   SkillLine.Mining},
                        new object[] { 18748,   SkillLine.Herbalism},
                        new object[] { 35093,   SkillLine.Riding},
                // ALLIANCE
                    // HELLFIRE PENINSULA
                        new object[] { 18779,   SkillLine.Mining},
                        new object[] { 18776,   SkillLine.Herbalism},
                        new object[] { 35100,   SkillLine.Riding},
                // BOTH
                    // SHATTRATH
                        // THE ALDORS
                            new object[] { 33682,   SkillLine.Mining},
                            new object[] { 33678,   SkillLine.Herbalism},
                        // THE SCRYERS
                            new object[] { 33617,   SkillLine.Mining},
                            new object[] { 33616,   SkillLine.Herbalism},

            // NORTHREND
                // HORDE
                    // BOREAN TUNDRA
                        new object[] { 35135,   SkillLine.Riding},
                        new object[] { 26976,   SkillLine.Mining},
                        new object[] { 26994,   SkillLine.Herbalism},
                // ALLIANCE
                    // BOREAN TUNDRA
                        new object[] { 35133,   SkillLine.Riding},
                        new object[] { 26999,   SkillLine.Mining},
                        new object[] { 26974,   SkillLine.Herbalism},
                    // HOWLING FJORD
                        new object[] { 26912,   SkillLine.Mining},
                        new object[] { 26910,   SkillLine.Herbalism},
                // BOTH
                    // SHOLAZAR BASIN
                        new object[] { 28746,   SkillLine.Riding},
                        new object[] { 98223,   SkillLine.Herbalism,  0,  571,    new Vector3(5858.911f, 4145.91f, -94.90401f),     35},
                    // DALARAN
                        new object[] { 31238,   SkillLine.Riding},
                        new object[] { 28698,   SkillLine.Mining},
                        new object[] { 28704,   SkillLine.Herbalism},

            // PANDARIA
                // HORDE
                    // THE JADE FOREST
                        new object[] { 66979,   SkillLine.Mining},
                        new object[] { 66980,   SkillLine.Herbalism},
                // ALLIANCE
                    // THE JADE FOREST
                        new object[] { 67024,   SkillLine.Mining},
                        new object[] { 67025,   SkillLine.Herbalism},
                // BOTH
                    // VALLEY OF THE FOUR WINDS
                        new object[] { 65877,   SkillLine.Herbalism},

            // BROKEN ISLES
                // BOTH
                    // DALARAN
                        new object[] { 93189,   SkillLine.Mining,    0,  1220,   new Vector3(-729.5382f,   4493.964f,  738.8925f),   35 },
                        new object[] { 92464,   SkillLine.Herbalism, 0,  1220,   new Vector3(-769.3635f,   4519.786f,  732.4017f),   35 }

            /* NOT IN HB DB > ADD MANUALLY ++++
                    // VAL'SHARAH
                        new object[] { 98135,   SkillLine.Herbalism},
                    // HIGHMOUNTAIN
                        new object[] { 93691,   SkillLine.Mining},
           */
        };

        private List<SkillLine> _gatheringProfessions = new List<SkillLine>
        {
            SkillLine.Herbalism,
            SkillLine.Mining,
          //  SkillLine.Skinning
        };

        private async Task<bool> HandleTrainSkill()
        {
            if (!Settings.GatheringProfessions_TrainSkill)
                return false;

            if (Me.IsNeutralPandaren)
                return false;

            if (TreeRoot.GoalText != null && StringContains(TreeRoot.GoalText, "OR", "Escort", "UseTransport"))
                return false;

            var _nodeId = (Me.IsHorde)
                            ? 536
                            : 582;
            var _checkNode = KnownFlightNodesManager.EnumerateKnownFlightNodes()
                                    .FirstOrDefault(n => n.Id == _nodeId);
            if (Me.Level <= 10 && !_inCapital && (!_inCapitalSurroundingZone || _checkNode == null) )
                return false;

            if (_checkedGatheringSkills)
                return false;

            if (ActuallyInCombat())
                return false;

            var _skill = _gatheringProfessions.FirstOrDefault(s => SkillNeedsTraining(s));
            if (_skill == 0)
            {
                _checkedGatheringSkills = true;
                return false;
            }

            /*
            if (!_inCapital)
            {
                if (!Settings.Hearth_Hearthstone ||
                    !Settings.Hearth_SetHearthstone ||
                    UnfavourableAreaSet())
                    return false;

                if (Me.Level < 10)
                    return false;

                DevLog("Hearth.Capital: TrainGatheringSkill");
                Log("Preparing for training {0}", _skill);
                HearthState = Hearth.Capital;
                return false;
            }


            var _trainerDataEntry = _trainerData.FirstOrDefault(e => (int)(e[0]) == Me.ZoneId &&
                                                                     (SkillLine)(e[1]) == _skill);
            var _npcId = (int)_trainerDataEntry[2];
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            */

            var _trainerDataEntry = GetClosestTrainerEntry(_skill);
            if (_trainerDataEntry == null)
                return false;

            var _npcId = (int)_trainerDataEntry[0];
            var _npcLocation = (Vector3)_trainerDataEntry[4];

            await TrainSkill(_skill, _npcId, _npcLocation);
            return true;
        }

        private SkillLine GetLowestNodeSkill()
        {
            var _availableSkills = _gatheringProfessions
                                        .Where(s => Me.GetSkill(s).CurrentValue > 0)
                                        .OrderBy(s => (Me.Race == WoWRace.Tauren && s == SkillLine.Herbalism)
                                                            ? Me.GetSkill(s).CurrentValue + 15
                                                            : Me.GetSkill(s).CurrentValue);

            if (_availableSkills.Count() == 0)
                return 0;

            return _availableSkills.First();
        }

        private List<int> _maxLevelData = new List<int>
        {
            60,	 // CLASSIC
			70,	 // BC
			80,  // WOTLK
			85,  // CATA
			90   // PANDA
	    };

        private double GetOptimumSkillValue()
        {
            double _optimumSkillValue = 0;
            if (Me.Level < 60)
            {
                _optimumSkillValue = (Me.Level + 1) * 5;
            }
            else if (Me.Level >= 90)
            {
                var _maxExpansionSkillLevel = (_accountExpansionLevel >= 6)
                                                    ? 800
                                                    : 700;
                _optimumSkillValue = Math.Min(_maxExpansionSkillLevel, 600 + ((Me.Level - 90) + 1) * 10);
            }
            else
            {
                var _nextMaxLevel = _maxLevelData.FirstOrDefault(l => Me.Level < l);
                var _lastMaxLevel = _maxLevelData
                                        .OrderByDescending(l => l)
                                        .FirstOrDefault(l => Me.Level >= l);
                var _index = _maxLevelData.IndexOf(_lastMaxLevel);

                var _startSkill = 300 + _index * 75;
                var _skillPerLevel = 75f / (_nextMaxLevel - _lastMaxLevel);
                var _deltaLevel = Me.Level - _lastMaxLevel;
                _optimumSkillValue = Math.Ceiling(_startSkill + (_deltaLevel + 1) * _skillPerLevel);
            }

            return _optimumSkillValue;
        }

        public string ToMD5Hash(string _string)

        {
            MD5 _md5 = MD5.Create();
            byte[] _stringBytes = Encoding.ASCII.GetBytes(_string);
            byte[] _hash = _md5.ComputeHash(_stringBytes);

            StringBuilder _stringBuilder = new StringBuilder();

            for (int i = 0; i < _hash.Length; i++)
            {
                _stringBuilder.Append(_hash[i].ToString("X2"));
            }

            return _stringBuilder.ToString();
        }

        private string SafeGuid(WoWGuid _guid)
        {
            var _md5 = ToMD5Hash(_guid.ToString());
            return String.Format("{0}{1}", _md5.Substring(0, 5),
                                           _md5.Substring(_md5.Length - 5));
        }

        private bool CanSafelyGather(WoWGameObject _node)
        {
            if (_node == null)
                return false;

            if (_node.IsHerb && Me.HasAura("Travel Form"))
                return true;

            return false;
        }

        private bool IsNodeValuable(WoWGameObject _node)
        {
            var _professions = Settings.GatheringProfessions_Professions.ToString();
            if ((_node.IsMineral && !_professions.Contains("Mining")) ||
                (_node.IsHerb && !_professions.Contains("Herbalism")) )
                return false;

            if (!_ignoredNodeGuids.ContainsKey(_node.Guid))
                _ignoredNodeGuids.Add(_node.Guid, 3);

            if (_ignoredNodeGuids[_node.Guid] == 0)
                return false;

            var _mobsNearNode = ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                                    .Where(u => u.IsValid &&
                                                u.IsAlive &&
                                                u.IsHostile &&
                                                u.Location.Distance(_node.Location) < 30 &&
                                                u.Level > Me.Level - 2);
            if (_mobsNearNode.Count() > 3 && !CanSafelyGather(_node))
                return false;

            var _mobsNearNodeClosest = _mobsNearNode
                                            .Where(u => u.Location.Distance(_node.Location) < 15);
            if (_mobsNearNodeClosest.Count() > 2 && !CanSafelyGather(_node))
                return false;

            var _correctLocation = (Me.IsOutdoors)
                                        ? _node.IsOutdoors
                                        : _node.IsIndoors;
            if (!_correctLocation)
                return false;

            var _skill = (_node.IsHerb)
                            ? SkillLine.Herbalism
                            : SkillLine.Mining;
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                ? _skillValue + 15
                                : _skillValue;
            int _greySkillLevel = -1;
            _node.GetDataSlot(GameObjectDataSlot.GreySkillLevel, out _greySkillLevel);
            if (_skillValue >= _greySkillLevel && !Settings.GatheringProfessions_GatherGrey)
            {
                DevLog("Node blacklisted");
                DevLog("     Name: {0} (Id: {1})", _node.Name, _node.Entry);
                DevLog("     Guid: {0}", SafeGuid(_node.Guid));
                DevLog("     SkillValue >= GreySkillvalue: {0} >= {1}", _skillValue, _greySkillLevel);

                _ignoredNodeGuids[_node.Guid] = 0;
                return false;
            }

            var _requiredSkillValue = _node.RequiredSkill;
            if (_skillValue < _requiredSkillValue)
            {
                /*
                DevLog("Node not (yet) valuable");
                DevLog("     Name: {0} (Id: {1})", _node.Name, _node.Entry);
                DevLog("     Guid: {0}", SafeGuid(_node.Guid));
                DevLog("     SkillValue < RequiredSkillvalue: {0} < {1}", _skillValue, _requiredSkillValue);
                */
                return false;
            }

            var _optimumSkillValue = GetOptimumSkillValue();
            if (!IsSkillMaxedOut(_skill) && _skillValue >= _optimumSkillValue)
            {
                DevLog("Node blacklisted");
                DevLog("     Name: {0} (Id: {1})", _node.Name, _node.Entry);
                DevLog("     Guid: {0}", SafeGuid(_node.Guid));
                DevLog("     SkillValue >= OptimumSkillvalue: {0} >= {1}", _skillValue, _optimumSkillValue);

                _ignoredNodeGuids[_node.Guid] = 0;
                return false;
            }

            return true;
        }

        /*
        private async Task<bool> HandlePickQuest(bool _notHandledByKOM = false)
        {
            return false;

            if(_notHandledByKOM)
            {
                if ((int)Settings.GatheringProfessions_HandleNodes >= 2)
                    return false;

                if (ActuallyInCombat())
                    return false;
            }

            var _questId = 38786;
            if (!HasQuest(_questId))
                return false;

            if (IsQuestCompleted(_questId))
                return false;

            var _itemId = 128767;
            var _questItem = Me.BagItems.FirstOrDefault(i => i.Entry == _itemId);
            if (_questItem == null)
                return false;

            if (_node == null)
                return false;

            if (!_node.WithinInteractRange)
                return false;

            if (!_node.IsMineral)
                return false;

            DevLog("Pick");
            _questItem.Use(_node.Guid);

            if (_notHandledByKOM)
            {
                if (await Coroutine.Wait(3 * 1000, () => Me.IsCasting))
                    if (await Coroutine.Wait(10 * 1000, () => !Me.IsCasting))
                        await Loot();
            }

            return true;
        }
        */

        private async Task<bool> HandleGetNode()
        {
            RuntimeLog(">>");

            if ((int)Settings.GatheringProfessions_HandleNodes <= 1)
                return false;

            RuntimeLog(">>");

            var _professions = Settings.GatheringProfessions_Professions.ToString();
            if (_professions == "None")
                return false;

            RuntimeLog(">>");

            if (IsHelperProfile())
                return false;

            if (ProfileIsArtifactQuest())
                return false;

            RuntimeLog(">>");

            if ((ActuallyInCombat() && !Me.Mounted))
                return false;

            if (DeltaTime(_timeInterrupted, "s") < 5 && !CanSafelyGather(_node))
                return false;

            RuntimeLog(">>");

            if (DeltaTime(_timeDied, "s") < DeltaTime(_timeUnghost, "s"))
                return false;

            RuntimeLog(">>");

            if (DeltaTime(_timeUnghost, "s") < 10 && Me.HealthPercent <= 90)
                return false;

            RuntimeLog(">>");

            if (_node == null)
                return false;

            RuntimeLog(">>");

            if (_nodeNpc != null)
                return false;

            RuntimeLog(">>");

            if (await HandleLoot())
                return false;

            if (_treasure != null && Settings.General_Treasures.ToString() == "PreferTreasuresToNodes")
                return false;

            if (_rareNpc != null && Settings.General_RareMobs.ToString() == "PreferRareMobsToNodes")
                return false;

            /*

            if ((ActuallyInCombat() && !Me.Mounted) || DeltaTime(_timeInterrupted, "s") < 5)
                return false;

                    RuntimeLog(">>"); */

            if (TreeRoot.GoalText != null && StringContains(TreeRoot.GoalText, "OR", "Escort", "UseTransport") )
                return false;

            RuntimeLog(">>");

            if (Me.HasAura("Resurrection Sickness"))
                return false;

            RuntimeLog(">>");

            if (Me.CurrentMap.ExpansionId == 5 &&
                !IsQuestDone((Me.IsHorde)
                                    ? 31766
                                    : 31733))
                return false;

            RuntimeLog(">>");

            if (breakOut)
                return false;

            RuntimeLog(">>");

            var _skill = (_node.IsHerb)
                              ? SkillLine.Herbalism
                              : SkillLine.Mining;
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                            ? _skillValue + 15
                            : _skillValue;
            var _skillMax = Me.GetSkill(_skill).MaxValue;
            var _skillMaxWithBonus = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                                ? _skillMax + 15
                                                : _skillMax;

            Log("Gathering {0}", (_skill == SkillLine.Herbalism)
                                    ? "Herb"
                                    : "Mineral");
            if (GetOptimumSkillValue() > _skillMaxWithBonus)
                Log("     OptimumSkillValue: {0} ({1})", _skillMaxWithBonus, GetOptimumSkillValue());
            else
                Log("     OptimumSkillValue: {0}", GetOptimumSkillValue());
            Log("     PreSkillValue: {0}", _skillValue);

            var _nodeName = _node.Name;
            var _nodeEntry = _node.Entry;
            var _nodeLocation = _node.Location;
            var _nodeInteractRange = _node.InteractRange;
            var _nodeGuid = _node.Guid;
            var _nodeIsHerb = _node.IsHerb;
            var _maxTime = GetMaxTime(GetPathDistance(_node));

            var _preGatheringItemCount = _gatheringItemCount;
            var _startGathering = DateTime.Now;
            var _switched = false;
            var _switchCount = 0;

            if (!_ignoredNodeGuids.ContainsKey(_nodeGuid))
                _ignoredNodeGuids.Add(_nodeGuid, 3);
            else
                _ignoredNodeGuids[_nodeGuid] = Math.Max(0, _ignoredNodeGuids[_nodeGuid] - 1);

            while (_gatheringItemCount == _preGatheringItemCount && !breakOut)
            {
                RuntimeLog("Start");

                if (_infoTimer.IsFinished || _switched)
                {
                    DevLog("WHILE: HandleGetNode");
                    DevLog("     Name: {0} (Id: {1})", _nodeName, _nodeEntry);
                    DevLog("     Location: {0}", _nodeLocation);
                    DevLog("     Guid: {0}", SafeGuid(_nodeGuid));
                    DevLog("     GatheringTries left: {0}", (!_ignoredNodeGuids.ContainsKey(_nodeGuid))
                                                                    ? " -/- "
                                                                    : $"{_ignoredNodeGuids[_nodeGuid]}");
                    DevLog("     Distance: {0}", Math.Round(Me.Location.Distance(_nodeLocation)));
                    DevLog("     MaxTime: {0}", _maxTime);
                    DevLog("     TimeLeft: {0}", Math.Ceiling(_maxTime - DeltaTime(_startGathering, "s")));

                    _infoTimer.Reset();
                    _switched = false;
                }

                if (_treasure != null && Settings.General_Treasures.ToString() == "PreferTreasuresToNodes")
                {
                    DevLog("--- Switching to treasure ---");
                    _ignoredNodeGuids[_nodeGuid] = 3;

                    _infoTimer.Reset();
                    return false;
                }

                if (_rareNpc != null && Settings.General_RareMobs.ToString() == "PreferRareMobsToNodes")
                {
                    DevLog("--- Switching to rare mob ---");
                    _ignoredNodeGuids[_nodeGuid] = 3;

                    _infoTimer.Reset();
                    return false;
                }

                RuntimeLog();

                if (_nodeRefreshTimer.IsFinished && _node != null)
                {
                    if (_nodeGuid != _node.Guid)
                    {
                        _switchCount++;
                        if (_switchCount > 1)
                        {
                            DevLog("     PreventLoop");
                            DevLog("     SwitchCount: {0}", _switchCount);
                            DevLog("     Blacklist Node: {0}", _nodeGuid);
                            _ignoredNodeGuids[_nodeGuid] = 0;
                        }

                        _nodeGuid = _node.Guid;
                        _nodeLocation = _node.Location;
                        _nodeInteractRange = _node.InteractRange;

                        _nodeName = _node.Name;
                        _nodeEntry = _node.Entry;
                        _nodeIsHerb = _node.IsHerb;

                        DevLog("--- Switching node ---");
                        _switched = true;
                    }

                    _nodeRefreshTimer.Reset();
                }

                /*
                if (Me.Location.Distance(_nodeLocation) < 50 && _node == null)
                {
                    DevLog("     Node == null");
                    _ignoredNodeGuids[_nodeGuid] = 0;
                    return false;
                }
				*/

                RuntimeLog();

                if (DeltaTime(_startGathering, "s") > _maxTime && !Me.IsCasting &&
                    (_node == null || !_node.WithinInteractRange))
                {
                    DevLog("WHILE: GatherNode");
                    DevLog("     MaxTime reached");
                    _ignoredNodeGuids[_nodeGuid] = 0;
                    return false;
                }

                RuntimeLog();

                if (Me.Location.Distance(_nodeLocation) > _nodeInteractRange)
                {
                    RuntimeLog();

                    if (!await MoveTo(_nodeLocation))
                    {
                        DevLog("     Moving failed");
                        _ignoredNodeGuids[_nodeGuid] = 0;
                        return false;
                    }

                    RuntimeLog();
                }
                else
                {
                    DevLog("     WithinInteractRange \nNode == null: {0} \nGuid: {1} \nGatheringTries left: {2}",
                                _node == null,
                                SafeGuid(_nodeGuid),
                                (!_ignoredNodeGuids.ContainsKey(_nodeGuid))
                                            ? " -/- "
                                            : $"{_ignoredNodeGuids[_nodeGuid]}");

                    if (_node == null)
                    {
                        _ignoredNodeGuids[_nodeGuid] = 0;
                        return false;
                    }

                    if (!Me.IsCasting)
                    {
                        if (Me.IsFlying)
                        {
                            if (!await CommonCoroutines.LandAndDismount(null, false, _nodeLocation, 8))
                                await CommonCoroutines.LandAndDismount(null, true, _nodeLocation, 8);
                        }
                        await CommonCoroutines.StopMoving();

                        if (_node != null)
                        {
                            // if (!await HandlePickQuest())
                             _node.Interact();

                            if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                            {
                                if (await Coroutine.Wait(10 * 1000, () => !Me.IsCasting &&
                                                                          (
                                                                            DeltaTime(_timeInterrupted, "s") < 5 ||
                                                                            DeltaTime(_timeSucceeded, "s") < 5
                                                                           )))
                                {
                                    if (DeltaTime(_timeSucceeded, "s") < 5)
                                    {
                                        if (await Loot())
                                        {
                                            await Coroutine.Wait(2 * 1000, () => _gatheringItemCount != _preGatheringItemCount);
                                            DevLog("     Succeeded");
                                            _ignoredNodeGuids[_nodeGuid] = 0;

                                            if (Me.Shapeshift == ShapeshiftForm.Travel && !Me.HasAura("Stag Form"))
                                                await Move("JF", 250, true);

                                            break;
                                        }
                                    }
                                    else
                                    {
                                        // _ignoredNodeGuids[_nodeGuid] += 0.5;
                                        DevLog("     Interrupted");

                                        if (CanSafelyGather(_node))
                                            DevLog("     CanSafelyGather");
                                        else
                                            break;
                                    }
                                }
                            }
                        }
                    }
                }

                RuntimeLog("Sum");
                await Coroutine.Yield();
            }

            if(!Me.IsAlive)
            {
                DevLog("     Died > Blacklisting");
                _ignoredNodeGuids[_nodeGuid] = 0;
            }

            if(_ignoredNodeGuids.ContainsKey(_nodeGuid))
                DevLog("     GatheringTries left: {0}", _ignoredNodeGuids[_nodeGuid]);

            _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                ? _skillValue + 15
                                : _skillValue;

            Log("     PostSkillValue: {0}", _skillValue);

            _lowestNodeSkill = GetLowestNodeSkill();
            _checkedGatheringSkills = false;

            return true;
        }

        private async Task<bool> HandleGraphicMode()
        {
            if (!Settings.Performance_LowGraphicMode &&
                _renderScale == Settings.Performance_DefaultRenderScale / 100 &&
                _maxFps == Settings.Performance_MaxFps &&
                _maxFpsBk == Settings.Performance_MaxFpsBk)
                return false;

            if (Settings.Performance_LowGraphicMode &&
                _renderScale == 0.1 &&
                _maxFps == 20 &&
                _maxFpsBk == 20)
                return false;

            if (!Settings.Performance_LowGraphicMode)
            {
                Log("Restoring graphic mode");
                LowGraphicMode(false);
            }
            else
            {
                Log("Enabling Low Graphic Mode");
                LowGraphicMode();
            }

            return true;
        }

        private async Task<bool> HandleGatheringSettings()
        {
            var _herbalism = Settings.GatheringProfessions_Professions.ToString().Contains("Herbalism");
            var _mining = Settings.GatheringProfessions_Professions.ToString().Contains("Mining");
            //   var _skinning = Settings.GatheringProfessions_Professions.ToString().Contains("Skinning");
            var _preCharacterSettings = CharacterSettings.Instance.GetSettings();
            var _preGatherbuddySettings = Bots.Gatherbuddy.GatherbuddySettings.Instance.GetSettings();

            RuntimeLog(">");

            CharacterSettings.Instance.LootMobs = ((int)Settings.GatheringProfessions_HarvestMobs > 0)
                                                        ? true
                                                        : CharacterSettings.Instance.LootMobs;
            /*
            CharacterSettings.Instance.SkinMobs = (_skinning)
                                                        ? true
                                                        : false; */

            RuntimeLog(">");

            var _setting = (int)Settings.GatheringProfessions_HandleNodes;
            if (_setting == 0 || _setting == 2 || IsHelperProfile())
            {
                CharacterSettings.Instance.HarvestHerbs = false;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherHerbs = false;
            }
            else
            {
                CharacterSettings.Instance.HarvestHerbs = true;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherHerbs = true;
            }

            RuntimeLog(">");

            if (_setting == 0 || _setting == 2 || IsHelperProfile())
            {
                CharacterSettings.Instance.HarvestMinerals = false;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherMinerals = false;
            }
            else
            {
                CharacterSettings.Instance.HarvestMinerals = true;
                Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherMinerals = true;
            }

            RuntimeLog(">");

            if (CharacterSettings.Instance.LootMobs         != (bool)_preCharacterSettings["LootMobs"]      ||
                CharacterSettings.Instance.HarvestHerbs     != (bool)_preCharacterSettings["HarvestHerbs"]  ||
                CharacterSettings.Instance.HarvestMinerals  != (bool)_preCharacterSettings["HarvestMinerals"])
                            CharacterSettings.Instance.Save();

            RuntimeLog(">");

            if (BotManager.Current.ToString() != "Gatherbuddy2")
            {
                if (Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherMinerals    != (bool)_preGatherbuddySettings["GatherMinerals"] ||
                    Bots.Gatherbuddy.GatherbuddySettings.Instance.GatherHerbs       != (bool)_preGatherbuddySettings["GatherHerbs"] )
                    Bots.Gatherbuddy.GatherbuddySettings.Instance.Save();
            }

            RuntimeLog(">");

            return true;
        }

        private List<object[]> _legionRankQuestData = new List<object[]>
        {
            /*
                                      Description               Skill,       Type,       TypeId, QuestId,
                                                                                                         TurnInId,
                                                                                                                 AcceptCondition,
                                                                                                                         RankHelper (WithoutCondition)
           */

            // MINING
                // LEYSTONE
                    // RANK 1
                        new object[] { "Leystone_1",            "Mining",    "Item",     124489, 38777,  93189 },
                        new object[] { "Leystone_1",            "Mining",    "Item",     124490, 38784,  93189 },
                        new object[] { "Leystone_1",            "Mining",    "Item",     124491, 38785,  93189 },
                    // RANK 2
                        new object[] { "Leystone_2",            "Mining",    "Npc",      93189,  38888,  93691,  new object[] { 38784, 38785, 38777 } },
                        new object[] { "Leystone_2",            "Mining",    "Npc",      93691,  38786,  93691,  38888 },            // PICK ON NODES
                        new object[] { "Leystone_2",            "Mining",    "Npc",      93691,  38787,  93691,  38786,  true },     // GOSSIP
                        new object[] { "Leystone_2",            "Mining",    "Item",     124492, 38789,  93691 },
                        new object[] { "Leystone_2",            "Mining",    "Item",     124493, 38790,  93691 },
                        new object[] { "Leystone_2",            "Mining",    "Item",     124494, 38791,  93691 },
                    // RANK 3
                        new object[] { "Leystone_3",            "Mining",    "Spawn",    93735,  38792,  93691 },
                        new object[] { "Leystone_3",            "Mining",    "Spawn",    93743,  38793,  93691 },
                        new object[] { "Leystone_3",            "Mining",    "Spawn",    93744,  38794,  93691 },
                // FELSLATE
                    // RANK 1
                        new object[] { "Felslate_1",            "Mining",    "Item",     124496, 38795,  93189 },
                        new object[] { "Felslate_1",            "Mining",    "Item",     124497, 38796,  93189 },
                        new object[] { "Felslate_1",            "Mining",    "Item",     124498, 38797,  93189 },
                    // RANK 2
                        new object[] { "Felslate_2",            "Mining",    "Npc",      93189,  38901,  92264, new object[] { 38795, 38796, 38797 } },
                        new object[] { "Felslate_2",            "Mining",    "Npc",      92264,  38798,  92264, 38901, true },      // GOSSIP
                        new object[] { "Felslate_2",            "Mining",    "Npc",      92264,  38799,  92264, 38798 },            // DUNGEON
                        new object[] { "Felslate_2",            "Mining",    "Item",     124499, 38800,  92264 },
                        new object[] { "Felslate_2",            "Mining",    "Item",     124500, 38801,  92264 },
                        new object[] { "Felslate_2",            "Mining",    "Item",     124501, 38802,  92264 },
                    // RANK 3
                        new object[] { "Felslate_3",            "Mining",    "Spawn",    93774,  38803,  92264, 38800, false },
                        new object[] { "Felslate_3",            "Mining",    "Spawn",    93776,  38804,  92264, 38801, false },
                        new object[] { "Felslate_3",            "Mining",    "Spawn",    93777,  38805,  92264, 38002, false },
                // INFERNAL BRIMSTONE
                    // RANK 1
                        new object[] { "Infernal Brimstone_1",  "Mining",    "Item",     124502, 38806,  93189 },
                    // RANK 2
                        new object[] { "Infernal Brimstone_2",  "Mining",    "Npc",      93189,  38807,  93189, new object[] { 38806, 38800, 38801, 38802, 38789, 38790, 38791 },
                                                                                                                true },    // ++++++
                    // RANK 3
                        new object[] { "Infernal Brimstone_3",  "Mining",    "Npc",      93189,  39790,  97360,  38807 },
                        new object[] { "Infernal Brimstone_3",  "Mining",    "Npc",      97360,  39763,  97360,  39790 },    // ++++++
                        new object[] { "Infernal Brimstone_3",  "Mining",    "Npc",      97360,  39817,  93189,  39763 },
                        new object[] { "Infernal Brimstone_3",  "Mining",    "Npc",      93189,  39830,  93189,  39817 },    // ++++++
/*
                // EMPYRIUM
                    // RANK 1
                        new object[] { "Empyrium_1",            "Mining",    "Item",     151860, 48034, 125341,     -1},
                        new object[] { "Empyrium_1",            "Mining",    "Item",     151863, 48037, 125341,     -1},
                    // RANK 2
                        new object[] { "Empyrium_2",            "Mining",    "Item",     151861, 48035, 125341,     -1},
                        new object[] { "Empyrium_2",            "Mining",    "Item",     151864, 48038, 125341,     -1},
                    // RANK 3
                        new object[] { "Empyrium_3",            "Mining",    "Item",     151862, 48036, 125341,     -1},
                        new object[] { "Empyrium_3",            "Mining",    "Item",     151865, 48039, 125341,     -1},
*/
            // HERBALISM
                // AETHRIL
                    // RANK 1
                        new object[] { "Aethril_1",             "Herbalism", "Item",     129117, 40013,  92464 },
                        new object[] { "Aethril_1",             "Herbalism", "Npc",      92464,  40014,  92464,  40013 },
                    // RANK 2
                        new object[] { "Aethril_2",             "Herbalism", "Item",     129135, 40015,  92464 },
                        new object[] { "Aethril_2",             "Herbalism", "Npc",      92464,  40016,  92464,  40015, true},       // STEMS
                    // RANK 3
                        new object[] { "Aethril_3",             "Herbalism", null,       0,      40017,  92464,  -1 },               // AUTO ACCEPTED
                // DREAMLEAF
                    // RANK 1
                        new object[] { "Dreamleaf_1",           "Herbalism", "Item",     129118, 40018,  92464 },
                        new object[] { "Dreamleaf_1",           "Herbalism", "Npc",      92464,  40019,  92464,  40018 },
                    // RANK 2
                        new object[] { "Dreamleaf_2",           "Herbalism", "Item",     129136, 40020,  98135 },
                        new object[] { "Dreamleaf_2",           "Herbalism", "Npc",      98135,  40021,  98135,  40020, true},      // SATYR
                    // RANK 3
                        new object[] { "Dreamleaf_3",           "Herbalism", "Item",     129141, 40022,  98135 },
                        new object[] { "Dreamleaf_3",           "Herbalism", "Npc",      98135,  40023,  98135,  40022, false },    // TUR'XORAN
                // FOXFLOWER
                    // RANK 1
                        new object[] { "Foxflower_1",           "Herbalism", "Item",     129119, 40024,  92464 },
                    // RANK 2
                        new object[] { "Foxflower_2",           "Herbalism", "Item",     129137, 40025,  92464 },
                        new object[] { "Foxflower_2",           "Herbalism", "Npc",      92464,  40026,  92464,  40025 },            // FOXTRAIL
                    // RANK 3
                        new object[] { "Foxflower_3",           "Herbalism", "Item",     129278, 40028,  92464 },
                // FJARNSKAGGL
                    // RANK 1
                        new object[] { "Fjarnskaggl_1",         "Herbalism", "Item",     129120, 40029,  92464 },
                    // RANK 2
                        new object[] { "Fjarnskaggl_2",         "Herbalism", "Item",     129138, 40030,  92464 },
                        new object[] { "Fjarnskaggl_2",         "Herbalism", "Npc",      92464,  40031,  92464,  40030, true },      // 3 ITEMS
                    // RANK 3
                        new object[] { "Fjarnskaggl_3",         "Herbalism", "Item",     129142, 40032,  92464 },
                        new object[] { "Fjarnskaggl_3",         "Herbalism", "Npc",      92464,  40033,  92464,  40032, false },    // TORGUS
                // STARLIGHT ROSE
                    // RANK 1
                        new object[] { "Starlight Rose_1",      "Herbalism", "Item",     129121, 40034,  92464 },
                        new object[] { "Starlight Rose_1",      "Herbalism", "Npc",      92464,  40035,  92464,  40034 },
                    // RANK 2
                        new object[] { "Starlight Rose_2",      "Herbalism", "Item",     129140, 40036,  92464 },
                        new object[] { "Starlight Rose_2",      "Herbalism", "Npc",      92464,  40037,  92464,  40036, true },    // CHEST
                    // RANK 3
                        new object[] { "Starlight Rose_3",      "Herbalism", "Item",     129143, 40038,  92464 },
                        new object[] { "Starlight Rose_3",      "Herbalism", "Npc",      92464,  40039,  92464,  40038, false },    // THARILLON
                // FELWORT
                    // RANK 1
                        new object[] { "Felwort_1",             "Herbalism", "Item",     129122, 40040,  92464 },
                    // RANK 2
                        new object[] { "Felwort_2",             "Herbalism", "Npc",      92464,  40041,  92464,  new object[] { 40040, 40014, 40019, 40024, 40029, 40035 },
                                                                                                                 true },    // ++++++
                    // RANK 3
                        new object[] { "Felwort_3",             "Herbalism", "Npc",      92464,  40042,  92464,  new object[] { 40041, 40016, 40021, 40031, 40037, 40026 } },    // +++++
/*
                // ASTRAL GLORY
                    // RANK 1
                        new object[] { "Astral Glory_1",        "Herbalism", "Item",     151856, 48027,  125346,     -1},
                    // RANK 2
                        new object[] { "Astral Glory_2",        "Herbalism", "Item",     151857, 48028,  125346,     -1},
                    // RANK 3
                        new object[] { "Astral Glory_3",        "Herbalism", "Item",     151858, 48029,  125346,     -1},
*/
        };

        /*
            /dump GetQuestID()
            /dump UnitGUID("target"):sub(-17, -12)
        */

        private Vector3 GetNpcLocation(int _npcId)
        {
            var _entry = _questNpcData.FirstOrDefault(e => _npcId == (int)e[0]);
            if (_entry == null)
                return Vector3.Zero;

            return (Vector3)_entry[1];
        }

        private List<object[]> _questNpcData = new List<object[]>()
        {
            new object[] { 93189, new Vector3(-729.5382f,   4493.964f,  738.8925f) },
            new object[] { 93691, new Vector3(2990.198f,    3794.458f,  1347.548f) },
            new object[] { 92264, new Vector3(1465.878f,    4993.25f,   182.8172f) },
            new object[] { 92464, new Vector3(-769.3635f,   4519.786f,  732.4017f) },
            new object[] { 98135, new Vector3(2327.461f,    6584.921f,  147.7281f) },
            new object[] { 97360, new Vector3(-830.2656f,   4497.752f,  696.7373f) },
        };

        private bool CanAcceptQuest(object[] _questEntry)
        {
                  RuntimeLog(">>>>");

            var _skill = (string)_questEntry[1];
            if (!Settings.GatheringProfessions_Professions.ToString().Contains(_skill))
                return false;

                    RuntimeLog(">>>>");

            var _type = (string)_questEntry[2];
            var _acceptCondition = _questEntry.Length < 7 ||
                                   IsQuestDone(_questEntry[6]);
            if (_type != "Item" && !_acceptCondition)
                return false;

                    RuntimeLog(">>>>");

            var _questId = (int)_questEntry[4];
            if (HasQuest(_questId) || IsQuestDone(_questId))
                return false;

                    RuntimeLog(">>>>");

            if (_ignoredQuests.ContainsKey(_questId) &&
                DeltaTime(_ignoredQuests[_questId], "m") < 120)
                return false;

                    RuntimeLog(">>>>");

            /*
             *  NOT RELIABLE AS THOSE VALUES RETURN INCORRECT VALUES
             *
            var _quest = Quest.FromId((uint)_questId);
            if (_quest == null || _quest.RequiredLevel > Me.Level)
            {
                if (_questId == 39134)
                    DevLog("{0} -- {1}", _quest == null, _quest.RequiredLevel);

                return false;
            }
            */

                    RuntimeLog(">>>>");

            if (_type == "Spawn")
            {
                var _npcId = (int)_questEntry[3];
                var _npc = GetNpc(_npcId);

                    RuntimeLog(">>>>");

                if (_npc == null) // || _npc.QuestGiverStatus != QuestGiverStatus.Available)
                    return false;
            }
            else if (_type == "Item")
            {
                var _itemId = (int)_questEntry[3];
                var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                            i.Entry == _itemId);
                if (_item == null)
                    return false;
            }

                    RuntimeLog(">>>>");

            return true;
        }

        private bool IsQuestCompleted(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                /*
                var _isDoneAndCompleted = _completedQuests.Any(id => id == (uint)_id);
                if (_isDoneAndCompleted)
                    return true;
                */

                var _isActiveAndCompleted = _allQuests.Any(q => q.Id == (uint)_id &&
                                                                q.IsCompleted);
                if (_isActiveAndCompleted)
                    return true;
            }

            return false;
        }

        private bool IsQuestDone(params object[] _ids)
        {
            var _list = new List<int>();

            foreach (var _o in _ids)
            {
                if (_o.GetType() == typeof(int))
                    _list.Add((int)_o);
                else
                {
                    foreach (var _e in (object[])_o)
                        _list.Add((int)_e);
                }
            }

            var _trueCount = 0;
            foreach (var _id in _list)
            {
                var _hasQuest = HasQuest(_id);
                var _isCompleted = _completedQuests.Any(id => id == _id);

                if (!_hasQuest && _isCompleted)
                    _trueCount++;
            }

            return _trueCount == _list.Count();
        }

        private static int[] StringToArray(string _string)
        {
            var _regex = new Regex("\\d+");
            var _result = _regex.Matches(_string);
            var _list = new List<int>();

            foreach (var _entry in _result)
                _list.Add(Convert.ToInt32(_entry.ToString()));

            return _list.ToArray();
        }

        private bool HasItem(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                var _hasItem = Me.BagItems.Any(i => i.IsValid &&
                                                    i.Entry == (uint)_id);
                if (_hasItem)
                    return true;
            }

            return false;
        }

        private bool HasQuest(params int[] _ids)
        {
            foreach (var _id in _ids)
            {
                var _hasQuest = _allQuests.Any(q => q.Id == (uint)_id);
                if (_hasQuest)
                    return true;
            }

            return false;
        }

        private WoWUnit _spawnNpc
        {
            get
            {
                var _entry = _legionRankQuestData
                                  .FirstOrDefault(e => (string)e[2] == "Spawn" &&
                                                       GetNpc((int)e[3]) != null &&
                                                       GetNpc((int)e[3]).QuestGiverStatus == QuestGiverStatus.Available);
                if (_entry != null)
                    return GetNpc((int)_entry[3]);

                var _rethu = ObjectManager.GetObjectsOfType<WoWUnit>()
                                    .FirstOrDefault(u => u.IsValid &&
                                                         u.Name.Contains("Rethu"));
                if (_rethu != null)
                    return _rethu;

                var _felslateRank3Mob = ObjectManager.GetObjectsOfType<WoWUnit>()
                                            .FirstOrDefault(u => u.IsValid &&
                                                                 StringContains(u.Name, "OR", "Ondri", "Lyrelle", "Rin'thissa"));
                if (_felslateRank3Mob != null)
                    return _felslateRank3Mob;

                return null;
            }
        }

        private async Task<bool> HandleAcceptQuest()
        {
            RuntimeLog(">>>");

            if (!Settings.GatheringProfessions_Professions.ToString().Contains("Mining") &&
                !Settings.GatheringProfessions_Professions.ToString().Contains("Herbalism"))
                return false;

            RuntimeLog(">>>");

            if (_checkedItems && _spawnNpc == null &&
                (_checkedQuests || IsHelperProfile()) )
                return false;

            RuntimeLog(">>>");

            var _questDataEntry = _legionRankQuestData
                                        .FirstOrDefault(e => CanAcceptQuest(e));

            RuntimeLog(">>>");

            if (_questDataEntry == null)
            {
                RuntimeLog(">>>");
                return false;
            }

            RuntimeLog(">>>");

            DevLog("{0}", String.Join(" - ", _questDataEntry) );

            var _skill = ((string)_questDataEntry[1] == "Mining")
                                ? SkillLine.Mining
                                : SkillLine.Herbalism;
            DevLog("     SkillValue: {0}", GetSkillValue(_skill));

            var _type = (string)_questDataEntry[2];
            var _typeId = (int)_questDataEntry[3];
            var _questId = (int)_questDataEntry[4];

            RuntimeLog(">>>");

            var _preAllCount = _allQuests.Count();
            if (!await AcceptQuest(_type, _typeId, _questId))
                return false;
            await Coroutine.Wait(7 * 1000, () => _allQuests.Count() != _preAllCount);

            RuntimeLog(">>>");

            return true;
        }

        private async Task<bool> UseDalaranHearthstone(Vector3 _location)
        {
            RuntimeLog(">>>");

            if (Me.OnTaxi)
                return false;

            RuntimeLog(">>>");

            if (Me.IsActuallyInCombat)
                return false;

            RuntimeLog(">>>");

            if (Me.ZoneId == 7502)
                return false;

            RuntimeLog(">>>");

            var _dalaranCenter = new Vector3(-842.3923f, 4463.632f, 735.8796f);
            var _isInDalaran = _location.Distance(_dalaranCenter) <= 200;
            if (!_isInDalaran)
                return false;

            RuntimeLog(">>>");

            var _dalaranHearthstone = GetItem(140192);
            var _cooldown = _dalaranHearthstone.CooldownTimeLeft.TotalSeconds;
            if (_cooldown > 0)
                return false;

            RuntimeLog(">>>");

            HearthState = Hearth.Dalaran;
            await UseHearthItem();
            HearthState = Hearth.Off;

            return true;
        }

        private async Task<bool> AcceptQuest(string _type, int _typeId, int _questId)
        {
            RuntimeLog(">>>>");

            if (IsFrameVisible("GossipFrame", true))
                GossipFrame.Instance.Hide();
            if (IsFrameVisible("QuestFrame", true))
                QuestFrame.Instance.Hide();

            RuntimeLog(">>>>");

            if (_type == "Npc" || _type == "Spawn")
            {
                var _quest = Quest.FromId((uint)_questId);
                var _npcId = _typeId;
                var _npcResult = Query.GetNpcById((uint)_npcId);
                var _npcLocation = (_npcResult == null)
                                        ? GetNpcLocation(_npcId)
                                        : _npcResult.Location;

                RuntimeLog(">>>>");

                var _npc = GetNpc(_npcId);
                if (_npc == null && _npcLocation == Vector3.Zero)
                    return false;

                RuntimeLog(">>>>");

                Log("Accepting quest");
                await Coroutine.Wait(3 * 1000, () => _quest != null);
                Log("     {0} (Id: {1})", (_quest != null)
                                                ? _quest.Name
                                                : "---", _questId);

                while (!HasQuest(_questId))
                {
                    DevLog("WHILE: AcceptQuest > Npc");
                    await UseDalaranHearthstone(_npcLocation);

                    if (!Me.IsAlive)
                        return false;

                    if (!_init)
                        return false;

                    RuntimeLog(">>>>");

                    _npc = GetNpc(_npcId);
                    if (_npc != null)
                        _npcLocation = _npc.Location;
                    if (_npc == null || !_npc.WithinInteractRange)
                    {
                        if (!await MoveTo(_npcLocation, true))
                        {
                            DevLog("PathGenerationFailed");
                            DevLog($"     Zone: {Me.RealZoneText} (ZoneId: {Me.ZoneId}) > {Me.MinimapZoneText}" + ((Me.SubZoneId != 0)
                                       ? $" (SubZoneId: {Me.SubZoneId})"
                                       : ""));
                            DevLog($"     Map: {Me.MapName} (MapId: {Me.MapId})");

                            return false;
                        }
                    }
                    else
                    {
                        await CommonCoroutines.StopMoving();

                        RuntimeLog(">>>>");

                        await Sleep();
                        _npc.Interact();

                        RuntimeLog(">>>>");

                        if (!await Coroutine.Wait(10 * 1000, () => IsFrameVisible("GossipFrame", true) || IsFrameVisible("QuestFrame")))
                        {
                            if (_ignoredQuests.ContainsKey(_questId))
                                _ignoredQuests[_questId] = DateTime.Now;
                            else
                                _ignoredQuests.Add(_questId, DateTime.Now);

                            DevLog("Gossip/QuestFrame not opened > Blacklist for 2h");
                            return false;
                        }

                        RuntimeLog(">>>>");

                        var _availableQuests = (IsFrameVisible("GossipFrame", true))
                                                    ? GossipFrame.Instance.AvailableQuests
                                                    : QuestFrame.Instance.AvailableQuests;

                        var _questEntry = _availableQuests.FirstOrDefault(e => e.Id == _questId);
                        if (_availableQuests.Count() >= 1)
                        {
                            if (_questEntry == null)
                            {
                                if (_ignoredQuests.ContainsKey(_questId))
                                    _ignoredQuests[_questId] = DateTime.Now;
                                else
                                    _ignoredQuests.Add(_questId, DateTime.Now);

                                var _frame = (IsFrameVisible("GossipFrame", true))
                                                    ? "GossipFrame"
                                                    : "QuestFrame";
                                DevLog($"Quest not available in {_frame} > Blacklist for 2h");
                                return false;
                            }
                            else
                            {
                                await Sleep();

                                RuntimeLog(">>>>");

                                if (IsFrameVisible("GossipFrame", true))
                                    GossipFrame.Instance.SelectAvailableQuest(_questEntry.Index);
                                else
                                    QuestFrame.Instance.SelectAvailableQuest(_questEntry.Index);
                            }
                        }

                        RuntimeLog(">>>>");

                        if (!await Coroutine.Wait(5 * 1000, () => IsFrameVisible("QuestFrame", true)))
                        {
                            if (_ignoredQuests.ContainsKey(_questId))
                                _ignoredQuests[_questId] = DateTime.Now;
                            else
                                _ignoredQuests.Add(_questId, DateTime.Now);

                            DevLog("QuestFrame not opened > Blacklist for 2h");
                            return false;
                        }
                        await Sleep();

                        RuntimeLog(">>>>");

                        QuestFrame.Instance.AcceptQuest();
                        await Coroutine.Wait(3 * 1000, () => HasQuest(_questId));
                        await Sleep();
                        QuestFrame.Instance.Hide();
                    }

                    await Coroutine.Yield();
                }
            }
            else if (_type == "Item")
            {
                var _itemId = _typeId;
                var _item = Me.BagItems.FirstOrDefault(i => i.IsValid &&
                                                            i.Entry == _itemId);

                Log("Accepting quest from {0} (Id: {1})", _item.Name, _item.Entry);
                Log("     QuestId: {0}", _questId);
                while (!HasQuest(_questId) && !breakOut)
                {
                    DevLog("WHILE: AcceptQuest > Item");

                    if (await UseItem(_item))
                    {
                        if (await Coroutine.Wait(3 * 1000, () => IsFrameVisible("QuestFrame")))
                        {
                            QuestFrame.Instance.AcceptQuest();
                            QuestFrame.Instance.Hide();
                            await Coroutine.Wait(3 * 1000, () => HasQuest(_questId));
                        }
                    }
                    else
                        break;

                    await Coroutine.Yield();
                }
            }

            RuntimeLog(">>>>");

            return true;
        }

        private async Task<bool> TurnInQuest(int _questId, int _npcId)
        {
            RuntimeLog(">>>>");

            if (!HasQuest(_questId))
                return false;

            RuntimeLog(">>>>");

            if (IsFrameVisible("GossipFrame"))
                GossipFrame.Instance.Hide();
            if (IsFrameVisible("QuestFrame"))
                QuestFrame.Instance.Hide();

            var _questName = QuestLog.Instance.GetAllQuests()
                                    .FirstOrDefault(q => q.Id == _questId).Name;
            var _npcResult = Query.GetNpcById((uint)_npcId);
            var _npcLocation = (_npcResult == null)
                                    ? GetNpcLocation(_npcId)
                                    : _npcResult.Location;
            var _npc = GetNpc(_npcId);

            RuntimeLog(">>>>");

            if (_npc == null && _npcLocation == Vector3.Zero)
                return false;

            RuntimeLog(">>>>");

            Log("Turning in quest");
            Log("     {0} (Id: {1})", _questName, _questId);

            while (HasQuest(_questId) && !breakOut)
            {
                DevLog("WHILE: TurnInQuest");
                await UseDalaranHearthstone(_npcLocation);

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;
                if (_npc == null || !_npc.WithinInteractRange)
                {
                    if (!await MoveTo(_npcLocation, true))
                    {
                        DevLog("PathGenerationFailed");
                        DevLog($"     Zone: {Me.RealZoneText} (ZoneId: {Me.ZoneId}) > {Me.MinimapZoneText}" + ((Me.SubZoneId != 0)
                                                                                                                       ? $" (SubZoneId: {Me.SubZoneId})"
                                                                                                                       : ""));
                        DevLog($"     Map: {Me.MapName} (MapId: {Me.MapId})");
                        return false;
                    }
                }
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();
                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("QuestFrame") || IsFrameVisible("GossipFrame"));

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _activeQuests = GossipFrame.Instance.ActiveQuests;
                        var _questEntry = _activeQuests.FirstOrDefault(e => e.Id == _questId);
                        await Sleep();
                        GossipFrame.Instance.SelectActiveQuest(_questEntry.Index);
                        await Coroutine.Wait(2000, () => IsFrameVisible("QuestFrame"));

                        if (!IsFrameVisible("QuestFrame"))
                        {
                            await Sleep();
                            GossipFrame.Instance.SelectActiveGossipQuest(_questEntry.Index);
                            await Coroutine.Wait(2000, () => IsFrameVisible("QuestFrame"));
                        }
                    }

                    while (IsFrameVisible("QuestFrame") && HasQuest(_questId) && !breakOut)
                    {
                        QuestFrame.Instance.SelectQuestReward(0);
                        await Sleep();
                        QuestFrame.Instance.CompleteQuest();
                        await Coroutine.Wait(2000, () => !HasQuest(_questId));
                        await Coroutine.Yield();
                    }
                    QuestFrame.Instance.Hide();
                }

                await Coroutine.Yield();
            }
            await Coroutine.Sleep(2000);

            return true;
        }

        private string PrintArray(object[] _array)
        {
            return String.Join(" - ", _array);
        }

        private async Task<bool> HandleTurnInQuest()
        {
            RuntimeLog(">>>");

            var _setting = Settings.GatheringProfessions_Professions.ToString();
            if (_setting == "None")
                return false;

            RuntimeLog(">>>");

            if (BotManager.Current.ToString() == "Dungeonbuddy")
                return false;

            RuntimeLog(">>>");

            if (IsHelperProfile())
                return false;

            RuntimeLog(">>>");

            if (_checkedQuests)
                return false;

            if (Me.IsInInstance)
                return false;

            RuntimeLog(">>>");

            var _questDataEntry = _legionRankQuestData.FirstOrDefault(e => _setting.Contains((string)e[1]) &&
                                                                           IsQuestCompleted((int)(e[4])));
            if (_questDataEntry == null)
                return false;

            RuntimeLog(">>>");

            DevLog("     {0}", PrintArray(_questDataEntry));

            RuntimeLog(">>>");

            var _questId = (int)_questDataEntry[4];
            var _npcId = (int)_questDataEntry[5];

            RuntimeLog(">>>");

            var _preCompletedCount = _completedQuests.Count();
            if (!await TurnInQuest(_questId, _npcId))
                return false;
            await Coroutine.Wait(7 * 1000, () => _completedQuests.Count() != _preCompletedCount);

            return true;
        }

        private bool NeedsRankHelper(object[] _questDataEntry)
        {
            if (_questDataEntry.Length < 8)
                return false;

            var _questId = (int)_questDataEntry[4];
            if (!HasQuest(_questId))
                return false;

            if (IsQuestCompleted(_questId))
                return false;

            var _withoutCondition = (bool)_questDataEntry[7];
            if (!_withoutCondition && !GetCondition(_questId.ToString()) )
                return false;

            var _skill = (string)_questDataEntry[1];
            var _setting = Settings.GatheringProfessions_Professions.ToString();
            if (!_setting.Contains(_skill))
                return false;

            return true;
        }

        private bool ProfileIsArtifactQuest()
        {
            var _profile = ProfileManager.XmlLocation;
            if (_profile == null)
                return false;

            if (_profile.Contains("Broken Isles\\Artifact Quests"))
                return true;

            return false;
        }

        private bool IsHelperProfile(string _type = "Both")
        {
            var _profile = ProfileManager.XmlLocation;
            if (_profile == null)
                return false;

            switch(_type)
            {
                case "Rank":
                    return _profile.Contains("RanklHelper.xml");
                    break;
                case "ClassHall":
                    return _profile.Contains("ClassHallHelper.xml");
                    break;
                default:
                    return StringContains(_profile, "OR", "RankHelper.xml", "ClassHallHelper.xml");
                    break;
            }

            return false;
        }

        private async Task<bool> HandleRankHelper()
        {
            RuntimeLog(">>>");

            if (_checkedQuests)
                return false;

            var _questDataEntry = _legionRankQuestData.FirstOrDefault(e => NeedsRankHelper(e));
            if (_questDataEntry == null)
                return false;

            RuntimeLog(">>>");

            if (IsHelperProfile())
                return false;

            RuntimeLog(">>>");

            var _profile = ProfileManager.XmlLocation;
            KOMApi._lastProfile = _profile;
            KOMApi._lastBotBaseName = BotManager.Current.Name;

            var _questId = (int)_questDataEntry[4];
            var _quest = Quest.FromId((uint)_questId);
            var _rankHelperProfile = $"{Application.StartupPath}\\..\\Plugins\\KeepOnMoving\\RankHelper.xml";

            DevLog("{0}", PrintArray(_questDataEntry));
            Log("Loading RankHelper");
            await Coroutine.Wait(2 * 1000, () => _quest != null);
            Log("     {0} (Id: {1})", (_quest != null)
                                            ? _quest.Name
                                            : "---", _questId);

            KOMApi.ChangeBotBase("Questing", _rankHelperProfile);
            await Coroutine.Wait(30 * 1000, () => IsHelperProfile());

            return true;
        }

        private WoWAreaTrigger _foxTrail
        {
            get
            {
                var _trigger = ObjectManager.GetObjectsOfType<WoWAreaTrigger>()
                                    .Where(t => t.Entry == 9756 &&
                                                t.IsValid)
                                    .OrderBy(t => t.Distance)
                                    .FirstOrDefault();
                return _trigger;
            }
        }

        private WoWUnit _fox
        {
            get
            {
               var _fox = ObjectManager.GetObjectsOfType<WoWUnit>()
                                .Where(u => u.Entry == 98235 &&
                                            u.IsValid)
                                .OrderBy(u => u.Distance)
                                .FirstOrDefault();
                return _fox;
            }
        }

        private string PrintVector(Vector3 _vector)
        {
            return String.Format("new Vector3({0}f, {1}f, {2}f)",
                _vector.X,
                _vector.Y,
                _vector.Z);
        }

        private async Task<bool> HandleFoxTrail()
        {
            if (_foxTrail == null && _fox == null)
                return false;

            Log("Gathering foxflower from trail");
            var _trailString = "var _locations = new List<Vector3>()\n{{\n";

            var _lastLocation = Me.Location;
            while((_foxTrail != null || _fox != null) && !breakOut)
            {
                var _location = (_foxTrail != null)
                                        ? _foxTrail.Location
                                        : (_fox != null)
                                                ? _fox.Location
                                                : Me.Location;

                if (_lastLocation != _location)
                {
                    _trailString += $"     {PrintVector(_location)},\n";
                    _lastLocation = _location;
                }

                await MoveTo(_location);
                await Coroutine.Yield();
            }
            _trailString += "}};";
            DevLog($"\n{_trailString}");

            return true;
        }

        private async Task<bool> HandleStarlightRoseDust()
        {
            var _professions = Settings.GatheringProfessions_Professions.ToString();
            if (!_professions.Contains("Herbalism"))
                return false;

            var _questId = 40039;
            if (IsQuestCompleted(_questId))
                return false;

            uint _itemId = 129158;
            var _isProtected = ProtectedItemsManager.Contains(_itemId);
            if (_isProtected)
                return false;

            ProtectedItemsManager.Add(_itemId);

            return true;
        }

        private async Task<bool> HandleLegionRanks()
        {
            RuntimeLog(">>");

            if (!Settings.GatheringProfessions_Ranks)
                return false;

            RuntimeLog(">>");

            await HandleStarlightRoseDust();

            RuntimeLog(">>");

            /*
            if (await HandlePickQuest(true))
                return true;
            */

            RuntimeLog(">>");

            while(true)
            {
                RuntimeLog(">>");

                if (!Me.IsAlive)
                    break;

                RuntimeLog(">>");

                if (!_init)
                    break;

                RuntimeLog(">>");

                if (Me.Combat && Me.HealthPercent <= 75 && !Me.Mounted && _spawnNpc == null)
                    break;

                RuntimeLog(">>");

                if (_knownNodesCount == 0 && !Flightor.CanFly && _spawnNpc == null)
                {
                    if (_taxiWarningTimer.IsFinished)
                    {
                        DevLog("Can't handle quests as we can neither fly nor have any taxi node available");
                        _taxiWarningTimer.Reset();
                    }
                    break;
                }

                RuntimeLog(">>");

                var _canAccept = await HandleAcceptQuest();

                RuntimeLog(">>");

                var _canTurnIn = await HandleTurnInQuest();

                RuntimeLog(">>");

                var _needsRankHelper = await HandleRankHelper();

                if (_canAccept || _canTurnIn || _needsRankHelper)
                {
                    DevLog("WHILE: HandleLegionRanks");
                    DevLog("     CanAccept: {0}", _canAccept);
                    DevLog("     CanTurnIn: {0}", _canTurnIn);
                    DevLog("     NeedsRankHelper: {0}", _needsRankHelper);

                    PrintRankInfo();
                }
                else
                    break;

				await Coroutine.Yield();
            }

            RuntimeLog(">>");

            return false;
        }

        private List<uint> _nodeNpcIds = new List<uint>()
        {
            98232,           // WITHERED HUNGERER   (AETHRIL, AZSUNA)
            98233,           // WITHERED HUNGERER   (STARLIGHT ROSE, SURAMAR)
            98234,           // NIGHTMARE CREEPER   (DREAMLEAF, VAL'SHARAH)
            104895,          // FELSLATE BASILISK
            104877           // LEYSTONE BASILISK
        };

        private List<int> _brimstoneQuestIds = new List<int>()
        {
            41208,
            41209,
            41210,
            41482,
            41483,
            41484,
            41486,
            41487,
            41488,
            41489,
            41490,
            41491,
            41492,
            41493
        };

        private bool CanAttackBrimstoneDestroyer()
        {
            var _professions = Settings.GatheringProfessions_Professions.ToString();
            var _harvestMobs = (int)Settings.GatheringProfessions_HarvestMobs;
            var _hasBrimstoneQuest = _brimstoneQuestIds.Any(id => HasQuest(id));

            return _professions.Contains("Mining") && _hasBrimstoneQuest &&
                   _harvestMobs != 0 && _harvestMobs != 2;
        }

        private bool IsRareOfInterest(WoWUnit _unit)
        {
            if (Settings.General_RareMobs.ToString() == "Disabled")
                return false;

            if (!_unit.IsValid)
                return false;

            var _isRare = (_unit.Classification == WoWUnitClassificationType.Rare);
            if (!_isRare)
                return false;

            var _isBlackspotted = BlackspotManager.IsBlackspotted(_unit.Location);
            if (_isBlackspotted)
                return false;

            /*
            if (_unit.IsIndoors)
                return false;
                */

            var _cannotReach = (Navigator.LookupPathInfo(_unit).Navigability != PathNavigability.Navigable);
            if (_cannotReach)
                return false;

            return true;
       }

        private WoWUnit _rareNpc
        {
            get
            {
                if (IsHelperProfile())
                    return null;

                var _units = ObjectManager.GetObjectsOfType<WoWUnit>()
                                .Where(u => IsRareOfInterest(u) &&
                                            (
                                                !_ignoredGeneralGuids.ContainsKey(u.Guid) ||
                                                DeltaTime(_ignoredGeneralGuids[u.Guid], "m") >= 60
                                            ) &&
                                            u.IsValid &&
                                            (
                                                (
                                                    u.IsAlive &&
                                                    u.Attackable
                                                ) ||
                                                (
                                                    !u.IsAlive &&
                                                    u.CanLoot &&
                                                    u.Lootable &&
                                                    !u.Looting
                                                )
                                            ))
                                .OrderBy(u => u.Distance);

                _units = (Me.IsFlying || (Flightor.CanFly && Me.IsOutdoors))
                                ? _units.OrderBy(o => o.Distance)
                                : _units.Where(o => GetPathDistance(o) > 0 &&
                                                    GetPathDistance(o) < 150)
                                        .OrderBy(o => GetPathDistance(o));

                return _units.FirstOrDefault();
            }
        }

        private WoWUnit _nodeNpc
        {
            get
            {
                if (Me.CurrentMap.ExpansionId != 6)
                    return null;

                // BRIMSTONE DESTROYER  (INFERNAL BRIMSTONE)
                    if (CanAttackBrimstoneDestroyer())
                        _nodeNpcIds.Add(_brimstoneDestroyerId);
                    else if (_nodeNpcIds.Contains(_brimstoneDestroyerId))
                        _nodeNpcIds.Remove(_brimstoneDestroyerId);
                //>>

                var _unit = ObjectManager.GetObjectsOfType<WoWUnit>()
                                .Where(u => _nodeNpcIds.Contains(u.Entry) &&
                                            u.IsValid &&
                                            (
                                                (
                                                    u.IsAlive &&
                                                    u.Attackable
                                                ) ||
                                                (
                                                    !u.IsAlive &&
                                                    u.CanLoot &&
                                                    u.Lootable &&
                                                    !u.Looting
                                                )
                                            ) )
                                .OrderBy(u => u.Distance)
                                .FirstOrDefault();
                return _unit;
            }
        }

        private void Targeting_WeighTargetsFilter(List<Targeting.TargetPriority> _priorities)
        {
            if (_nodeNpc == null)
                return;

            foreach(var _priority in _priorities)
            {
                var _unit = _priority.Object.ToUnit();
                if (_unit != _nodeNpc)
                    continue;

                var _score = _priority.Score;
                if (_score == Double.MaxValue)
                    continue;

                DevLog("High Priority: \"{0}\" (Id: {1})", _unit.Name, _unit.Entry);
                _priority.Score = Double.MaxValue;
            }
        }

        private uint _brimstoneDestroyerId
        {
            get
            {
                return 93237;
            }
        }

        private string GetPullSpell()
        {
            var _spells = new List<string>()
            {
                "Death Grip",
                "Death Coil",
                "Icy Touch",
                "Dark Command",
                "Wrath",
                "Moonfire",
                "Faerie Fire (Feral)",
                "Growl",
                "Arcane Shot",
                "Steady Shot",
                "Fireball",
                "Frostbolt",
                "Dizzying Haze",
                "Provoke",
                "Shoot",
                "Throw",
                "Avenger's Shield",
                "Judgment",
                "Exorcism",
                "Hand of Reckoning",
                "Mind Blast",
                "Smite",
                "Lightning Bolt",
                "Shadow Bolt",
                "Heroic Throw",
                "Throw",
                "Taunt"
            };

            var _spell = _spells.FirstOrDefault(s => SpellManager.HasSpell(s) &&
                                                     SpellManager.CanCast(s));
            return _spell;
        }

        private async Task<bool> InterruptBlazingHellfire()
        {
            if (_nodeNpc == null)
                return false;

            var _interruptSpell = ScriptHelpers.GetInterruptSpell();
            if (_interruptSpell == null)
                return false;

            if (!SpellManager.CanCast(_interruptSpell))
                return false;

            if (_nodeNpc.Entry != _brimstoneDestroyerId)
                return false;

            if (!_nodeNpc.IsChanneling)
                return false;

            SpellManager.Cast(_interruptSpell);

            return true;
        }

        private WoWUnit _npcOfInterest
        {
            get
            {
                return _nodeNpc ?? _rareNpc;
            }
        }

        private int GetMaxTime(float _pathDistance, float _minValue = 5f, float _minSpeed = 4f)
        {
            return (int)Math.Max(_minValue, _pathDistance / _minSpeed);
        }

        private async Task<bool> HandleNpcs()
        {
            RuntimeLog(">");

            if (_npcOfInterest == null)
                return false;

            RuntimeLog(">");

            await InterruptBlazingHellfire();

            RuntimeLog(">");

            if (Me.Durability <= 0.3)
                return false;

            RuntimeLog(">");

            if (_nodeNpc != null &&
                (
                    _nodeNpc.TaggedByOther ||
                    (
                        _nodeNpc.Entry == _brimstoneDestroyerId &&
                        !CanAttackBrimstoneDestroyer()
                    )
                ))
                return false;

            RuntimeLog(">");

            if (!Me.Mounted && _npcOfInterest.TaggedByMe)
                return false;

            RuntimeLog(">");

            if (breakOut)
            {
                DevLog("HandleNpcs: BreakOut");
                return false;
            }

            var _npcOfInterestLocation = _npcOfInterest.Location;
            var _npcOfInterestEntry = _npcOfInterest.Entry;
            var _npcOfInterestName = _npcOfInterest.Name;
            var _npcOfInterestGuid = _npcOfInterest.Guid;
            var _startTime = DateTime.Now;
            var _maxTime = GetMaxTime(GetPathDistance(_npcOfInterest));

            var _isRare = (_npcOfInterest == _rareNpc);
            DevLog("IsRare: {0}", _isRare);

            if (_ignoredGeneralGuids.ContainsKey(_npcOfInterestGuid))
                DevLog("Blacklisted since: {0}", DeltaTime(_ignoredGeneralGuids[_npcOfInterestGuid], "m"));
            else
                DevLog("Not blacklisted");

            Log("{0} \"{1}\" (Id: {2})", ((_npcOfInterest.IsAlive)
                                                ? "Attacking"
                                                : "Looting"), _npcOfInterestName, _npcOfInterestEntry);
            while ((_npcOfInterest != null || Me.Location.Distance(_npcOfInterestLocation) > 10) && !breakOut)
            {
                if (!_isRare)
                    DevLog("WHILE: HandleNpcs");
                else if (_infoTimer.IsFinished)
                {
                    DevLog("WHILE: HandleNpcs");
                    DevLog("     Name: {0} (Id: {1})", _npcOfInterestName, _npcOfInterestEntry);
                    DevLog("     Location: {0}", _npcOfInterestLocation);
                    DevLog("     Guid: {0}", SafeGuid(_npcOfInterestGuid));
                    DevLog("     Distance: {0}", Math.Round(Me.Location.Distance(_npcOfInterestLocation)));
                    DevLog("     MaxTime: {0}", _maxTime);
                    DevLog("     TimeLeft: {0}", Math.Ceiling(_maxTime - DeltaTime(_startTime, "s")));
                    _infoTimer.Reset();
                }

                if (DeltaTime(_startTime, "s") > _maxTime &&
                    (_npcOfInterest == null || !_npcOfInterest.WithinInteractRange) && _isRare)
                {
                    DevLog("     MaxTime reached");
                    DevLog("     Blacklisting for 1 hour");
                    DevLog("     Name: {0} (Id: {1})", _npcOfInterestName, _npcOfInterestEntry);
                    if (!_ignoredGeneralGuids.ContainsKey(_npcOfInterestGuid))
                        _ignoredGeneralGuids.Add(_npcOfInterestGuid, DateTime.Now);
                    else
                        _ignoredGeneralGuids[_npcOfInterestGuid] = DateTime.Now;
                    return false;
                }

                if (_npcOfInterest != null)
                    _npcOfInterestLocation = _npcOfInterest.Location;

                if (_npcOfInterest == null || !_npcOfInterest.WithinInteractRange)
                    await MoveTo(_npcOfInterestLocation);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await CommonCoroutines.LandAndDismount(null, true);

                    if (_npcOfInterest == null && !_isRare)
                        return false;

                    if (_npcOfInterest != null && !_npcOfInterest.IsAlive)
                    {
                        _npcOfInterest.Interact();
                        await Loot();
                        await Coroutine.Wait(2000, () => (_nodeNpc == _npcOfInterest &&
                                                         _nodeNpc == null) || _rareNpc != null);
                    }
                    else if(_npcOfInterest != null)
                    {
                        if (_npcOfInterest == _rareNpc)
                        {
                            _lastRareGuid = _npcOfInterest.Guid;
                            _lastRareName = _npcOfInterest.Name;
                            _lastRareId = _npcOfInterest.Entry;
                            _timeStartRareAttack = DateTime.Now;
                        }

                        _npcOfInterest.Target();
                        Lua.DoString("StartAttack()");
                        await Coroutine.Wait(2000, () => _npcOfInterest != null && _npcOfInterest.TaggedByMe);
                        return false;
                    }
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleGatheringProfessions()
        {
            if (Settings.GatheringProfessions_Professions.ToString() == "None")
                return false;

            if (!Me.IsAlive)
                return false;

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
                return false;

            await HandleGatheringSettings();

            RuntimeLog(">");

            if (await HandleGetNode())
                return true;

            RuntimeLog(">");

            if (await HandleFoxTrail())
                return true;

            RuntimeLog(">");

            if (await HandleTrainSkill())
                return true;

            RuntimeLog(">");

            if (await HandleRankHelper())
                return true;

            RuntimeLog(">");

            if (Me.CurrentMap.ExpansionId != 6)
                return false;

            RuntimeLog(">");

            if (await HandleLegionRanks())
                return true;

            return false;
        }

        private bool _inShrine
        {
            get
            {
                return (Me.IsHorde)
                            ? Me.ZoneId == 6141 || Me.ZoneId == 6554
                            : Me.ZoneId == 6142 || Me.ZoneId == 6553;
            }
        }

        private bool _inCapital
        {
            get
            {
                return (Me.IsHorde)
                            ? Me.ZoneId == 1637
                            : Me.ZoneId == 1519;
            }
        }

        private bool _inCapitalSurroundingZone
        {
            get
            {
                return (Me.IsHorde)
                            ? Me.ZoneId == 14
                            : Me.ZoneId == 12;
            }
        }

        private int _classicHearthstoneAreaId
        {
            get
            {
                return (Me.IsHorde)
                            ? 5170
                            : 5148;
            }
        }

        private int _pandariaHearthstoneAreaId
        {
            get
            {
                return (Me.IsHorde)
                            ? 6519
                            : 6484;
            }
        }

        /*
        private async Task<bool> HandleHearthstoneArea(bool _useTaxi = false, bool _ignoreCapital = false)
        {
            if (!Settings.Hearth_Hearthstone ||
                !Settings.Hearth_SetHearthstone)
                return false;

            var _inValeOfEternalBlossoms =
                 Me.ZoneId == 5840 || ((Me.IsHorde)
                                        ? Me.ZoneId == 6554 || Me.ZoneId == 6141
                                        : Me.ZoneId == 6553 || Me.ZoneId == 6142);
            if (!_ignoreCapital &&
                !_inValeOfEternalBlossoms &&
                !_inCapital &&
                !_inCapitalSurroundingZone)
                return false;

            var _hearthstone = Me.BagItems.FirstOrDefault(i => i.Entry == 6948);
            if (_hearthstone == null)
                return false;

            if (Me.HearthstoneAreaId == _classicHearthstoneAreaId && !_inValeOfEternalBlossoms)
                return false;
            if (Me.HearthstoneAreaId == _pandariaHearthstoneAreaId)
                return false;

            Log("Setting Hearthstone");
            var _npcId = (Me.IsHorde)
                                ? (_inValeOfEternalBlossoms)
                                        ? 63008
                                        : 6929
                                : (_inValeOfEternalBlossoms)
                                        ? 64149
                                        : 6740;
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            var _npc = GetNpc(_npcId);
            var _newHearthstoneAreaId = (_inValeOfEternalBlossoms)
                                                ? _pandariaHearthstoneAreaId
                                                : _classicHearthstoneAreaId;
            while (Me.HearthstoneAreaId != _newHearthstoneAreaId && !breakOut)
            {
                DevLog("WHILE: HearthstoneArea");

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;
                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation, _useTaxi);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();
                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("GossipFrame") || IsFrameVisible("StaticPopup1"));

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _binderEntry = GossipFrame.Instance.GossipOptionEntries
                                                .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Binder);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_binderEntry.Index);
                        await Coroutine.Wait(5000, () => IsFrameVisible("StaticPopup1"));
                    }

                    if (IsFrameVisible("StaticPopup1"))
                    {
                        await Sleep();
                        await ClickButton("StaticPopup1Button1");
                    }

                    await Coroutine.Wait(5000, () => Me.HearthstoneAreaId == _newHearthstoneAreaId);
                }

                await Coroutine.Yield();
            }
            _checkedItems = false;
            _checkedQuests = false;
            _checkedGatheringSkills = false;

            return true;
        }
        */

        bool IsSkillMaxedOut(SkillLine _skill)
        {
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = (Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism)
                                ? _skillValue + 15
                                : _skillValue;

            if (_skillValue == 0)
                return false;

            var _skillMaxValue = (_accountExpansionLevel >= 6)
                                        ? 800
                                        : 700;
            if (_skillValue >= _skillMaxValue)
                return true;

            return false;
        }

        private void LowGraphicMode(bool _enable = true)
        {
            if (_enable)
            {
                _renderScale = 0.1;
                _maxFps = 20;
                _maxFpsBk = 20;
            }
            else
            {
                _renderScale = Settings.Performance_DefaultRenderScale / 100;
                _maxFps = Settings.Performance_MaxFps;
                _maxFpsBk = Settings.Performance_MaxFpsBk;
            }
            Lua.DoString($"SetCVar('RenderScale', {_renderScale})");
            Lua.DoString($"SetCVar('MaxFPS', {_maxFps})");
            Lua.DoString($"SetCVar('MaxFPSBk', {_maxFpsBk})");
        }

        private async Task<bool> HandleStop()
        {
            if (Settings.Stop_Level == 0 &&
                Settings.Stop_Item == "0" &&
                Settings.Stop_Quest == "0" &&
                !Settings.Stop_SkillMax)
                return false;

            var _stopLevel = Settings.Stop_Level != 0 && Me.Level >= Settings.Stop_Level;
            var _stopQuest = Settings.Stop_Quest != "0" && HasQuest(StringToArray(Settings.Stop_Quest));
            var _stopItem = Settings.Stop_Item != "0" && HasItem(StringToArray(Settings.Stop_Item));

            RuntimeLog(">");

            var _professionsMaxSkilled = _gatheringProfessions
                                                .Where(s => IsSkillMaxedOut(s))
                                                .Count();

            RuntimeLog(">");

            var _stopSkillMax = (Settings.GatheringProfessions_Professions.ToString().Contains("And"))
                                        ? _professionsMaxSkilled == 2
                                        : _professionsMaxSkilled == 1;

            RuntimeLog(">");

            if (!_stopLevel && !_stopQuest && !_stopSkillMax && !_stopItem)
                return false;

            var _stopReason = (_stopItem)
                                    ? "StopItem received"
                                    : ((_stopLevel)
                                            ? "StopLevel reached"
                                            : ((_stopQuest)
                                                    ? "StopQuest accepted"
                                                    : "Gathering Professions max skilled"));

            Log("Stopping HB as {0}", _stopReason);
            LowGraphicMode(false);

            if (Settings.Stop_HBRelog)
                Bots.Professionbuddy.Dynamic.Helpers.HBRelog.StopProfile(Bots.Professionbuddy.Dynamic.Helpers.HBRelog.CurrentProfileName);
            else
            {
                Settings.General_InfiniteMode = false;
                TreeRoot.Stop("KeepOnMoving");
            }

            if (Settings.Stop_Action != "0")
                Bots.Professionbuddy.Dynamic.Helpers.HBRelog.StartProfile(Settings.Stop_Action);

            return true;
        }

        /*
        private async Task<bool> HandleMounts()
        {
            if (Settings.G_GroundMount == "0" && Settings.G_FlyingMount == "0")
                return false;

            var _groundMountSpellId = CharacterSettings.Instance.GroundMountSpellId.ToString();
            var _flyingMountSpellId = CharacterSettings.Instance.FlyingMountSpellId.ToString();

            if (Settings.G_GroundMount != "0" && _groundMountSpellId == Settings.G_GroundMount &&
                Settings.G_FlyingMount != "0" && _flyingMountSpellId == Settings.G_FlyingMount)
                return false;

            if (Settings.G_GroundMount != "0" && _groundMountSpellId != Settings.G_GroundMount)
            {
                var _newSpellId = -1;
                if (Settings.G_GroundMount.All(char.IsDigit))
                    _newSpellId = Convert.ToInt32(Settings.G_GroundMount);
                else
                {
                    var _mount = Mount.GroundMounts.FirstOrDefault(m => m.Name == Settings.G_GroundMount);
                    if (_mount != null)
                        _newSpellId = _mount.CreatureSpellId;
                }

                if (_newSpellId != -1 && SpellManager.CanCast(_newSpellId))
                {
                    CharacterSettings.Instance.GroundMountSpellId = _newSpellId;
                    CharacterSettings.Instance.Save();
                }
            }

            if (Settings.G_FlyingMount != "0" && _flyingMountSpellId != Settings.G_FlyingMount)
            {
                var _newSpellId = -1;
                if (Settings.G_FlyingMount.All(char.IsDigit))
                    _newSpellId = Convert.ToInt32(Settings.G_FlyingMount);
                else
                {
                    var _mount = Mount.FlyingMounts.FirstOrDefault(m => m.Name == Settings.G_FlyingMount);
                    if (_mount != null)
                        _newSpellId = _mount.CreatureSpellId;
                }

                if (_newSpellId != -1 && SpellManager.CanCast(_newSpellId))
                {
                    CharacterSettings.Instance.FlyingMountSpellId = _newSpellId;
                    CharacterSettings.Instance.Save();
                }
            }

            return true;
        }
        */

        private async Task<bool> HandleJump()
        {
            if (!Settings.Jump_Enable)
                return false;

            if (!Me.IsMoving)
                return false;

            if (Settings.Jump_MinPause > Settings.Jump_MaxPause)
                return false;

            if (Me.IsFlying)
                return false;

            if (Me.IsSwimming)
                return false;

            if (Me.Combat && !Me.Mounted)
                return false;

            var _clickPos = WoWMovement.ClickToMoveInfo.ClickPos;
            if (_clickPos.Distance(Me.Location) < 20)
                return false;

            var _inFront = WoWMathHelper.CalculatePointInFront(Me.Location, Me.Rotation, _clickPos.Distance(Me.Location));
            if (_inFront.Distance(_clickPos) > 5)
                return false;

            if (Me.Mounted && Mount.Current.IsFlyingMount && !Me.HasAura("Stag Form"))
                return false;

            if (DeltaTime(_timeLastJump, "ms") <= _deltaNextJump)
                return false;

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
                return false;

            _timeLastJump = DateTime.Now;
            _deltaNextJump = _rnd.Next(Settings.Jump_MinPause, Settings.Jump_MaxPause);
            await Move("J", 50);

            return true;
        }

        private async Task<bool> Loot()
        {
            /*
            if (DeltaTime(_timeDialogOpened, "ms") > DeltaTime(_timeDialogClosed, "ms"))
                return false;
                */

            if (!await Coroutine.Wait(1000, () => IsFrameVisible("LootFrame") ||
                                                  DeltaTime(_timeDialogOpened, "ms") > DeltaTime(_timeDialogClosed, "ms")))
                return false;

            LootFrame.Instance.LootAll();
            return true;
        }

        private async Task<bool> HandleLoot()
        {
            var _harvestMobs = (int)Settings.GatheringProfessions_HarvestMobs;
            if (!Settings.General_SpeedLoot && _harvestMobs == 0)
                return false;

            if (!CharacterSettings.Instance.LootMobs)
                return false;

            if (!Me.IsAlive)
                return false;

            if (DeltaTime(_timeUnghost, "s") < 10 && Me.HealthPercent <= 90)
                return false;

            if (Me.IsCasting)
                return false;

            if (Me.Combat && _lootableMob == null)
                return false;

            if (_lootableMob == null && (_harvestMobs != 0 && _interactableMob == null))
                return false;

            var _mob = (_lootableMob != null)
                            ? _lootableMob
                            : _harvestMobs != 0
                                    ? _interactableMob
                                    : null;

            var _startTime = DateTime.MinValue;
            while (_mob != null)
            {
                DevLog("WHILE: HandleLoot");

                if (_mob != null &&
                    _startTime != DateTime.MinValue &&
                    DeltaTime(_startTime, "s") >= 10)
                {
                    DevLog("Needed too long to loot > Blacklist for 1 hour");
                    _ignoredGeneralGuids.Add(_mob.Guid, DateTime.Now);
                    return false;
                }

                if (!_init)
                    return false;

                if (!Me.IsAlive)
                    return false;

                if (Me.Combat && _lootableMob == null)
                    return false;

                try
                {
                    var _harvest = _mob.CanSkin;
                    while(!_mob.WithinInteractRange && !breakOut)
                    {
                        DevLog("WHILE: HandleLoot > GoTo");

                        if(!await MoveTo(_mob.Location))
                            return false;
                        await Coroutine.Yield();
                    }
                    await CommonCoroutines.StopMoving();
                    if (Me.IsFlying)
                        await CommonCoroutines.Dismount(null, false);

                    _startTime = (_startTime == DateTime.MinValue)
                                        ? DateTime.Now
                                        : _startTime;
                    _mob.Interact();

                    if(await Loot())
                    {
                        if (await Coroutine.Wait(4000, () => (_harvest)
                                                                ? !_mob.CanSkin
                                                                : !_mob.CanLoot))
                        {
                            if (_harvest)
                                break;
                        }
                    }
                }
                catch(Exception)
                {
                    //
                }

                _mob = (_lootableMob != null)
                            ? _lootableMob
                            : (_harvestMobs != 0)
                                    ? _interactableMob
                                    : null;
                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleResurrect()
        {
            RuntimeLog(">");

            if (Settings.General_ResurrectionTimer == 0)
                return false;

            RuntimeLog(">");

            if (!Me.IsGhost)
                return false;

            RuntimeLog(">");

            if (_timeDied == DateTime.MinValue)
                _timeDied = DateTime.Now;

            RuntimeLog(">");

            var _beginResurrect = Settings.General_ResurrectionTimer;
            if (_failedReviveCount < 2 && DeltaTime(_timeDied, "s") < _beginResurrect)
            {
                var _passedTime = Math.Floor(DeltaTime(_timeDied, "s"));
                if (_passedTime >= 10 && _logTimer.IsFinished)
                {
                    var _remainingTime = _beginResurrect - _passedTime;

                    DevLog("RESURRECT in {0}", TimeString(_remainingTime));
                    DevLog("     Dead for {0}", TimeString(_passedTime));

                    _logTimer.Reset();
                }
                return false;
            }

            Log("Resurrecting at graveyard");
            while (!Me.IsAlive && _init)
            {
                DevLog("WHILE: HandleResurrect Start");

                while ((_spirithealer == null || Me.Location.Distance(_spirithealer.Location) > 50) && !Me.IsAlive)
                {
                    DevLog("WHILE: Resurrect PortBack");

                    await ClickButton("GhostFrame");
                    await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld && _spirithealer != null && _init);
                    await Coroutine.Yield();
                }

                if (!_spirithealer.WithinInteractRange)
                    await MoveTo(_spirithealer.Location);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _spirithealer.Interact();
                    await Coroutine.Wait(5000, () => IsFrameVisible("StaticPopup1"));

                    while (IsFrameVisible("StaticPopup1") && _init)
                    {
                        DevLog("WHILE: Resurrect Click");
                        await ClickButton("StaticPopup1Button1");
                        await Coroutine.Yield();
                    }
                }
                await Coroutine.Yield();
            }

            _failedReviveCount = 0;
            return true;
        }

        private bool StringContains(object _text, string _operator = "OR", params object[] _strings)
        {
            var _count = 0;
            foreach(var _string in _strings)
            {
                if(_text.ToString().Contains(_string.ToString()))
                {
                    if (_operator == "OR")
                        return true;

                    _count++;
                }
            }

            return _count == _strings.Count();
        }

        private async Task<bool> HandleFrameBug()
        {
            RuntimeLog(">");

            if (!IsFrameVisible("QuestFrame") && !IsFrameVisible("GossipFrame") &&
                !IsFrameVisible("TaxiFrame", true) && !IsFrameVisible("FlightMapFrame", true))
            {
                RuntimeLog(">");

                _lastFrameText = null;
                _timeFrameChange = DateTime.Now;
                return false;
            }

            if(
                (
                    IsFrameVisible("TaxiFrame", true) ||
                    IsFrameVisible("FlightMapFrame", true)
                ) &&
                StringContains(BotPoi.Current.Type, "OR", "QuestTurnIn", "QuestPickUp") &&
                DeltaTime(_timeDialogOpened, "s") > 5)
            {
                TaxiFrame.Instance.Hide();
                return false;
            }

            var _frameText = (IsFrameVisible("QuestFrame"))
                                    ? Lua.GetReturnVal<string>("return GetQuestText()", 0)
                                    : GossipFrame.Instance.GossipText;
            if (_frameText != _lastFrameText)
            {
                _lastFrameText = _frameText;
                _timeFrameChange = DateTime.Now;
            }

            if (DeltaTime(_timeFrameChange, "s") < 8)
                return false;

            if (IsFrameVisible("QuestFrame"))
            {
                Log("Closing QuestFrame");
                QuestFrame.Instance.Hide();
                await Coroutine.Wait(2000, () => !IsFrameVisible("QuestFrame"));
            }
            else
            {
                Log("Closing GossipFrame");
                GossipFrame.Instance.Hide();
                await Coroutine.Wait(2000, () => !IsFrameVisible("GossipFrame"));
            }
            _lastFrameText = null;
            _timeFrameChange = DateTime.Now;

            return true;
        }

        private async Task<bool> HandleGetOffGround()
        {
            if (!Settings.Quest_GetOffGround)
                return false;

            if (Me.IsSwimming)
                return false;

            if (!Me.Mounted)
                return false;

            if (!Flightor.CanFly && !Mount.Current.IsFlyingMount)
                return false;

            if (!Me.IsMoving)
                return false;

            if (!WoWMovement.ClickToMoveInfo.IsClickMoving)
                return false;

            /*
            if (!Mount.Current.IsFlyingMount)
                return false;
            */

            if (Me.HasAura("Stag Form"))
                return false;

            if (Me.IsIndoors)
                return false;

            var _target = Me.CurrentTarget;
            if (_target != null && Me.Location.Distance(_target.Location) < 10)
                return false;

            if (Me.IsFlying || _timeStartFlyingMountRun == DateTime.MinValue)
            {
                _timeStartFlyingMountRun = DateTime.Now;
                return false;
            }

            if (DeltaTime(_timeStartFlyingMountRun, "ms") < 1500)
                return false;

            if (Flightor.CanFly && DeltaTime(_timeGettingOff, "ms") > 3000)
            {
                Log("Getting off ground");
                var _endLocation = ExtractVector3(TreeRoot.GoalText);
                var _CTMLocation = (_endLocation != Vector3.Zero)
                                            ? _endLocation
                                            : WoWMovement.ClickToMoveInfo.ClickPos;

                var _distance = (int)Math.Round(Me.Location.Distance(_CTMLocation));
                var _interval = Math.Max(100, Math.Min(2000, _distance * 6));

                await Move("JF", _interval);
                Navigator.Clear();
                BotPoi.Current.NavType = NavType.Fly;
                ProfileManager.CurrentProfile.NavType = NavType.Fly;
                Flightor.MoveTo(_CTMLocation);
            }
            else
            {
                Log("Switching to ground mount");
                await CommonCoroutines.LandAndDismount(null, true);
                await CommonCoroutines.SummonGroundMount();
            }


            _timeGettingOff = DateTime.Now;
            return true;
        }

        /*
        private async Task<bool> TakeTaxi(Vector3 _location)
        {
            RuntimeLog(">>>>");

            if (Flightor.CanFly)
                return false;

            var _knownNodes = KnownFlightNodesManager.EnumerateKnownFlightNodes()
                                        .Where(n => n.MapId == Me.MapId);
            if (_knownNodes.Count() == 0)
                return false;

            RuntimeLog(">>>>");

            var _closestNodeToMe = _knownNodes.OrderBy(n => n.Location.Distance(Me.Location))
                                              .FirstOrDefault();
            var _closestNodeToLocation = _knownNodes.OrderBy(n => n.Location.Distance(_location))
                                                    .FirstOrDefault();

            if (_closestNodeToMe == _closestNodeToLocation)
                return false;

            RuntimeLog(">>>>");

            var _taxiPathDistance = _closestNodeToMe.Location.Distance(Me.Location) +
                                    _closestNodeToMe.Location.Distance(_closestNodeToLocation.Location) / 1.8 +
                                    _closestNodeToLocation.Location.Distance(_location);
            var _directPathDistance = Me.Location.Distance(_location);

            if (_taxiPathDistance >= _directPathDistance)
                return false;

            RuntimeLog(">>>>");

            Log("Taking taxi");
            Log("     < {0} \t\t\t({1})", _closestNodeToMe.Name,
                                    _closestNodeToMe.Location);
            Log("     > {0} \t\t\t({1})", _closestNodeToLocation.Name,
                                    _closestNodeToLocation.Location);
            var _goTo = _closestNodeToMe.Location;

            while (!Me.OnTaxi && !breakOut)
            {
                DevLog("WHILE: TakeTaxi");

                RuntimeLog(">>>>");

                if (_flightmaster != null)
                    _goTo = _flightmaster.Location;
                if (_flightmaster == null || !_flightmaster.WithinInteractRange)
                    await MoveTo(_goTo);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _flightmaster.Interact();

                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _taxiEntry = GossipFrame.Instance.GossipOptionEntries
                                                    .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Taxi);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_taxiEntry.Index);
                        await Coroutine.Wait(3 * 1000, () => IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"));
                    }

                    if (IsFrameVisible("TaxiFrame") || IsFrameVisible("FlightMapFrame"))
                    {
                        var _mapNodes = WoWTaxi.GetTaxiMapNodes().ToList();
                        var _flyToNode = _mapNodes.FirstOrDefault(n => n.NodeId == _closestNodeToLocation.Id);
                        await Sleep();
                        _flyToNode.TakeFlight();
                        await Coroutine.Wait(3 * 1000, () => Me.OnTaxi);
                    }
                }

                RuntimeLog(">>>>");

                await Coroutine.Yield();
            }
            DevLog("TakeTaxi > OnTaxi");
            await Coroutine.Wait(1800000, () => !Me.OnTaxi || !_init);
            DevLog("TakeTaxi > OffTaxi");

            return true;
        }
        */

        private static double DistanceToLine(Vector3 _location, Vector3 _lineStart, Vector3 _lineEnd)
        {
            var _u = _lineStart - _lineEnd;
            var _a = _lineStart;
            var _p = _location;

            return Vector3.Cross(Vector3.Subtract(_p, _a), _u).Length() / _u.Length();
        }

        private static double DistanceToPath(Vector3 _location, Vector3[] _path)
        {
            double _closestDistance = _location.Distance(_path[0]);

            for (int i = 1; i <= _path.Count() - 1; i++)
            {
                var _start = _path[i - 1];
                var _end = _path[i];
                var _length = _start.Distance(_end);

                var _toStart = _start.Distance(Me.Location);
                var _toEnd = _end.Distance(Me.Location);
                var _toLine = DistanceToLine(_location, _start, _end);

                var _distance = (_toStart + _toEnd < 2 * _length)
                                        ? _toLine
                                        : Math.Min(_toStart, _toEnd);
                if (_distance < _closestDistance)
                    _closestDistance = _distance;
            }

            return _closestDistance;
        }

        private class Blackspot
        {
            public Blackspot(int _mapId,
                             string _name)
            {
                MapId = _mapId;
                Name = _name;
            }

            public int MapId { get; }
            public string Name { get; }
        }

        private bool FlyingBelowDalaran(Vector3 _center, int _radius)
        {
            if (!Me.IsFlying)
                return false;

            var _craterCenter = _center + new Vector3(0f, 0f, -50);
            var _belowCenter = Me.Z < _craterCenter.Z;
            if (!_belowCenter)
                return false;

            var _inSphere = Me.Location.Distance(_craterCenter) <= _radius;
            if (_inSphere)
                return true;

            return false;
        }

        private List<Blackspot> _navigationBlackspotsData = new List<Blackspot>
        {
            // EASTERN KINGDOMS
                // EBON HOLD: FLY WAY OUT
                    new Blackspot( 0,   "EbonHold_FlyWayOut" ),
                // SWAMP OF SORROWS: TOWER
                    new Blackspot( 0,   "SwampOfSorrows_Tower" ),

            // KALIMDOR
                // ORGRIMMAR: CLEFT OF SHADOWS
                    new Blackspot( 1,   "Orgrimmar_CleftOfShadows" ),
                // ULDUM: CHAMBER OF THE SUN
                    new Blackspot( 1,   "Uldum_ChamberOfTheSun" ),

            // OUTLAND
                // SHATTRATH : TERACE OF LIGHT
                    new Blackspot( 530, "Shattrath_TeraceOfLight" ),
                // HELLFIRE PENINSULA: ELITES AT PORTAL
                    new Blackspot( 530, "HellfirePeninsula_ElitesAtPortal" ),

            // NORTHREND
                // GRIZZLY HILLS: GRIZZLEMAW
                    new Blackspot( 571, "GrizzlyHills_Grizzlemaw" ),
                // STORM PEAK: MINE
                    new Blackspot( 571, "StormPeak_Mine" ),
                // OLD DALARAN: FLYING BELOW
                    new Blackspot( 571, "OldDalaran_FlyingBelow" ),


            // PANDARIA
                // VALLEY OF FOUR WINDS: CLIFF
                    new Blackspot( 870, "ValleyOfFourWinds_Cliff" ),

            // BROKEN ISLES
                // NEW DALARAN: HERABLISM TRAINER
                    new Blackspot( 1220,   "NewDalaran_HerbalismTrainer" ),
                // NEW DALARAN: FLYING BELOW
                    new Blackspot( 1220,   "NewDalaran_FlyingBelow" )
        };

        private Vector3 GetAboveDalaranLocation(Vector3 _center)
        {
            var _verticalProjection = new Vector3(Me.Location.X,
                                                  Me.Location.Y,
                                                  _center.Z + 30);
            var _centerInAir = _center + new Vector3(0, 0, 30);
            var _centerAirToVerticalProjection = _verticalProjection - _centerInAir;

            var _unitVector = Vector3.Normalize(_centerAirToVerticalProjection);

            var _worldLines = new List<WorldLine>();
            for(var i = 20; i <= 500; i += 20)
            {
                var _flyTo = _centerInAir + Vector3.Multiply(i, _unitVector);
                _worldLines.Add(new WorldLine(Me.Location, _flyTo));
            }
            GameWorld.MassTraceLine(_worldLines.ToArray(), TraceLineHitFlags.Collision, out var _hitResult);
            var _bestFlyToIndex = _hitResult
                                        .ToList()
                                        .IndexOf(false);
            if (_bestFlyToIndex == -1)
                return Vector3.Zero;

            var _bestWorldLine = _worldLines[_bestFlyToIndex];
            var _bestFlyTo = _bestWorldLine.End;

            return _bestFlyTo;
        }

        private async Task<bool> HandleNavigationBlackspots()
        {
            var _navigationBlackspot = _navigationBlackspotsData
                                            .FirstOrDefault(b => b.MapId == Me.MapId &&
                                                                 GetCondition(b.Name));
            if (_navigationBlackspot == null)
                return false;

            Vector3 _goTo;
            var _task = _navigationBlackspot.Name;

            Log("Avoiding blackspot by taking alternative route");
            switch (_task)
            {
                case "NewDalaran_FlyingBelow":
                    var _newCenter = new Vector3(-844.2977f, 4472.829f, 735.8796f);
                    _goTo = GetAboveDalaranLocation(_newCenter);
                    await FollowPath(_task, 20, true, false, _goTo);
                    break;

                case "OldDalaran_FlyingBelow":
                    var _oldCenter = new Vector3(5808.426f, 638.0281f, 647.6355f);
                    _goTo = GetAboveDalaranLocation(_oldCenter);
                    await FollowPath(_task, 20, true, false, _goTo);
                    break;

                case "NewDalaran_HerbalismTrainer":
                    await FollowPath(_task, 5, true, false,
                        new Vector3(-776.153f, 4516.234f, 731.9677f),
                        new Vector3(-782.0711f, 4509.167f, 730.7239f));
                    break;

                case "Uldum_ChamberOfTheSun":
                    await FollowPath(_task, 10, false, false,
                                     new Vector3(-10250.65f, -331.821f, 271.3362f),
                                     new Vector3(-10165.33f, -311.9824f, 328.9292f));
                    break;

                case "Shattrath_TeraceOfLight":
                    await FollowPath(_task, 10, false, false,
                                    new Vector3(-1850.099f, 5402.949f, -2.474063f),
                                    new Vector3(-1828.878f, 5359.747f, -5.543963f),
                                    new Vector3(-1816.314f, 5312.777f, 16.35586f));
                    break;

                case "StormPeak_Mine":
                    await FollowPath(_task, 10, false, false,
                                     new Vector3(6896.477f, -1123.058f, 823.0789f),
                                     new Vector3(6898.772f, -1123.095f, 871.7703f));
                    break;

                case "Orgrimmar_CleftOfShadows":
                    await FollowPath(_task, 10, false, false,
                                     new Vector3(1839.717f, -4409.466f, 14.28226f),
                                     new Vector3(1888.178f, -4423.875f, 21.56979f),
                                     new Vector3(1894.588f, -4513.915f, 44.03855f));
                    break;


                case "HellfirePeninsula_ElitesAtPortal":
                    _goTo = ExtractVector3(TreeRoot.GoalText);
                    _goTo = (_goTo == Vector3.Zero)
                                        ? (Me.IsHorde)
                                                ? new Vector3(225.7677f, 2634.509f, 87.71759f)
                                                : new Vector3(-669.2073f, 2711.317f, 94.43723f)
                                        : _goTo;

                    await FollowPath(_task, 10, false, false, _goTo);
                    break;

                case "GrizzlyHills_Grizzlemaw":
                    await FollowPath(_task, 10, false, false, new Vector3(4131.344f, -3686.705f, 350.3196f));
                    break;

                case "SwampOfSorrows_Tower":
                    await FollowPath(_task, 2, false, false,
                            new Vector3(-10400.3f, -2610.552f, 22.6615f),
                            new Vector3(-10410.17f, -2610.601f, 25.42214f),
                            new Vector3(-10408.05f, -2620.31f, 29.35211f));
                    break;

                case "ValleyOfFourWinds_Cliff":
                    _goTo = WoWMathHelper.CalculatePointInFront(Me.Location, Me.Rotation, 100) +
                            new Vector3(0, 0, 100f);
                    await FollowPath(_task, 10, false, false, _goTo);
                    break;

                case "EbonHold_FlyWayOut":
                    await FollowPath(_task, 10, false, false, new Vector3(2339.165f, -5676.859f, 447.7418f));
                    break;
            }

            return true;
        }

        private async Task<bool> FollowPath(string _logDetail = null, int _pathTolerance = 10, bool _useCTM = false, bool _preventBlackspot = true, params Vector3[] _wayPath)
        {
            if (_logDetail != null)
                DevLog(_logDetail);

            foreach (var _moveTo in _wayPath)
            {
                while (Me.Location.Distance(_moveTo) > _pathTolerance && !breakOut)
                {
                    DevLog("WHILE: FollowPath{0}", (_logDetail != null)
                        ? $" > {_logDetail}"
                        : "");

                    if (_useCTM)
                        WoWMovement.ClickToMove(_moveTo);
                    else
                        await MoveTo(_moveTo, _preventBlackspot, _preventBlackspot);
                    await Coroutine.Yield();
                }
            }

            return true;
        }

        private async Task<bool> HandleInit()
        {
            var _plugin = PluginManager.Plugins.FirstOrDefault(p => p.Name == Name && p.Enabled);
            if (_plugin == null)
                return false;

            _init = true;
            return true;
        }

        async Task<bool> UseTransport(Vector3 _wait,
                                      Vector3 _stand,
                                      Vector3 _exit,
                                      int _transportId,
                                      Vector3 _transportStart,
                                      Vector3 _transportEnd)
        {
            while (Me.Location.Distance(_wait) > 2 && !breakOut)
            {
                DevLog("WHILE: UseTransport > GoToWait");
                await MoveTo(_wait, true);
                await Coroutine.Yield();
            }
            await CommonCoroutines.LandAndDismount(null, true);

            DevLog("UseTransport > WaitForTransportStart");
            Func<bool> _atStart = () =>
            {
                if (breakOut)
                    return true;

                var _transport = ObjectManager.GetObjectsOfType<WoWGameObject>().FirstOrDefault(o => o.Entry == _transportId);
                return _transport != null && _transport.Location.Distance(_transportStart) < 2;
            };
            await Coroutine.Wait(10 * 60 * 1000, _atStart);

            while (Me.Location.Distance(_stand) > 2 && !breakOut)
            {
                DevLog("WHILE: UseTransport > GoToStand");
                if (!await MoveTo(_stand))
                    WoWMovement.ClickToMove(_stand);
                await Coroutine.Yield();
            }

            DevLog("UseTransport > WaitForTransportEnd");
            Func<bool> _atEnd = () =>
            {
                if (breakOut)
                    return true;

                var _transport = ObjectManager.GetObjectsOfType<WoWGameObject>().FirstOrDefault(o => o.Entry == _transportId);
                return _transport != null && _transport.Location.Distance(_transportEnd) < 2;
            };
            await Coroutine.Wait(10 * 60 * 1000, _atEnd);

            while (Me.Location.Distance(_exit) > 2 && !breakOut)
            {
                DevLog("WHILE: UseTransport > GoToExit");
                if (!await MoveTo(_exit))
                    WoWMovement.ClickToMove(_exit);
                await Coroutine.Yield();
            }

            return true;
        }

        private List<object[]> _transportBugData = new List<object[]>
        {
            // STORMWIND: SHIP TO NORTHREND
                new object[] {  0, "Stormwind_ShipToNorthrend",     new Vector3(-7747.651f, 1858.269f, 9.289278f) },
            // ORGRIMMAR: ZEPPELIN TO NORTHREND
                new object[] {  1, "Orgrimmar_ZeppelinToNorthrend", new Vector3(1992.438f, -4129.078f, 227.6518f) },
        };

        private async Task<bool> HandleTransportBug()
        {
            var _transportBugLocation = _transportBugData
                                            .FirstOrDefault(l => (int)(l[0]) == Me.MapId &&
                                                                 Me.Location.Distance((Vector3)(l[2])) < 40);

            if (_transportBugLocation == null || _timeAtTransportBugLocation == DateTime.MinValue)
            {
                _timeAtTransportBugLocation = DateTime.Now;
                return false;
            }

            if (DeltaTime(_timeAtTransportBugLocation, "s") < 10)
                return false;

            var _task = (string)_transportBugLocation[1];
            Log("Taking alternative route as transport seems to be bugged");
            switch (_task)
            {
                case "Orgrimmar_ZeppelinToNorthrend":
                    DevLog("Orgrimmar_ZeppelinToNorthrend");

                    if (Flightor.CanFly)
                    {
                        await MountUp();
                        await FollowPath("Orgrimmar_ZeppelinToNorthrend > FlyTo", 10, false, true,
                                         new Vector3(2017.099f, -4171.22f, 225.1838f));
                    }
                    else
                    {
                        Log("     Waiting for zeppelin to dock again");
                        DevLog("Orgrimmar_ZeppelinToNorthrend > WaitToDockAgain");
                        var _getOff = new Vector3(1768.282f, -4293.236f, 133.1137f);

                        await Coroutine.Wait(240000, () => Me.Location.Distance(_getOff) < 15 || breakOut);
                    }

                    Log("     Taking zeppelin to Undercity");
                    DevLog("Orgrimmar_ZeppelinToNorthrend > OrgrimmarToUndercity");
                    await UseTransport(new Vector3(1843.144f, -4391.853f, 135.2331f),
                                        new Vector3(1834.708f, -4385.79f, 135.0204f),
                                        new Vector3(2066.38f, 287.1158f, 97.03137f),
                                        164871,
                                        new Vector3(1833.509f, -4391.543f, 152.7679f),
                                        new Vector3(2062.376f, 292.998f, 114.973f));

                    Log("     Taking zeppelin to Northrend");
                    DevLog("Orgrimmar_ZeppelinToNorthrend > UndercityToNorthrend");
                    await UseTransport(new Vector3(2061.519f, 355.6173f, 82.44997f),
                                        new Vector3(2059.365f, 371.6779f, 82.5022f),
                                        new Vector3(1975.594f, -6097.703f, 67.15379f),
                                        181689,
                                        new Vector3(2056.493f, 381.6347f, 100.362f),
                                        new Vector3(1989.353f, -6082.776f, 85.59646f));
                    break;
                case "Stormwind_ShipToNorthrend":
                    DevLog("Stormwind_ShipToNorthrend");

                    if (!Flightor.CanFly)
                    {
                        Log("     Taking tram to Ironforge");
                        DevLog("Stormwind_ShipToNorthrend > EnterTram");
                        var _tramEntrance = new Vector3(-8354.604f, 511.9481f, 84.4087f);
                        while (StyxWoW.IsInWorld && !breakOut)
                        {
                            await MoveTo(_tramEntrance);
                            await Coroutine.Yield();
                        }
                        await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);

                        DevLog("Stormwind_ShipToNorthrend > UseTram");
                        await UseTransport(new Vector3(14.45237f, 2470.982f, -4.294516f),
                                           new Vector3(5.2006f, 2469.381f, -4.484428f),
                                           new Vector3(12.54623f, -10.97226f, -3.989917f),
                                           218208,
                                           new Vector3(4.498615f, 2470.302f, 5.983582f),
                                           new Vector3(4.498832f, -11.34751f, 7.011068f));

                        DevLog("Stormwind_ShipToNorthrend > ExitTram");
                        var _tramExit = new Vector3(72.52491f, 10.46509f, -4.29681f);
                        while (StyxWoW.IsInWorld && !breakOut)
                        {
                            await MoveTo(_tramExit);
                            await Coroutine.Yield();
                        }
                        await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);

                        await FollowPath("Stormwind_ShipToNorthrend > ExitTunnel", 2, false, true,
                                         new Vector3(-4833.128f, -1283.088f, 501.8689f));
                    }

                    Log("     Taking ship from Menethil to Northrend");
                    DevLog("Stormwind_ShipToNorthrend > MenethilToNorthrend");
                    await UseTransport(new Vector3(-3685.863f, -600.9024f, 4.384491f),
                                       new Vector3(-3675.086f, -593.4379f, 9.47117f),
                                       new Vector3(592.4054f, -5102.106f, 5.260393f),
                                       181688,
                                       new Vector3(-3674.43f, -586.774f, 0.0499605f),
                                       new Vector3(584.0138f, -5118.855f, 0.002798604f));
                    break;
            }

            ReloadProfile();
            Reset();

            return true;
        }

        private async Task<bool> HandleMountUp()
        {
            RuntimeLog(">");

            var _hasAura = Me.HasAura("Eye of Kilrogg");
            if (_hasAura)
                return false;

            if (Me.Class != WoWClass.Druid && Me.Combat)
                return false;

            RuntimeLog(">");

            if (Settings.General_MountUpDistance == 0)
                return false;

            RuntimeLog(">");

            if (Me.Mounted)
                return false;

            RuntimeLog(">");

            if (!WoWMovement.ClickToMoveInfo.IsClickMoving)
                return false;

            RuntimeLog(">");

            var _clickPos = WoWMovement.ClickToMoveInfo.ClickPos;
            if (Me.Location.Distance(_clickPos) < Settings.General_MountUpDistance)
                return false;

            RuntimeLog(">");

            if (await MountUp())
                DevLog("HandleMountUp");

            RuntimeLog(">");

            return true;
        }

        private async Task<bool> HandleAutoLoot()
        {
            if (_autoLoot == 1)
                return false;

            Lua.DoString("SetCVar('AutoLootDefault', 1)");
            _autoLoot = 1;

            return true;
        }

        private async Task<bool> HandleDuelInvite()
        {
            if (!Settings.Dialog_DeclineDuelInvites)
                return false;

            if (DialogState != Dialog.Duel)
                return false;

            Log("Declining duel invite");
            await ClickButton("StaticPopup1Button2", 3000);

            return false;
        }

        private async Task<bool> HandlePartyInvite()
        {
            if (!Settings.Dialog_DeclinePartyInvites)
                return false;

            if (DialogState != Dialog.Party)
                return false;

            Log("Declining party invite");
            await ClickButton("StaticPopup1Button2", 3000);

            return false;
        }

        private async Task<bool> HandleGuildInvite()
        {
            var _setting = Settings.Dialog_DeclineGuildInvites;
            if (_declineGuildInvite == _setting)
                return false;

            _declineGuildInvite = _setting;
            var _luaString = string.Format("SetAutoDeclineGuildInvites({0})", (_setting)
                                                                                    ? "true"
                                                                                    : "false");
            Lua.DoString(_luaString);

            return true;
        }

        private async Task<bool> HandleDialog()
        {
            if (!_declineGuildInvite &&
                !Settings.Dialog_DeclineGuildInvites &&
                !Settings.Dialog_DeclinePartyInvites &&
                !Settings.Dialog_DeclineDuelInvites)
                return false;

            await HandleGuildInvite();

            if (DialogState == Dialog.None)
                return false;

            if (!IsFrameVisible("StaticPopup1"))
            {
                DialogState = Dialog.None;
                return false;
            }

            await HandlePartyInvite();
            await HandleDuelInvite();

            return false;
        }

        private async Task<bool> HandleDig()
        {
            if (!Me.IsAlive)
                return false;

            if (Me.Combat)
                return false;

            if (BotManager.Current.ToString() != "ArchaeologyBuddy")
                return false;

            var _auraId = 210837;
            if (!Me.HasAura(_auraId))
                return false;

            Log("Digging");
            while (Me.HasAura(_auraId) && !breakOut)
            {
                DevLog("WHILE: HandleDig");

                //await CommonCoroutines.LandAndDismount(null, true, Me.Location, 5);
                await CommonCoroutines.Dismount(null, true);
                await CommonCoroutines.StopMoving();

                if (!Me.IsCasting)
                {
                    await Coroutine.Wait(4000, () => SpellManager.CanCast(80451));
                    SpellManager.CastSpellById(80451);
                }

                await Coroutine.Wait(2000, () => !Me.HasAura(_auraId));
            }

            _checkedArchaeologySkill = false;
            return true;
        }

        private async Task<bool> TakePortal(int _portalId, Vector3 _portalLocation)
        {
            var _portal = GetObject(_portalId);
            if (_portal != null)
                _portalLocation = _portal.Location;

            if (_portal == null || !_portal.WithinInteractRange)
                await MoveTo(_portalLocation, false, true);
            else
            {
                await CommonCoroutines.StopMoving();
                await Sleep();
                _portal.Interact();

                await Coroutine.Wait(3 * 1000, () => !StyxWoW.IsInWorld);
                await Coroutine.Wait(3 * 60 * 1000, () => StyxWoW.IsInWorld);
            }

            return true;
        }

        private List<int> _continentsSkillData = new List<int>
        {
            525,    // Pandaria
		    600,	// Draenor
		    700    // BrokenIsles
	    };

        private async Task<bool> HandleSwitchContinents()
        {
            if (Settings.Archaeology_GoToPandaria && ((Me.IsHorde)
                                                ? Me.MapId == 0
                                                : Me.MapId == 1))
            {
                Log("Char needs to be placed in {0} in order to switch to Pandaria.",
                                        (Me.IsHorde)
                                              ? "Kalimdor"
                                              : "Eastern Kingdoms");
                Settings.General_InfiniteMode = false;
                Log("     STOPPING bot...");
                TreeRoot.Stop("KeepOnMoving");
            }

            var _continentEntry = _continentsSkillData.LastOrDefault(c => GetCondition(((Hearth)(_continentsSkillData.IndexOf(c) + 3)).ToString(), true) &&
                                                                          Me.GetSkill(SkillLine.Archaeology).CurrentValue >= c);
            if (_continentEntry == 0)
                return false;

            /*
            if (_continentEntry == 525 &&
                (!Settings.Hearth_Hearthstone ||
                 !Settings.Hearth_SetHearthstone ||
                 UnfavourableAreaSet()))
                return false;
            */

            var _index = _continentsSkillData.IndexOf(_continentEntry) + 3;
            if (Me.CurrentMap.ExpansionId == _index + 1)
                return false;

            if (_index >= 4)        // Draenor + BrokenIsles
            {
                var _continent = ((Hearth)_index).ToString();

                DevLog("Hearth.{0}: Archaeology", _continent);
                Log("Continuing archaeology on next continent");
                HearthState = (Hearth)_index;

                await HandleSellTrash(true);
                return false;
            }

            var _portalId = (Me.IsHorde)
                                ? 215424
                                : 215457;
            var _portalLocation = (Me.IsHorde)
                                        ? new Vector3(2014.819f, -4700.274f, 28.62439f)
                                        : new Vector3(-8197.333f, 528.3328f, 117.16f);
            var _portal = GetObject(_portalId);

            Log("Taking portal to Pandaria");
            while (Me.CurrentMap.ExpansionId != 4 && !breakOut)
            {
                DevLog("WHILE: HandleSwitchContinents");

                await TakePortal(_portalId, _portalLocation);
                await Coroutine.Yield();
            }
            await WaitForMapChange();
            return true;
        }

        private async Task<bool> WaitForMapChange()
        {
            DevLog("Waiting for map change...");
            if (await Coroutine.Wait(10 * 1000, () => DeltaTime(_timeMapChanged, "ms") < 5000))
            {
                DevLog("Map changed");

                // Reset Digsites
                if (BotManager.Current.ToString() == "ArchaeologyBuddy")
                {
                    DevLog("Restart ArchaeologyBuddy");
                    ArchBuddy.Instance.Stop();
                    ArchBuddy.Instance.Start();
                }

                return true;
            }

            return false;
        }

        private async Task<bool> TrainSkill(SkillLine _skill, int _npcId, Vector3 _npcLocation)
        {
            var _skillValue = Me.GetSkill(_skill).CurrentValue;
            _skillValue = ((Me.Race == WoWRace.Tauren && _skill == SkillLine.Herbalism) /*||
                            Me.Race == WoWRace.Worgen && _skill == SkillLine.Skinning   */)
                                    ? _skillValue + 15
                                    : _skillValue;

            if(_skill != SkillLine.Riding)
                Log("Training {0}", _skill);
            DevLog("     CurrentValue: {0}", _skillValue);
            DevLog("     NpcId: {0}", _npcId);

            var _preMax = Me.GetSkill(_skill).MaxValue;
            var _npc = GetNpc(_npcId);
            while (Me.GetSkill(_skill).MaxValue == _preMax && !breakOut)
            {
                DevLog("WHILE: TrainSkill{0}", _skill);

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;

                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation, true, true);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _npc.Interact();

                    await Coroutine.Wait(3 * 1000, () => IsFrameVisible("ClassTrainerFrame") || IsFrameVisible("GossipFrame"));
                    if (IsFrameVisible("GossipFrame"))
                    {
                        var _trainerEntry = GossipFrame.Instance.GossipOptionEntries
                                                .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Trainer);
                        await Sleep();
                        GossipFrame.Instance.SelectGossipOption(_trainerEntry.Index);
                        await Coroutine.Wait(3 * 1000, () => IsFrameVisible("ClassTrainerFrame"));
                    }

                    TrainerFrame.Instance.SetServiceFilter(TrainerServiceFilter.Available, true);
                    TrainerFrame.Instance.SetServiceFilter(TrainerServiceFilter.Unavailable, false);
                    TrainerFrame.Instance.SetServiceFilter(TrainerServiceFilter.Used, false);

                    while (IsFrameVisible("ClassTrainerScrollFrameButton1") && !breakOut)
                    {

                        await ClickButton("ClassTrainerFrameSkillStepButton");
                        await ClickButton("ClassTrainerTrainButton");

                        if (IsFrameVisible("StaticPopup1"))
                            await ClickButton("StaticPopup1Button1");

                        if (_skill == SkillLine.Riding)
                            break;

                        await Coroutine.Yield();
                    }

                    await Coroutine.Wait(3 * 1000, () => Me.GetSkill(_skill).MaxValue != _preMax);
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleTrainArchaeology()
        {
            if (!Settings.Archaeology_Train)
                return false;

            /*
            if (Me.GetSkill(SkillLine.Archaeology).CurrentValue == 0)
                return false;
                */

            if (Me.GetSkill(SkillLine.Archaeology).MaxValue >= 700)
                return false;

            RuntimeLog(">>");

            /*
            if (!SkillNeedsTraining(SkillLine.Archaeology, true) && false)
            {
                if (HearthState == Hearth.Off)
                    _checkedArchaeologySkill = true;
                return false;
            }

            if (!_inCapital)
            {
                if (Settings.Hearth_Hearthstone &&
                    !Settings.Hearth_SetHearthstone &&
                    UnfavourableAreaSet())
                    return false;

                DevLog("Hearth.Capital: TrainArchaeologySkill");
                Log("Preparing for training archaeology skill");
                HearthState = Hearth.Capital;
                return false;

            }

            var _skill = SkillLine.Archaeology;
            var _trainerDataEntry = _trainerData.FirstOrDefault(e => (int)(e[0]) == Me.ZoneId &&
                                                                     (SkillLine)(e[1]) == SkillLine.Archaeology);
            var _npcId = (int)_trainerDataEntry[2];
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            */

            var _skill = SkillLine.Archaeology;
            var _trainerDataEntry = GetClosestTrainerEntry(_skill);
            if (_trainerDataEntry == null)
                return false;

            var _npcId = (int)_trainerDataEntry[0];
            var _npcLocation = (Vector3)_trainerDataEntry[4];

            await TrainSkill(_skill, _npcId, _npcLocation);

            return true;
        }

        private WoWUnit _vendor
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         u.IsRepairMerchant &&
                         u.WithinInteractRange &&
                         u.IsValid &&
                         u.IsAlive
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private IEnumerable<WoWItem> _trash
        {
            get
            {
                return Me.BagItems.Where(i => i.IsValid &&
                                                i.Quality == WoWItemQuality.Poor);
            }

        }

        private async Task<bool> HandleSellTrash(bool _force = false)
        {
            RuntimeLog(">>");

            if (_trash.Count() == 0)
                return false;

            if (_trash.Count() < 10 && !_force)
                return false;

            RuntimeLog(">>");

            var _vendorMountSpells = new List<int>()
            {
                122708,
                61447,
                61425
            };
            var _vendorMountSpellId = _vendorMountSpells.FirstOrDefault(s => SpellManager.HasSpell(s) &&
                                                                             SpellManager.CanCast(s));
            if (_vendorMountSpellId == 0)
                return false;

            RuntimeLog(">>");

            Log("Selling trash");
            while (!IsFrameVisible("MerchantFrame") && !breakOut)
            {
                DevLog("WHILE: HandleSellTrash");

                if (_vendor == null)
                {
                    await CommonCoroutines.LandAndDismount();
                    await CommonCoroutines.StopMoving();

                    if (!Me.IsCasting)
                        SpellManager.CastSpellById(_vendorMountSpellId);
                    await Coroutine.Wait(5000, () => _vendor != null);

                    await Coroutine.Yield();
                }
                else
                {
                    _vendor.Interact();
                    await Coroutine.Wait(5000, () => IsFrameVisible("MerchantFrame"));
                }

                await Coroutine.Yield();
            }

            if (IsFrameVisible("MerchantFrame"))
            {
                await Vendors.SellItems(_trash);
                if (!await Coroutine.Wait(5000, () => _trash == null))
                {
                    var _item = _trash.FirstOrDefault();
                    while (_item != null && !breakOut)
                    {
                        DevLog("WHILE: HandleSellTrash > Destroy");

                        await Sleep();
                        _item.PickUp();
                        Lua.DoString("DeleteCursorItem();");
                        await Coroutine.Wait(2000, () => !HasItem((int)_item.Entry));

                        _item = _trash.FirstOrDefault();
                        await Coroutine.Yield();
                    }
                }
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleArchaeologyLoot()
        {
            if (!CharacterSettings.Instance.LootMobs)
                return false;

            CharacterSettings.Instance.LootMobs = false;

            return true;
        }

        private WoWUnit _ancientHaunt
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         u.Entry == 62596 &&
                         (
                            u.IsAlive ||
                            (
                                u.CanLoot &&
                                u.Lootable &&
                                !u.Looting &&
                                !u.IsAlive
                            )
                         ) &&
                         u.IsValid
                     orderby
                        u.Distance
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private async Task<bool> HandleAncientHaunt()
        {
            if (_ancientHaunt == null)
                return false;

            Log("Looting ancient haunt");
            while (_ancientHaunt != null && !breakOut)
            {
                DevLog("WHILE: HandleAncientHaunt");

                if (!_ancientHaunt.WithinInteractRange)
                    await MoveTo(_ancientHaunt.Location);
                else
                {
                    await CommonCoroutines.StopMoving();

                    _ancientHaunt.Interact();
                    await Loot();
                    await Coroutine.Wait(3000, () => _ancientHaunt == null);
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleBuyFragments()
        {
            if ((int)Settings.Archaeology_Fragments == 0)
                return false;

            if (Me.CurrentMap.ExpansionId != 4)
                return false;

            if (!Flightor.CanFly)
                return false;

            if (Me.GetSkill(SkillLine.Archaeology).CurrentValue < 600)
                return false;

            var _restoredArtifactId = 87399;
            var _restoredArtifact = GetItem(_restoredArtifactId);
            if (_restoredArtifact == null)
                return false;

            var _npcId = 64922;
            var _npc = GetNpc(_npcId);
            var _npcLocation = Query.GetNpcById((uint)_npcId).Location;
            if (Me.Location.Distance(_npcLocation) > 600 && GetItemCount(_restoredArtifactId) < 10)
                return false;

            var _fragmentContainerId = (int)Settings.Archaeology_Fragments;
            var _fragmentContainer = GetItem(_fragmentContainerId);

            Log("Buying fragments");
            while (_restoredArtifact != null && !breakOut)
            {
                DevLog("WHILE: HandleBuyFragments");

                _npc = GetNpc(_npcId);
                if (_npc != null)
                    _npcLocation = _npc.Location;

                if (_npc == null || !_npc.WithinInteractRange)
                    await MoveTo(_npcLocation);
                else
                {
                    if (!IsFrameVisible("MerchantFrame"))
                    {
                        await CommonCoroutines.StopMoving();
                        await Sleep();
                        _npc.Interact();

                        await Coroutine.Wait(3 * 1000, () => IsFrameVisible("MerchantFrame") || IsFrameVisible("GossipFrame"));
                        if (IsFrameVisible("GossipFrame"))
                        {
                            var _vendorEntry = GossipFrame.Instance.GossipOptionEntries
                                                    .FirstOrDefault(e => e.Type == GossipEntry.GossipEntryType.Vendor);
                            await Sleep();
                            GossipFrame.Instance.SelectGossipOption(_vendorEntry.Index);
                            await Coroutine.Wait(3 * 1000, () => IsFrameVisible("MerchantFrame"));
                        }
                    }
                    else
                    {
                        if (_trash.Count() > 0)
                            await Vendors.SellItems(_trash);

                        var _allItems = MerchantFrame.Instance.GetAllMerchantItems().ToList();
                        var _index = _allItems.FirstOrDefault(i => i.ItemId == _fragmentContainerId).Index;
                        MerchantFrame.Instance.BuyItem(_index, GetItemCount(_restoredArtifactId));

                        await Coroutine.Wait(3000, () => GetItem(_restoredArtifactId) == null);
                        _restoredArtifact = GetItem(_restoredArtifactId);
                    }
                }
                _checkedItems = false;
                await Coroutine.Yield();
            }

            return true;
        }

        private List<int> _races = new List<int>
        {
            1174,   // 1 - Demonic
            1173,   // 2 - HighmountainTauren
            1172,   // 3 - Highborne
            828,    // 4 - Ogre
            821,    // 5 - DraenorClans
            829,    // 6 - Arakkoa
            677,    // 7 - Mogu
            676,    // 8 - Pandaren
            754,    // 9 - Mantid
            399,    // 10 - Vyrkul
            385,    // 11 - Troll
            401,    // 12 - Tol'vir
            397,    // 13 - Orc
            400,    // 14 - Nerubian
            394,    // 15 - NightElf
            393,    // 16 - Fossil
            398,    // 17 - Draenei
            384     // 18 - Dwarf
        };

        private int GetRaceIndex(WoWCurrency _currency)
        {
            return _races.IndexOf((int)_currency.CurrencyType) + 1;
        }

        private bool CurrencyIsCritical(WoWCurrency _currency)
        {
            return _currency.CurrencyType.ToString().Contains("ArchaeologyFragment") &&
                   _currency.Amount >= 190;
        }

        private async Task<bool> HandleSolveArtifact()
        {
            var _currency = WoWCurrency.GetDiscoveredCurrencies()
                                       .FirstOrDefault(c => CurrencyIsCritical(c));
            if (_currency == null)
                return false;

            Log("Solving artifact");
            WoWCurrency _lastCurrency = null;
            while (_currency != null && !breakOut)
            {
                DevLog("WHILE: HandleSolveArtifact");

                if (_currency != _lastCurrency)
                {
                    Log("     {0} (Id: {1})", _currency.Name, _currency.Entry);
                    _lastCurrency = _currency;
                }

                await Sleep();
                Lua.DoString($"SolveArtifact({GetRaceIndex(_currency)})");

                var _preAmount = _currency.Amount;
                if (await Coroutine.Wait(2 * 1000, () => Me.IsCasting))
                    if (await Coroutine.Wait(5 * 1000, () => !Me.IsCasting))
                        await Coroutine.Wait(2 * 1000, () => _preAmount != _currency.Amount);

                _currency = WoWCurrency.GetDiscoveredCurrencies()
                                       .FirstOrDefault(c => CurrencyIsCritical(c));
                await Coroutine.Yield();
            }

            await HandleSorting();
            return true;
        }

        private async Task<bool> HandleArchaeology()
        {
            if (BotManager.Current.ToString() != "ArchaeologyBuddy")
                return false;

            RuntimeLog(">");

            if (await HandleBuyFragments())
                return true;

            RuntimeLog(">");

            if (await HandleSellTrash())
                return true;

            RuntimeLog(">");

            if (await HandleAncientHaunt())
                return true;

            RuntimeLog(">");

            await HandleArchaeologyLoot();

            RuntimeLog(">");

            if (_checkedArchaeologySkill)
                return false;

            RuntimeLog(">");

            if (await HandleTrainArchaeology())
                return true;

            RuntimeLog(">");

            if (await HandleSwitchContinents())
                return true;

            RuntimeLog(">");

            return false;
        }

        private string TimeString(double _seconds)
        {
            _seconds = Math.Round(_seconds);

            if (_seconds < 60)
                return $"{_seconds} seconds";
            else
            {
                var _minutes = Math.Floor(_seconds / 60);
                _seconds = _seconds % 60;

                return String.Format("{0}:{1} minutes", _minutes, (_seconds < 10)
                                                                        ? $"0{_seconds}"
                                                                        : $"{_seconds}");
            }
        }

        private async Task<bool> HandleObjectiveChange()
        {
            if (Settings.Motionless_ObjectiveTimer == 0)
                return false;

            if (_lastObjective != TreeRoot.GoalText || _timeLastObjectiveChange == DateTime.MinValue)
            {
                _timeLastMoved = DateTime.Now;
                _lastObjective = TreeRoot.GoalText;
                return false;
            }

            if (DeltaTime(_timeLastObjectiveChange, "s") < Settings.Motionless_ObjectiveTimer)
                return false;

            DevLog("ObjectiveChange");
            return true;
        }

        private async Task<bool> HandleNoMovement()
        {
            RuntimeLog(">>");

            if (Settings.Motionless_Timer == 0)
                return false;

            RuntimeLog(">>");

            if (Me.IsMoving || _timeLastMoved == DateTime.MinValue ||
                Me.OnTaxi || Me.IsOnTransport)                           //+++ || IsFrameVisible("OverrideActionBar"))
            {
                _timeLastMoved = DateTime.Now;
                return false;
            }

            RuntimeLog(">>");

            if (!Me.IsAlive)
                return false;

            RuntimeLog(">>");

            var _beginMotionless = (BotManager.Current.ToString() == "Questing" &&
                                    ProfileManager.XmlLocation != null &&
                                    ProfileManager.XmlLocation.ToLower().Contains("questing profile pack"))
                                                ? 330
                                                : Settings.Motionless_Timer;

            RuntimeLog(">>");

            if (DeltaTime(_timeLastMoved, "s") < _beginMotionless)
            {
                var _passedTime = Math.Floor(DeltaTime(_timeLastMoved, "s"));
                if (_passedTime >= 10 && _logTimer.IsFinished)
                {
                    var _remainingTime = _beginMotionless - _passedTime;

                    DevLog("MOTIONLESS in {0}", TimeString(_remainingTime));
                    DevLog("     No movement for {0}", TimeString(_passedTime));

                    _logTimer.Reset();
                }
                return false;
            }

            DevLog("NoMovement");
            return true;
        }

        private async Task<bool> HandleReloadStage(int _increaseDistanceFactor = 0)
        {
            if (StuckState != Stuck.Stage6 &&
                MotionlessState != Motionless.Stage1)
                return false;

            if (Me.Location.Distance(_lastStuckLocation) >= (_increaseDistanceFactor + 1) * _defaultSpeed)
            {
                Reset();
                return false;
            }

            if (DeltaTime(_timeReload, "s") < 45)
            {
                var _passedTime = Math.Floor(DeltaTime(_timeReload, "s"));
                if (_passedTime >= 10 && _logTimer.IsFinished)
                {
                    var _remainingTime = 45 - _passedTime;

                    DevLog("UNSTUCK in {0}", TimeString(_remainingTime));
                    DevLog("     No movement for {0}", TimeString(_passedTime));

                    _logTimer.Reset();
                }

                return false;
            }

            return true;
        }

        private async Task<bool> JumpToUnstuckStage()
        {
            DevLog("Stuck handling disabled: {0}", Settings.Stuck_Disable);
            Log("     Stage 2");

            MotionlessState = Motionless.Stage2;
            StuckState = Stuck.On;

            if (!await DoUnstuck(1, true))
            {
                DevLog("Motionless: Unstuck unsuccessful > Hearth");
                await JumpToHearthStage();
                return false;
            }
            else
            {
                DevLog("Motionless: Unstuck successful > Wait 20 seconds");

                await CommonCoroutines.StopMoving();
                if (!await Coroutine.Wait(20 * 1000, () => Me.IsMoving))
                {
                    DevLog("Motionless: Still not moving > Hearth");
                    await JumpToHearthStage();
                    return false;
                }
            }

            Reset();
            return false;
        }

        private class EscapeCombatAbility
        {
            public EscapeCombatAbility(  bool _condition,
                                         string _spellName)
            {
                Condition = _condition;
                SpellName = _spellName;
            }

            public bool Condition { get; }
            public string SpellName { get; }
        }

        private List<EscapeCombatAbility> _escapeCombatAbilityData = new List<EscapeCombatAbility>()
        {
            new EscapeCombatAbility( Me.Class == WoWClass.Rogue,     "Vanish" ),
            new EscapeCombatAbility( Me.Class == WoWClass.Hunter,    "Feign Death" ),
            new EscapeCombatAbility( Me.Class == WoWClass.Mage,      "Invisility" ),
            new EscapeCombatAbility( Me.Race  == WoWRace.NightElf,   "Shadowmeld" ),
        };

        private async Task<bool> EscapeCombat()
        {
            if (!Me.Combat)
                return false;

            var _escapeCombatAbility = _escapeCombatAbilityData
                                            .FirstOrDefault(a => a.Condition &&
                                                                 SpellManager.HasSpell(a.SpellName) &&
                                                                 SpellManager.CanCast(a.SpellName));
            if (_escapeCombatAbility == null)
                return false;

            var _spellName = _escapeCombatAbility.SpellName;
            SpellManager.Cast(_spellName);
            await Coroutine.Wait(3 * 1000, () => !SpellManager.CanCast(_spellName));

            return true;
        }

        private async Task<bool> HandleMotionless()
        {
            RuntimeLog(">");

            if (await HandleReloadStage(1))
            {
                await JumpToUnstuckStage();
                return false;
            }
            else if ((int)MotionlessState >= 2)
                return false;

            RuntimeLog(">");

            if ((int)MotionlessState >= 2)
                return false;

            if (Me.HasAura("Resurrection Sickness") || IsFrameVisible("MerchantFrame")) // || Me.IsActuallyInCombat)
            {
                _timeLastMoved = DateTime.Now;
                _timeLastObjectiveChange = DateTime.Now;
                return false;
            }

            if (!await HandleNoMovement() &&
                !await HandleObjectiveChange())
                return false;

            RuntimeLog(">");

            Log("MOTIONLESS! Bringing back movement...");
            Log("     Stage 1");

            PrintInfo(false);

            MotionlessState = Motionless.Stage1;
            _lastStuckLocation = Me.Location;
            _timeReload = DateTime.Now;

            await EscapeCombat();
            ReloadProfile();

            return true;
        }

        private async Task<bool> HandleFeralCombat()
        {
            return false;

            if (Me.Class != WoWClass.Druid)
                return false;

            if (Me.Specialization != WoWSpec.DruidFeral)
                return false;

            if (Me.Shapeshift == ShapeshiftForm.Cat)
                return false;

            if (Me.Mounted)
                return false;

            RuntimeLog(">");

            if (Me.Mounted && Mount.Current.IsFlyingMount && !Me.HasAura("Stag Form"))
                return false;

            RuntimeLog(">");

            if (!Me.Combat)
                return false;

            RuntimeLog(">");

            if (Me.IsActuallyInCombat)
                return false;

            RuntimeLog(">");

            if (!SpellManager.CanCast(768))
                return false;

            RuntimeLog(">");

            Log("Forcing cat form");

            SpellManager.CastSpellById(768);
            await Coroutine.Wait(2000, () => Me.Shapeshift == ShapeshiftForm.Cat);

            return true;
        }

        private async Task<bool> HandleMeleeDismount()
        {
            if (!Me.IsMelee)
                return false;

            if (!_meleeDismount)
                return false;

            if (!Me.Mounted)
                return false;

            if (Me.CurrentTarget == null)
                return false;

            Log("Dismounting near target");

            Navigator.Clear();
            Flightor.Clear();

            var _targetLocation = Me.CurrentTarget.Location;
            while (Me.Mounted && !breakOut)
            {
                if (Me.Location.Distance(_targetLocation) > 5)
                    await MoveTo(_targetLocation);
                else
                {
                    await CommonCoroutines.LandAndDismount(null, true, _targetLocation);
                    await CommonCoroutines.StopMoving();

                    var _target = Me.CurrentTarget;
                    if (_target != null)
                        Lua.DoString("StartAttack()");
                }

                await Coroutine.Yield();
            }

            _meleeDismount = false;
            return true;
        }

        private async Task<bool> HandleReloadProfile()
        {
            if (!_reloadProfile)
                return false;

            if (ActuallyInCombat())
                return false;

            Log("Reloading profile");
            ReloadProfile();
            _reloadProfile = false;

            return true;
        }

        /*
        private bool QuestCanBeTurnedIn(PlayerQuest _quest)
        {
            if (!_quest.IsCompleted)
                return false;

            var _npcId = GetTurnInNpcId((int)_quest.Id);
            if (_npcId == -1)
                return false;

            var _npcResult = Query.GetNpcById((uint)_npcId);
            if (_npcResult == null)
                return false;

            if (_npcResult.MapId != Me.MapId)
                return false;

            return true;
        }

        private double GetTurnInDistance(PlayerQuest _quest)
        {
            var _npcId = GetTurnInNpcId((int)_quest.Id);
            if (_npcId == -1)
                return -1;

            var _npcResult = Query.GetNpcById((uint)_npcId);
            if (_npcResult == null)
                return -1;

            var _npcLocation = _npcResult.Location;
            if (_npcLocation == Vector3.Zero)
                return -1;

            return Me.Location.Distance(_npcLocation);
        }

        private int GetTurnInQuestId()
        {
            var _quests = QuestLog.Instance.GetAllQuests()
                                    .Where(q => QuestCanBeTurnedIn(q) &&
                                                GetTurnInDistance(q) > -1)
                                    .OrderBy(q => GetTurnInDistance(q));

            if (_quests.Count() == 0)
                return -1;

            return (int)_quests.FirstOrDefault().Id;
        }

        private async Task<bool> HandleCompletedQuests(bool _force = false)
        {
            return false;

            if (!_force)
            {
                if (!_profileChange)
                    return false;

                if (Me.Combat)
                    return false;

                if (!Me.IsMoving)
                    return false;

                var _profile = ProfileManager.XmlLocation;
                if (_lastProfile == null && !_profile.ToLower().Replace("auto loader", "autoloader").Contains("autoloader"))
                    _lastProfile = _profile;

                if (_lastProfile == _profile)
                    return false;
            }

            var _questId = GetTurnInQuestId();
            if (_questId == -1)
                return false;

            while (_questId != -1 && !breakOut)
            {
                DevLog("WHILE: HandleCompletedQuests");

                var _npcId = GetTurnInNpcId(_questId);
                await TurnInQuest(_questId, _npcId);

                _questId = GetTurnInQuestId();
                await Coroutine.Yield();
            }

            _lastProfile = ProfileManager.XmlLocation;
            _profileChange = false;
            return true;
        }
        */

        private bool InVehicle()
        {
            if (_timeEnterVehicle == DateTime.MinValue &&
                _timeExitVehicle == DateTime.MinValue)
            {
                if (Me.InVehicle)
                    _timeEnterVehicle = DateTime.Now;
                else
                    _timeExitVehicle = DateTime.Now;
            }

            return DeltaTime(_timeEnterVehicle, "ms") < DeltaTime(_timeExitVehicle, "ms");
        }

        /*
        private async Task<bool> HandleNavType()
        {
            if (BotPoi.Current.NavType == NavType.Run &&
                ProfileManager.CurrentProfile.NavType == NavType.Run)
                return false;

            BotPoi.Current.NavType = NavType.Run;
            ProfileManager.CurrentProfile.NavType = NavType.Run;

            return true;
        }

        private async Task<bool> HandleLegionFlying()
        {
            await HandleNavType();
            await HandleFlightor();

            return true;
        }

        private async Task<bool> HandleFlightor()
        {
            Flightor.Clear();

            if (!_lastLogEntry.Contains("[Flightor] Set up for movement from"))
                return false;

            if (_intervalLastFlightorLog > 2000)
            {
                _intervalLastFlightorLog = 0;
                Log("Forcing ground navigation");
            }

            await MountUp(null, 0, true);

            var _endLocation = ExtractVector3(_lastLogEntry, 2);
            if (_endLocation == Vector3.Zero)
                return false;

            Navigator.MoveTo(_endLocation);

            return true;
        }
        */

        private Vector3 ExtractVector3(string _string, int _index = 1)
        {
            if (_string == null)
                return Vector3.Zero;

            var _regex = new Regex("(?<=<)[^a-z]+(?=>)");
            var _result = _regex.Matches(_string);
            if (_result.Count == 0)
                return Vector3.Zero;

            _regex = new Regex("[-0-9.]+");
            var _safeIndex = Math.Min(_index, _result.Count) - 1;
            _result = _regex.Matches(_result[_safeIndex].ToString());
            if (_result.Count != 3)
                return Vector3.Zero;

            var _vector = new Vector3(float.Parse(_result[0].ToString()),
                                      float.Parse(_result[1].ToString()),
                                      float.Parse(_result[2].ToString()));
            return _vector;
        }

        private async Task<bool> HandleForceTravelForm(int _groundMount, int _flyingMount)
        {
            RuntimeLog(">");

            if (!Me.Mounted)
                return false;

            RuntimeLog(">");

            var _aura = Me.GetAuraById(783);
            if (_aura != null)
                return false;

            RuntimeLog(">");

            if (!Me.IsMoving)
                return false;

            RuntimeLog(">");

            if (Me.IsFlying && _flyingMount != 783)
                return false;

            RuntimeLog(">");

            if (!Me.IsFlying && _groundMount != 783)
                return false;

            RuntimeLog(">");

            if (!SpellManager.CanCast(783))
                return false;

            RuntimeLog(">");

            if (Flightor.CanFly && !Me.IsFlying)
                return false;

            RuntimeLog(">");

            Log("Switching to \"Travel Form\"");
            SpellManager.CastSpellById(783);

            return true;
        }

        private async Task<bool> HandleTravelFormSettings(int _groundMount, int _flyingMount)
        {
            var _setting = (int)Settings.General_TravelForm;
            if (_setting == 0)
                return false;

            if (_groundMount == 783 && _flyingMount == 783)
                return false;

            CharacterSettings.Instance.GroundMountSpellId = (_setting == 1 || _setting == 3)
                ? 783
                : _groundMount;
            CharacterSettings.Instance.FlyingMountSpellId = (_setting == 2 || _setting == 3)
                ? 783
                : _flyingMount;

            return true;
        }

        private async Task<bool> HandleTravelForm()
        {
            if (!SpellManager.HasSpell(783))
                return false;

            var _groundMount = CharacterSettings.Instance.GroundMountSpellId;
            var _flyingMount = CharacterSettings.Instance.FlyingMountSpellId;

            await HandleForceTravelForm(_groundMount, _flyingMount);
            await HandleTravelFormSettings(_groundMount, _flyingMount);

            return true;
        }

        private static unsafe void ReplaceMethod(MethodInfo _oldMethod, MethodInfo _newMethod)
        {
            var _oldMethodAddress = new IntPtr((int*)_oldMethod.MethodHandle.Value.ToPointer() + 2);
            var _destination = (uint*)_oldMethodAddress.ToPointer();
            *_destination = (uint)_newMethod.MethodHandle.GetFunctionPointer().ToInt32();
        }

        private async Task<bool> HandleLegionFlying()
        {
            if (!Settings.General_DisableLegionFlying)
                return false;

            if (Me.CurrentMap.ExpansionId != 6)
                return false;

            if (!Flightor.CanFly)
                return false;

            DevLog("Replace CanFly");
            try
            {
                var _currentMethod = typeof(Flightor).GetProperty("CanFly", BindingFlags.Public | BindingFlags.Static).GetMethod;
                var _newMethod = typeof(KOMFlightor).GetProperty("CanFly", BindingFlags.Public | BindingFlags.Static).GetMethod;
                ReplaceMethod(_currentMethod, _newMethod);

                DevLog("     Successful");
            }
            catch (Exception _ex)
            {
                DevLog("     Failed");
                DevLog("          {0}", _ex.ToString());
            }

            return true;
        }

        private bool InQueue()
        {
            var _state = Me.LfgMode(LfgCategory.LookingForDungeon);
            return _state == LfgState.InQueue;
        }

        private async Task<bool> HandleDungeonProposal()
        {
            if (DungeonState != Dungeon.Proposal)
                return false;

            if (Me.ZoneId == 7673)
                return false;

            if (Me.IsActuallyInCombat)
                return false;

            Log("Waiting while proposal is shown");
            await CommonCoroutines.StopMoving();
            if (await Coroutine.Wait(60 * 1000, () => !StyxWoW.IsInWorld ||
                                                      DungeonState == Dungeon.InQueue ||
                                                      !_init))
                await Coroutine.Wait(3 * 60 * 1000, () => Me.ZoneId == 7673 ||
                                                          !_init);

            return true;
        }

        private async Task<bool> HandleDungeonQueue()
        {
            if ((int)DungeonState > 1)
                return false;

            if (InQueue())
                return false;

            if (DungeonState == Dungeon.Proposal)
                return false;

            if (Me.HasAura("Dungeon Deserter"))
                return false;

            Log("Queueing for \"Darkhearth Thicket\"");
            Lua.DoString(@"
                    SetLFGRoles(false, false, false, true);
                    SetLFGDungeon(1, 1201);
                    JoinLFG(1);
            ");

            if (await Coroutine.Wait(3 * 1000, () => InQueue()) )
            {
                DungeonState = Dungeon.InQueue;
                return true;
            }
            else
                return false;
        }

        private class Lockout
        {
            public Lockout( Vector3 _outside,
                            Vector3 _inside)
            {
                Outside = _outside;
                Inside = _inside;
            }

            private Vector3 Outside { get; }
            private Vector3 Inside { get; }

            public bool Active
            {
                get
                {
                    var _worldLines = new List<WorldLine>();

                    var _inFront = WoWMathHelper.CalculatePointInFront(Me.Location + new Vector3(0, 0, 5), Me.Rotation, 3);
                    _worldLines.Add(new WorldLine(Me.Location, _inFront));
                    _worldLines.Add(new WorldLine(Outside, Inside));

                    GameWorld.MassTraceLine(_worldLines.ToArray(), TraceLineHitFlags.Collision, out var _hitResult, out var _hitLocations);
                    var _allObstructed = _hitResult.All(r => r);
                    if (!_allObstructed)
                        return false;

                    var _center = _hitLocations[1];
                    var _distanceToCenter = _inFront.Distance(_center);
                    if (_distanceToCenter > 5)
                        return false;

                    return true;
                }
            }
        }

        private List<Lockout> _lockoutData = new List<Lockout>()
        {
            new Lockout(new Vector3(2939.204f, 1964.933f, 189.7318f),   // 1st BOSS
                        new Vector3(2927.79f, 1965.561f, 189.7134f) ),
            new Lockout(new Vector3(3065.992f, 1531.74f, 137.7991f),    // 3rd BOSS
                        new Vector3(3047.269f, 1517.997f, 138.0178f) ),
            new Lockout(new Vector3(2746.549f, 1381.367f, 136.2714f),   // 4th BOSS
                        new Vector3(2737.434f, 1367.72f, 128.3686f) )
        };

        private async Task<bool> HandleLockouts()
        {
            var _lockout = _lockoutData.FirstOrDefault(l => l.Active);
            if (_lockout == null)
                return false;

            Log("Being locked out of boss fight. Waiting until combat ends");

            await CommonCoroutines.StopMoving();
            await Coroutine.Wait(3 * 60 * 1000, () => !_lockout.Active || !_init);

            return true;
        }

        public void SetDungeonBuddySettings()
        {
            DungeonBuddySettings.Instance.SelectedDungeonIds = new uint[] { 1201 };
            DungeonBuddySettings.Instance.Role = DungeonBuddySettings.QueueRole.Damage;
            DungeonBuddySettings.Instance.TankInRandomGroups = false;
            DungeonBuddySettings.Instance.UseArtifactPowerItems = true;
            DungeonBuddySettings.Instance.GroupLootMode = GroupLootMode.Personal;
            DungeonBuddySettings.Instance.LootMode = LootMode.BossesOnly;
            DungeonBuddySettings.Instance.MailBoeItems = false;
            DungeonBuddySettings.Instance.SellItemQuality = WoWItemQuality.Poor;
            DungeonBuddySettings.Instance.MailItemQuality = WoWItemQuality.Poor;
            DungeonBuddySettings.Instance.AbandonLowLevelQuests = false;
            DungeonBuddySettings.Instance.CombatMovement = false;
            DungeonBuddySettings.Instance.FollowingDistance = 10;
            DungeonBuddySettings.Instance.HearthOnVendorRun = false;
            DungeonBuddySettings.Instance.KillOptionalBosses = false;
            DungeonBuddySettings.Instance.PickupQuests = false;
            DungeonBuddySettings.Instance.PortOutsideWhenMemberReleasesSpirit = false;
            DungeonBuddySettings.Instance.QueueForBestAlternative = false;
            DungeonBuddySettings.Instance.PartyMode = PartyMode.Off;
            DungeonBuddySettings.Instance.InsideInstanceMinFreeBagSlots = 3;
            DungeonBuddySettings.Instance.InsideInstanceRepairPercent = 10;
            DungeonBuddySettings.Instance.OutsideInstanceMinFreeBagSlots = 15;
            DungeonBuddySettings.Instance.OutsideInstanceRepairPercent = 50;
            DungeonBuddySettings.Instance.PortOutsideInstanceToBuy = false;
            DungeonBuddySettings.Instance.PortOutsideInstanceToSell = false;
            DungeonBuddySettings.Instance.RepairWithGuildBankFunds = false;
        }

        public void DisablePugWarning()
        {
            var _file = $"{Application.StartupPath}\\..\\Dungeon Scripts\\Legion\\Dungeons\\DarkheartThicket.cs";
            var _content = File.ReadAllText(_file);

            var _searchFor = "if (displayPugGroupWarning)";
            var _replaceWith = "if (false && displayPugGroupWarning)";

            var _result = _content.Replace(_searchFor, _replaceWith);
            File.WriteAllText(_file, _result);
        }

        private async Task<bool> HandleDungeonInDungeon()
        {
            if (Me.ZoneId != 7673)
                return false;

            if (BotManager.Current.ToString() != "Dungeonbuddy")
            {
                Log("Switching to Dungeonbuddy");

                KOMApi._lastProfile = ProfileManager.XmlLocation;
                KOMApi._lastBotBaseName = BotManager.Current.Name;

                KOMApi.ChangeBotBase("Dungeonbuddy");
                await Coroutine.Wait(30 * 1000, () => BotManager.Current.Name == "Dungeonbuddy");

                return true;
            }

            if (DungeonState != Dungeon.InDungeon)
                DungeonState = Dungeon.InDungeon;

            await HandleStuck();
            await HandleInit();
            if (!await HandleTps())
                return true;

            await HandleGraphicMode();
            await HandleLockouts();

            return true;
        }

        private async Task<bool> HandleDungeonQuest()
        {
            if (!Settings.GatheringProfessions_DungeonQuest)
                return false;

            var _questId = 38799;
            if (!HasQuest(_questId))
                return false;

            if (IsQuestCompleted(_questId))
            {
                if (BotManager.Current.ToString() == "Dungeonbuddy")
                {
                    await Sleep();
                    Lua.DoString("LeaveParty()");

                    await Coroutine.Wait(10 * 1000, () => !StyxWoW.IsInWorld);
                    await Coroutine.Wait(3 * 60 * 1000, () => Me.ZoneId != 7673 && StyxWoW.IsInWorld);

                    KOMApi.SwitchBack();
                    await Coroutine.Wait(3 * 60 * 1000, () => BotManager.Current.Name != "Dungeonbuddy");
                    return false;
                }
            }

            await HandleDungeonQueue();
            await HandleDungeonProposal();

            if (await HandleDungeonInDungeon())
                return true;

            return false;
        }

        private class ClassHall
        {
            public ClassHall(WoWClass _class,
                             uint _zoneId,
                             int _forgeObjectId,
                             Vector3 _forgeLocation,
                             bool _hasTravelScript)
            {
                Class = _class;
                ZoneId = _zoneId;
                ForgeObjectId = _forgeObjectId;
                ForgeLocation = _forgeLocation;
                HasTravelScript = _hasTravelScript;
            }

            public WoWClass Class { get; }
            public uint ZoneId { get; }
            public int ForgeObjectId { get; }
            public Vector3 ForgeLocation { get; }
            public bool HasTravelScript { get; }
        }

        private List<ClassHall> _classHallData = new List<ClassHall>()
        {
            new ClassHall(WoWClass.Druid,       7846,   244734, new Vector3(3959.931f, 7515.002f, 52.96376f),   true ),
            new ClassHall(WoWClass.Warrior,     7813,   245726, new Vector3(1030.769f, 7361.063f, 107.3078f),   false ),
            new ClassHall(WoWClass.Mage,        7879,   245006, new Vector3(-816.5347f, 4692.826f, 939.6634f),  false ),
            new ClassHall(WoWClass.Hunter,      7877,   245037, new Vector3(4499.827f, 5250.55f, 863.7143f),    false ),
			new ClassHall(WoWClass.DemonHunter, 8022, 	250244, new Vector3(1634.911f, 1414.202f, 209.0089f),   false ),
            new ClassHall(WoWClass.DeathKnight, 7679,   245433, new Vector3(-1538.837f, 999.9219f, 313.6087f),  false ),
            new ClassHall(WoWClass.Monk,        7902,   245098, new Vector3(942.2118f, 3604.37f, 196.174f),     false ),
        };

        // Shaman
        // Priest
        // Warlock
        // Rogue
        // Paladin

        private class Trait
        {
            public Trait(int _id,
                         int _tier,
                         int _cost,
                         int _currentRank,
                         int _maxRank,
                         int _bonusRanks,
                         bool _prereqsMet,
                         bool _isStart,
                         bool _isGoldMedal,
                         int _spellId)
            {
                Id = _id;
                Tier = _tier;
                Cost = _cost;
                CurrentRank = _currentRank;
                MaxRank = _maxRank;
                BonusRanks = _bonusRanks;
                PrereqsMet = _prereqsMet;
                IsStart = _isStart;
                IsGoldMedal = _isGoldMedal;
                SpellId = _spellId;
            }

            public int Id { get; }
            public int Tier { get; }
            public int Cost { get; }
            public int CurrentRank { get; }
            public int MaxRank { get; }
            public int BonusRanks { get; }
            public bool PrereqsMet { get; }
            public bool IsStart { get; }
            public bool IsGoldMedal { get; }
            public int SpellId { get; }

            public override string ToString()
            {
                return String.Format("Id: {0}, Tier: {1}, Cost: {2}, CurrentRank: {3}, MaxRank: {4}, BonusRanks: {5}, PrereqsMet: {6}, IsStart: {7}, IsGoldMedal: {8}, Spell: {9} (Id: {10})",
                                        Id,
                                        Tier,
                                        Cost,
                                        CurrentRank,
                                        MaxRank,
                                        BonusRanks,
                                        PrereqsMet,
                                        IsStart,
                                        IsGoldMedal,
                                        WoWSpell.FromId(SpellId).Name,
                                        SpellId);
            }
        }

        private List<Trait> GetTraits()
        {
            var _lua = @"
	            local data = {{}}

	            local addToTable = function(...)
		            local row = {}
		            for k, v in pairs({...}) do
			            table.insert(row, tostring(v) )
		            end
		            table.insert(data, table.concat(row, "","") )
	            end

	            for k, v in pairs(C_ArtifactUI.GetPowers()) do
		            local info = C_ArtifactUI.GetPowerInfo(v)
		            addToTable(v,
                               info.tier,
				               info.cost,
				               info.currentRank,
				               info.maxRank,
				               info.bonusRanks,
				               info.prereqsMet,
				               info.isStart,
				               info.isGoldMedal,
                               info.spellID)
	            end

	            return unpack(data)
            ";
            var _data = Lua.GetReturnValues(_lua);
            _data.RemoveAt(0);
            var _traitData = new List<Trait>();

            foreach (var _entry in _data)
            {
                var _delimiter = ',';
                var _parts = _entry.Split(_delimiter);

                var _id =           Convert.ToInt32(_parts[0]);
                var _tier =         Convert.ToInt32(_parts[1]);
                var _cost =         Convert.ToInt32(_parts[2]);
                var _currentRank =  Convert.ToInt32(_parts[3]);
                var _maxRank =      Convert.ToInt32(_parts[4]);
                var _bonusRanks =   Convert.ToInt32(_parts[5]);
                var _prereqsMet =   Convert.ToBoolean(_parts[6]);
                var _isStart =      Convert.ToBoolean(_parts[7]);
                var _isGoldMedal =  Convert.ToBoolean(_parts[8]);
                var _spellId =      Convert.ToInt32(_parts[9]);

                _traitData.Add(new Trait(_id,
                                         _tier,
                                         _cost,
                                         _currentRank,
                                         _maxRank,
                                         _bonusRanks,
                                         _prereqsMet,
                                         _isStart,
                                         _isGoldMedal,
                                         _spellId) );
            }

            return _traitData;
        }

        private Trait GetBestTrait()
        {
            if (!IsFrameVisible("ArtifactFrame"))
                return null;

            var _traitData = GetTraits();
            var _possibleTraits = _traitData.Where(t => t.PrereqsMet &&
                                                        t.CurrentRank != t.MaxRank);
            var _normalTraits = _possibleTraits.Where(t => !t.IsGoldMedal);
            var _goldMedals = _possibleTraits.Where(t => t.IsGoldMedal);
            var _newGoldMedals = _goldMedals.Where(t => t.CurrentRank == 0);

            if (_newGoldMedals.Count() != 0)
                return _newGoldMedals.First();

            if (_normalTraits.Count() != 0)
                return _normalTraits.First();

            if (_goldMedals.Count() != 0)
                return _goldMedals.First();

            return null;
        }

        private async Task<bool> TrainTraits()
        {
            if (!IsFrameVisible("ArtifactFrame"))
                return false;

            while(KOMApi._availableTraits > 0 && !breakOut)
            {
                var _bestTrait = GetBestTrait();
                if (_bestTrait == null)
                    return false;

                var _spellId = _bestTrait.SpellId;
                var _spell = WoWSpell.FromId(_spellId);
                Log("     {0} #{1} (Id: {2})", _spell.Name,
                                               _bestTrait.CurrentRank + 1,
                                               _spellId);
                Lua.DoString($"C_ArtifactUI.AddPower({_bestTrait.Id})");
                await Sleep();

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> HandleClassHallHelper()
        {
            if (IsHelperProfile())
                return false;

            var _classHall = _classHallData.FirstOrDefault(c => c.Class == Me.Class &&
                                                                c.ZoneId != Me.ZoneId &&
                                                                c.HasTravelScript);
            if (_classHall == null)
                return false;

            var _profile = ProfileManager.XmlLocation;
            KOMApi._lastProfile = _profile;
            KOMApi._lastBotBaseName = BotManager.Current.Name;

            var _classHallHelperProfile = $"{Application.StartupPath}\\..\\Plugins\\KeepOnMoving\\ClassHallHelper.xml";

            DevLog("{0}", Me.Class);
            Log("Loading ClassHallHelper");
            Log("     {0} trait{1} available", KOMApi._availableTraits, (KOMApi._availableTraits > 1)
                                                                                ? "s"
                                                                                : "");

            KOMApi.ChangeBotBase("Questing", _classHallHelperProfile);
            await Coroutine.Wait(30 * 1000, () => IsHelperProfile());

            return true;
        }

        private async Task<bool> HandleArtifactTraits()
        {
            if (!Settings.Artifact_TrainTraits)
                return false;

            if (Me.IsInInstance)
                return false;

            if (_checkedTraits && !IsHelperProfile("ClassHall"))
                return false;

            if (KOMApi._availableTraits == 0)
                return false;

            if (await HandleClassHallHelper())
                return false;

            var _classHall = _classHallData.FirstOrDefault(c => c.Class == Me.Class &&
                                                                c.ZoneId == Me.ZoneId);
            if (_classHall == null)
                return false;

            Log("Training artifact traits");

            var _forgeLocation = _classHall.ForgeLocation;
            var _forgeObjectId = _classHall.ForgeObjectId;
            var _forge = GetObject(_forgeObjectId);
            while (KOMApi._availableTraits > 0 && !breakOut)
            {
                DevLog("WHILE: HandleArtifactTraits");

                _forge = GetObject(_forgeObjectId);
                if (_forge != null)
                    _forgeLocation = _forge.Location;

                if (_forge == null || !_forge.WithinInteractRange)
                    await MoveTo(_forgeLocation);
                else
                {
                    await CommonCoroutines.StopMoving();
                    await Sleep();
                    _forge.Interact();

                    await Coroutine.Wait(5 * 1000, () => IsFrameVisible("ArtifactFrame"));
                    await TrainTraits();
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private class RelicSlot
        {
            public RelicSlot(int _id,
                             bool _locked,
                             string _type,
                             int _powerId,
                             int _itemLevel)
            {
                Id = _id;
                Locked = _locked;
                Type = _type;
                PowerId = _powerId;
                ItemLevel = _itemLevel;
            }

            public int Id { get; }
            public bool Locked { get; }
            public string Type { get; }
            public int PowerId { get; }
            public int ItemLevel { get; }

            public override string ToString()
            {
                return String.Format("Id: {0}, Locked: {1}, Type: {2}, PowerId: {3}, ItemLevel: {4}",
                                        Id,
                                        Locked,
                                        Type,
                                        PowerId,
                                        ItemLevel);
            }
        }

        private List<RelicSlot>  GetRelicSlots()
        {
            if (!IsFrameVisible("ArtifactFrame"))
                return new List<RelicSlot>();

            var _lua = @"
	            local data = {{}}

	            local addToTable = function(...)
		            local row = {}
		            for k, v in pairs({...}) do
			            table.insert(row, tostring(v) )
		            end
		            table.insert(data, table.concat(row, "";"") )
	            end

	            for i = 1, 3 do
		            local locked = (C_ArtifactUI.GetRelicLockedReason(i) ~= nil)
		            local type = C_ArtifactUI.GetRelicSlotType(i)
		            local link = select(4, C_ArtifactUI.GetRelicInfo(i))
		            if link ~= nil then
			            local itemLevel = select(4, GetItemInfo(link))
			            local powerId = C_ArtifactUI.GetPowersAffectedByRelic(i)
			            addToTable(i, locked, type, powerId, itemLevel)
		            else
			            addToTable(i, locked, type, 0, 0)
		            end
	            end

	            return unpack(data)
            ";

            var _data = Lua.GetReturnValues(_lua);
            _data.RemoveAt(0);
            var _relicSlotData = new List<RelicSlot>();

            foreach (var _entry in _data)
            {
                var _delimiter = ';';
                var _parts = _entry.Split(_delimiter);

                var _id =           Convert.ToInt32(_parts[0]);
                var _locked =       Convert.ToBoolean(_parts[1]);
                var _type =         _parts[2];

                var _powerId = (_parts.Count() >= 4)
                                    ? Convert.ToInt32(_parts[3])
                                    : 0;
                var _itemLevel = (_parts.Count() >= 5)
                                    ? Convert.ToInt32(_parts[4])
                                    : 0;

                _relicSlotData.Add(new RelicSlot(_id,
                                                 _locked,
                                                 _type,
                                                 _powerId,
                                                 _itemLevel));
            }

            return _relicSlotData;
        }

        private async Task<bool> OpenArtifactFrame()
        {
            while(!IsFrameVisible("ArtifactFrame") && !breakOut)
            {
                Lua.DoString("SocketInventoryItem(16)");
                await Coroutine.Wait(2 * 1000, () => IsFrameVisible("ArtifactFrame"));
                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> CloseArtifactFrame()
        {
            while (IsFrameVisible("ArtifactFrame") && !breakOut)
            {
                Lua.DoString("ArtifactFrame:Hide()");
                await Coroutine.Wait(2 * 1000, () => !IsFrameVisible("ArtifactFrame"));
                await Coroutine.Yield();
            }

            return true;
        }

        private int GetRelicSlotIndex(WoWItem _item)
        {
            if (_item == null)
                return 0;

            if (!_item.IsValid)
                return 0;

            if (_item.ItemInfo == null)
                return 0;

            if (!IsArtifactRelic(_item))
                return 0;

            if (!IsFrameVisible("ArtifactFrame"))
                return 0;

            var _lua = @"
	            local _ , _, type, link = C_ArtifactUI.GetRelicInfoByItemID(" + _item.Entry + @")
                local itemLevel = select(4, GetItemInfo(link))
	            local powerId = C_ArtifactUI.GetPowersAffectedByRelicItemLink(link)
	            return type, powerId, itemLevel
            ";
            var _relicInfo = Lua.GetReturnValues(_lua);
            var _slotType   = _relicInfo[0];
            var _powerId    = Convert.ToInt32(_relicInfo[1]);
            var _itemLevel  = Convert.ToInt32(_relicInfo[2]);

            var _setting = Settings.Artifact_ApplyRelics.ToString();
            var _relicSlotData = GetRelicSlots();
            var _slotToApply = _relicSlotData
                                        .Where(s => !s.Locked &&
                                                    _itemLevel > s.ItemLevel &&
                                                    _slotType == s.Type &&
                                                    (
                                                        _setting != "KeepAffectedTrait" ||
                                                        _powerId == s.PowerId
                                                    ))
                                        .OrderBy(s => s.ItemLevel)
                                        .FirstOrDefault();
            if (_slotToApply == null)
                return 0;

            var _slotId = _slotToApply.Id;
            return _slotId;
        }

        private int GetRelicSlotItemLevel(int _index)
        {
            if (!IsFrameVisible("ArtifactFrame"))
                return 0;

            var _lua = @"
                local link = select(4, C_ArtifactUI.GetRelicInfo(" + _index + @"))
                if link ~= nil then
                    return select(4, GetItemInfo(link))
                else
                    return 0
                end
            ";
            var _itemLevel = Lua.GetReturnVal<int>(_lua, 0);
            return _itemLevel;
        }

        private int _artifactRelicCount
        {
            get
            {
                return Me.BagItems
                            .Where(i => IsArtifactRelic(i))
                            .Count();
            }

        }

        private async Task<bool> HandleRelicSlots()
        {
            if (Settings.Artifact_ApplyRelics.ToString() == "Disabled")
                return false;

            if (_checkedRelics)
                return false;

            await OpenArtifactFrame();
            var _item = Me.BagItems
                            .Where(i => GetRelicSlotIndex(i) > 0)
                            .OrderByDescending(i => i.ItemInfo.Level)
                            .FirstOrDefault();
            if (_item == null)
            {
                await CloseArtifactFrame();
                _checkedRelics = true;
                return false;
            }

            Log("Applying artifact relics");

            var _lastSlotIndex = 0;
            while (_item != null && !breakOut)
            {
                DevLog("WHILE: HandleRelicSlots");

                var _slotIndex = GetRelicSlotIndex(_item);
                var _slotItemLevel = GetRelicSlotItemLevel(_slotIndex);
                var _preItemLevel = _item.ItemInfo.Level;

                if (_slotIndex != _lastSlotIndex)
                {
                    Log("     {0} to slot #{1} (Id: {2})", _item.Name, _slotIndex, _item.Entry);
                    DevLog("{0} > {1}", _slotItemLevel, _item.ItemInfo.Level);
                    _lastSlotIndex = _slotIndex;
                }

                _item.PickUp();
                await Sleep();
                Lua.DoString($"C_ArtifactUI.ApplyCursorRelicToSlot({_slotIndex})");
                await Coroutine.Wait(2 * 1000, () => GetRelicSlotItemLevel(_slotIndex) == _preItemLevel);
                await Coroutine.Sleep(750);

                _item = Me.BagItems
                            .Where(i => GetRelicSlotIndex(i) > 0)
                            .OrderByDescending(i => i.ItemInfo.Level)
                            .FirstOrDefault();
                await Coroutine.Yield();
            }
            await CloseArtifactFrame();
            _checkedRelics = true;

            await HandleSorting();
            return true;
        }

        private bool ArtifactEquipped()
        {
            var _mainHand = Me.Inventory.GetItemBySlot((uint)Styx.WoWEquipSlot.MainHand);
            if (_mainHand == null)
                return false;

            if (_mainHand.Quality != WoWItemQuality.Artifact)
                return false;

            return true;
        }

        private async Task<bool> HandleArtifactWeapon()
        {
            RuntimeLog(">");

            if (Me.CurrentMap.ExpansionId != 6)
                return false;

            RuntimeLog(">");

            if (!ArtifactEquipped())
                return false;

            RuntimeLog(">");

            if (IsHelperProfile("Rank"))
                return false;

            RuntimeLog(">");

            if (await HandleArtifactTraits())
                return true;

            RuntimeLog(">");

            if (_checkedItems)
                return false;

            RuntimeLog(">");

            if (await HandleRelicSlots())
                return true;

            RuntimeLog(">");

            if (await HandleArtifactRelics())
                return true;

            RuntimeLog(">");

            if (await HandleArtifactPowerItems())
                return true;

            RuntimeLog(">");

            /*
            if (await HandleArtifactKnowledgeItems())
                return true;
            */

            return false;
        }

        private List<uint> _blacklistedTreasures = new List<uint>()
        {
            269138,     // ANCIENT DRAUGHT OF REGENERATION
            268723,     // ZSERALINE POULTICE
            246915,     // MAMMOTH SKIN BAG
            250448,     // IMP IN A JAR
            249513,     // WQ
            249702      // STOLEN LEY CRYSTAL
        };

        private WoWGameObject _treasure
        {
            get
            {
                var _objects = ObjectManager.GetObjectsOfType<WoWGameObject>()
                                    .Where(o => !_blacklistedTreasures.Contains(o.Entry) &&
                                                (
                                                    (
                                                        o.SubType == WoWGameObjectType.Chest &&
                                                        (
                                                            o.LockType == WoWLockType.Treasure ||
                                                            o.LockType == WoWLockType.Open
                                                        )
                                                    ) ||
                                                    (
                                                        o.SubType == WoWGameObjectType.GatheringNode &&
                                                        o.LockType == WoWLockType.OpenKneeling
                                                    )
                                                ) &&
                                                o.CanLoot &&
                                                (
                                                    !_ignoredGeneralGuids.ContainsKey(o.Guid) ||
                                                    DeltaTime(_ignoredGeneralGuids[o.Guid], "m") >= 60
                                                ) &&
                                                o.IsValid)
                                    .OrderBy(o => o.Distance);

                _objects = (Me.IsFlying || (Flightor.CanFly && Me.IsOutdoors))
                                ? _objects.OrderBy(o => o.Distance)
                                : _objects.Where(o => GetPathDistance(o) > 0 &&
                                                      GetPathDistance(o) < 150)
                                          .OrderBy(o => GetPathDistance(o));

                return _objects.FirstOrDefault();
            }
        }

        private async Task<bool> HandleTreasures()
        {
            if (Settings.General_Treasures.ToString() == "Disabled")
                return false;

            if (IsHelperProfile())
                return false;

            if (ProfileIsArtifactQuest())
                return false;

            if (Me.IsActuallyInCombat && !Me.Mounted)
                return false;

            if (DeltaTime(_timeUnghost, "s") < 10 && Me.HealthPercent <= 90)
                return false;

            if (_treasure == null)
                return false;

            if (await HandleLoot())
                return false;

            if (TreeRoot.GoalText != null && StringContains(TreeRoot.GoalText, "OR", "Escort", "UseTransport"))
                return false;

            if (Me.HasAura("Resurrection Sickness"))
                return false;

            if (Me.CurrentMap.ExpansionId == 5 &&
                !IsQuestDone((Me.IsHorde)
                                    ? 31766
                                    : 31733))
                return false;

            if (breakOut)
                return false;

            Log("Getting Treasure");
            var _treasureName = _treasure.Name;
            var _treasureEntry = _treasure.Entry;
            var _treasureGuid = _treasure.Guid;
            var _treasureLocation = _treasure.Location;
            var _startTime = DateTime.Now;
            var _maxTime = GetMaxTime(GetPathDistance(_treasure));

            while ((_treasure != null || Me.Location.Distance(_treasureLocation) > 30) && !breakOut)
            {
                if (_infoTimer.IsFinished)
                {
                    DevLog("WHILE: HandleTreasures");
                    DevLog("     Name: {0} (Id: {1})", _treasureName, _treasureEntry);
                    DevLog("     Location: {0}", _treasureLocation);
                    DevLog("     Guid: {0}", SafeGuid(_treasureGuid));
                    DevLog("     Distance: {0}", Math.Round(Me.Location.Distance(_treasureLocation)));
                    DevLog("     MaxTime: {0}", _maxTime);
                    DevLog("     TimeLeft: {0}", Math.Ceiling(_maxTime - DeltaTime(_startTime, "s")));
                    _infoTimer.Reset();
                }

                /*
                if (false && DeltaTime(_timeInterrupted, "s") < 5)
                {
                    DevLog("Interrupted");
                    return false;
                }
                */

                if (DeltaTime(_startTime, "s") > _maxTime &&
                    (_treasure == null || !_treasure.WithinInteractRange))
                {
                    DevLog("     MaxTime reached");
                    DevLog("     Blacklisting for 1 hour");
                    DevLog("     Name: {0} (Id: {1})", _treasureName, _treasureEntry);

                    if (!_ignoredGeneralGuids.ContainsKey(_treasureGuid))
                        _ignoredGeneralGuids.Add(_treasureGuid, DateTime.Now);
                    else
                        _ignoredGeneralGuids[_treasureGuid] = DateTime.Now;
                    return false;
                }

                if (_treasure != null)
                    _treasureLocation = _treasure.Location;

                if (_treasure == null || !_treasure.WithinInteractRange)
                {
                    if (!await MoveTo(_treasureLocation))
                    {
                        DevLog("     Moving failed");

                        if (!_ignoredGeneralGuids.ContainsKey(_treasureGuid))
                            _ignoredGeneralGuids.Add(_treasureGuid, DateTime.Now);
                        else
                            _ignoredGeneralGuids[_treasureGuid] = DateTime.Now;
                        return false;
                    }
                }
                else
                {
                    await CommonCoroutines.StopMoving();
                    await CommonCoroutines.LandAndDismount(null, true);

                    _treasure.Interact();
                    await Loot();
                    await Coroutine.Wait(2 * 1000, () => _treasure == null);
                }

                await Coroutine.Yield();
            }

            return true;
        }

        private async Task<bool> KOMloop()
        {
            if (!StyxWoW.IsInWorld)
                return false;

            if (_spawnNpc != null)
                DevLog("Spawned! {0} (Id: {1})", _spawnNpc.Name, _spawnNpc.Entry);

            /*
            if (DeltaTime(_timeRun, "s") > 5)
            {
                _timeLastMoved = DateTime.Now;
                _timeLastObjectiveChange = DateTime.Now;
            }
            */

            RuntimeLog("Start");

            if (!await HandleInit())
                return false;

            RuntimeLog("#");

            if (await HandleDungeonQuest())
            {
                RuntimeLog("Sum");
                return false;
            }

            RuntimeLog("#");

            if (await HandleStuck())
                return true;

            RuntimeLog("#");

            if (await HandleMeleeDismount())
                return true;

            RuntimeLog("#");

            if (await HandleDig())
                return true;

            RuntimeLog("#");

            await HandleJump();

            RuntimeLog("#");

            await HandleLoot();

            RuntimeLog("#");

            if (!await HandleTps())
                return EndRun(false, false);

            RuntimeLog();

            if (await HandleMotionless())
                return true;

            RuntimeLog();

            if (await HandleHearth())
                return true;

            RuntimeLog();

            await HandleLegionFlying();

            RuntimeLog();

            await HandleGraphicMode();

            RuntimeLog();

            await HandleDisableAddons();

            RuntimeLog();

            await HandleKillBetweenHotspots();

            RuntimeLog();

            await HandleAutoLoot();

            RuntimeLog();

            if (await HandleReloadProfile())
                return true;

            RuntimeLog();

            if (await HandleResurrect())
                return true;

            RuntimeLog();

            if (!Me.IsAlive)
                return EndRun(false);

            RuntimeLog();

            await HandleDialog();

            RuntimeLog();

            if (await HandleTreasures())
                return true;

            RuntimeLog();

            if (await HandleGatheringProfessions())
                return true;

            RuntimeLog();

            if (await HandleNpcs())
                return true;

            RuntimeLog();

            await HandleFrameBug();

            RuntimeLog();

            await HandleFeralCombat();

            RuntimeLog();

            if (Me.Combat)
            {
                return EndRun(true);
            }

            RuntimeLog();

            if (await HandleTransportBug())
                return true;

            RuntimeLog();

            if (Me.OnTaxi || Me.IsOnTransport || InVehicle())
                return EndRun(false);

            RuntimeLog();

            await HandleTravelForm();

            RuntimeLog();

            if (await HandleMountUp())
                return true;

            RuntimeLog();

            if (await HandleGetOffGround())
            {
                _timeStartFlyingMountRun = DateTime.Now;
                return true;
            }

            RuntimeLog();

            if (await HandleNavigationBlackspots())
                return true;

            RuntimeLog();

            if (await HandleStop())
                return true;

            RuntimeLog();

            if (await HandleSpec())
                return true;

            RuntimeLog();

            if (await HandleArchaeology())
                return true;

            RuntimeLog();

            if (await HandleArtifactWeapon())
                return true;

            RuntimeLog();

            if (await HandleAbandonQuest())
                return true;

            RuntimeLog();

            if (await HandleTaxiNodes())
                return true;

            RuntimeLog();

            if (await HandleNewMails())
                return true;

            RuntimeLog();

            if (await HandleItems())
                return true;

            RuntimeLog();

            if (await HandleGarrisonResources())
                return true;

            RuntimeLog();

            if (await HandleTrainFlying())
                return true;

            return EndRun();
        }

        private bool EndRun(bool _flagAsChecked = true, bool _setTime = true)
        {
            RuntimeLog("Sum");

            if (_flagAsChecked)
            {
                if (!IsHelperProfile() && !Me.IsInInstance && !Me.Combat)
                    _checkedQuests = true;
                if(DeltaTime(_timeCheckedItems, "ms") >= 2000)
                    _checkedItems = true;

                if (!Me.Combat)
                   _checkedTraits = true;
            }

            if(_setTime)
                _timeRun = DateTime.Now;

            return false;
        }

        private ulong _flyingGold
        {
            get
            {
                var _factionId = (Me.IsHorde)
                                    ? 76
                                    : 72;

                var _amount = 5000 - ((int)Me.GetReputationLevelWith((uint)_factionId) - 3) * 250;
                return (ulong)_amount;
            }
        }

        private ulong GetFlyingGold(uint _factionId)
        {
            var _base = 5000;
            var _levelAboveNeutral = Math.Max(0, (int)Me.GetReputationLevelWith(_factionId) - 3);
            var _discount = (Me.Race == WoWRace.Goblin)
                                    ? 20
                                    : _levelAboveNeutral * 5;
            var _amount = 5000 * (1 - _discount / 100);

            return (ulong)_amount;
        }

        private WoWUnit _flightmaster
        {
            get
            {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         ((UnitNPCFlags)u.NpcFlags).HasFlag(UnitNPCFlags.Flightmaster) &&
                         u.IsValid &&
                         u.CanInteractNow &&
                         u.IsAlive
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private WoWUnit _spirithealer
        {
            get {
                IEnumerable<WoWUnit> _units =
                    (from u in ObjectManager.GetObjectsOfTypeFast<WoWUnit>()
                     where
                         ((UnitNPCFlags)u.NpcFlags).HasFlag(UnitNPCFlags.Spirithealer) &&
                         u.IsAlive &&
                         u.IsValid
                     select u);

                return (_units.Count() > 0)
                             ? _units.First()
                             : null;
            }
        }

        private void ReloadProfile()
        {
            string _profile = ProfileManager.XmlLocation;
            DevLog("Reloading");
            DevLog("     Current Profile: {0}", (_profile == null)
                                                    ? "null"
                                                    : _profile);

            if (!Me.IsNeutralPandaren)
                _profile = GetAutoLoader();

            DevLog("     New Profile: {0}", _profile);
            ProfileManager.LoadNew(_profile);
        }

        public override void OnDisable()
        {
            BotEvents.OnBotStarted -= BotEvents_OnOnBotStarted;
            BotEvents.OnBotStopped -= BotEvents_OnOnBotStopped;
            BotEvents.OnBotPaused -= BotEvents_OnOnBotPaused;
            BotEvents.OnBotResumed -= BotEvents_OnOnBotResumed;
            Logging.OnLogMessage -= Logging_OnLogMessage;
            BotEvents.Profile.OnNewOuterProfileLoaded -= Profile_OnNewOuterProfileLoaded;
            Targeting.Instance.WeighTargetsFilter -= Targeting_WeighTargetsFilter;

            TreeHooks.Instance.RemoveHook("InGame_Check", _KOM);

            Lua.Events.DetachEvent("PLAYER_LEVEL_UP", LuaEvents_PlayerLevelUp);
            Lua.Events.DetachEvent("PLAYER_DEAD", LuaEvents_PlayerDead);
            Lua.Events.DetachEvent("PLAYER_UNGHOST", LuaEvents_PlayerUnghost);

            Lua.Events.DetachEvent("QUEST_WATCH_UPDATE", LuaEvents_QuestWatchUpdate);
            Lua.Events.DetachEvent("QUEST_TURNED_IN", LuaEvents_QuestTurnedIn);
            Lua.Events.DetachEvent("QUEST_ACCEPTED", LuaEvents_QuestAccepted);
            Lua.Events.DetachEvent("QUEST_REMOVED", LuaEvents_QuestRemoved);

            Lua.Events.DetachEvent("BAG_UPDATE_DELAYED", LuaEvents_BagUpdateDelayed);
            Lua.Events.DetachEvent("ENCOUNTER_LOOT_RECEIVED", LuaEvents_EncounterLootReceived);

            Lua.Events.DetachEvent("DUEL_REQUESTED", LuaEvents_DuelRequested);
            Lua.Events.DetachEvent("PARTY_INVITE_REQUEST", LuaEvents_PartyInviteRequest);

            Lua.Events.DetachEvent("UI_ERROR_MESSAGE", LuaEvents_UiErrorMessage);

            Lua.Events.DetachEvent("UNIT_SPELLCAST_INTERRUPTED", LuaEvents_UnitSpellcastInterrupted);
            Lua.Events.DetachEvent("UNIT_SPELLCAST_SUCCEEDED", LuaEvents_UnitSpellcastSucceeded);

            Lua.Events.DetachEvent("TAXIMAP_OPENED", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("GOSSIP_SHOW", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("MAIL_SHOW", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("TRAINER_SHOW", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("LOOT_OPENED", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("CINEMATIC_START", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("QUEST_DETAIL", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("QUEST_PROGRESS", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("QUEST_COMPLETE", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("QUEST_GREETING", LuaEvents_DialogOpened);
            Lua.Events.DetachEvent("MERCHANT_SHOW", LuaEvents_DialogOpened);

            Lua.Events.DetachEvent("LOOT_CLOSED", LuaEvents_DialogClosed);

            Lua.Events.DetachEvent("UNIT_ENTERING_VEHICLE", LuaEvents_UnitEnteringVehicle);
            Lua.Events.DetachEvent("UNIT_EXITED_VEHICLE", LuaEvents_UnitExitedVehicle);

            Lua.Events.DetachEvent("LFG_PROPOSAL_SHOW", LuaEvents_LfgProposalShow);
            Lua.Events.DetachEvent("LFG_PROPOSAL_SUCCEEDED", LuaEvents_LfgProposalSucceeded);
            Lua.Events.DetachEvent("LFG_PROPOSAL_FAILED", LuaEvents_LfgProposalFailed);

            Lua.Events.DetachEvent("UPDATE_PENDING_MAIL", LuaEvents_UpdatePendingMail);

            Lua.Events.DetachEvent("ARTIFACT_XP_UPDATE", LuaEvents_ArtifactXpUpdateArtifactUpdate);
            Lua.Events.DetachEvent("ARTIFACT_UPDATE", LuaEvents_ArtifactXpUpdateArtifactUpdate);

            Reset(true);
            Log("Disabled");
        }

        private bool IsPlayerInRange
        {
            get
            {
                return ObjectManager.GetObjectsOfTypeFast<WoWPlayer>().Any(p => p.Distance <= 100);
            }
        }

        private void DisableAddons()
        {
            DevLog("DisableAddons");
            DevLog("     PreventDisableAddons: {0}", Settings.Quest_PreventDisableAddons);
            DevLog("     QuestingPack: {0}", Settings.Quest_QuestingPack);

            if (Settings.Quest_QuestingPack.ToString() != "New")
                return;

            var _enabled = "Lua.DoString(\"for i=1,GetNumAddOns()";
            var _disabled = "Lua.DoString(\"--for i=1,GetNumAddOns()";
            var _questProfile = "QuestOrder";

            var _questingFolder = $"{Application.StartupPath}\\..\\Default Profiles\\Questing Profile Pack\\";
            var _files = Directory.GetFiles(_questingFolder, "*.xml", SearchOption.AllDirectories).ToList();
            foreach (var _file in _files)
            {
                var _content = File.ReadAllText(_file);

                var _searchFor = (Settings.Quest_PreventDisableAddons)
                                        ? _enabled
                                        : _disabled;
                var _replaceWith = (Settings.Quest_PreventDisableAddons)
                                           ? _disabled
                                           : _enabled;

                if (StringContains(_content, "AND", _questProfile, _searchFor))
                {
                    var _result = _content.Replace(_searchFor, _replaceWith);
                    File.WriteAllText(_file, _result);
                }
            }

            _disableAddonsSetting = Convert.ToInt32(Settings.Quest_PreventDisableAddons);
        }

        private async Task<bool> HandleDisableAddons()
        {
            if (Settings.Quest_QuestingPack.ToString() != "New")
                return false;

            if (_disableAddonsSetting == Convert.ToInt32(Settings.Quest_PreventDisableAddons) )
                return false;

            DisableAddons();
            return true;
        }

        private void RemoveNameFromLog()
        {
            DevLog("RemoveNameFromLog");

            var _file = Logging.LogFilePath;
            var _result = Regex.Replace(File.ReadAllText(_file), Me.Name, "", RegexOptions.IgnoreCase);

            File.WriteAllText(_file, _result);
        }

        private void Profile_OnNewOuterProfileLoaded(BotEvents.Profile.NewProfileLoadedEventArgs _args)
        {
            _timeProfileLoaded = DateTime.Now;
        }

        private void Logging_OnLogMessage(ReadOnlyCollection<Logging.LogMessage> _messages)
        {
            foreach (var _item in _messages)
            {
                _lastLogEntry = _item.Message;

                if (_lastLogEntry.Contains("[AutoEquip]"))
                    _timeAutoEquip = DateTime.Now;

                if (_lastLogEntry.Contains("Reason: Exception thrown"))
                    _timeExceptionThrown = DateTime.Now;

                if (_lastLogEntry.ToLower().Contains(Me.Name.ToLower()))
                    RemoveNameFromLog();

                var _message = Settings.Stop_Message;
                if (_message != "0" && _lastLogEntry.ToLower().Contains(_message.ToLower()))
                    _stopMessage = true;

                if (_lastLogEntry.Contains("[Flightor] Set up for movement from"))
                {
                    _intervalLastFlightorLog = DeltaTime(_timeLastFlightorLog, "ms");
                    _timeLastFlightorLog = DateTime.Now;
                }

                if (_lastLogEntry.Contains("Changed maps to "))
                    _timeMapChanged = DateTime.Now;

                if (_lastLogEntry.Contains("Dismount to kill bot poi."))
                    _meleeDismount = true;

                if (StringContains(_lastLogEntry, "OR",
                        "Navigator unable to move",
                        "[Flightor] We are indoors, but we could not find a take off spot. Retrying soon..",
                        "[Flightor] Failed ground nav from ",
                        "MoveTo failed to move to the location"))
                    StuckState = Stuck.On;

                if (_lastLogEntry.Contains("Stop and dismount. Reason: Using transport"))
                    _timeTransportDismount = DateTime.Now;


                if (_lastLogEntry.Contains("Path distances request failed on map:"))
                {
                    if (DeltaTime(_timePathDistancesRequestCount, "ms") < 3000)
                    {
                        _failedPathDistancesRequestCount++;
                        DevLog("FailedPathDistancesRequestCount++ ({0})", _failedPathDistancesRequestCount);
                    }
                    else if (_failedPathDistancesRequestCount > 0)
                    {
                        DevLog("FailedPathDistancesRequestCount Reset");
                        _failedPathDistancesRequestCount = 0;
                    }

                    _timePathDistancesRequestCount = DateTime.Now;
                }

                if (StringContains(_lastLogEntry, "OR", "Mount up failed unexpectedly", "Mount up:"))
                {
                    if (DeltaTime(_timeMountUp, "ms") < 6000)
                    {
                        _failedMountUpCount++;
                        DevLog("FailedMountUpCount++ ({0})", _failedMountUpCount);
                    }
                    else if (_failedMountUpCount > 0)
                    {
                        DevLog("FailedMountUpCount Reset");
                        _failedMountUpCount = 0;
                    }

                    _timeMountUp = DateTime.Now;
                }

                /*
                if (_lastLogEntry.Contains("We are stuck"))
                    StuckState = Stuck.On;
                    */

                if (_lastLogEntry.Contains("Done selling/repairing") || _lastLogEntry.Contains("Done mailing"))
                {
                    if (Settings.Bag_Sort)
                    {
                        Log("Sorting bags");
                        Lua.DoString("SortBags()");
                    }
                }

                if (
                        (_lastLogEntry.Contains("Moving into") && _lastLogEntry.Contains("indoor area")) ||
                        _lastLogEntry.Contains("Destination appears to be indoors")
                    )
                    _timeMovingInto = DateTime.Now;
            }
        }

        public void PrintSettings(bool _enable = false, bool _imported = false)
        {
            var _logString = "\n\n\n";

            if (!_enable)
                _logString += String.Format("Settings changed!{0}\n\n", (_imported)
                                                                            ? " (imported)"
                                                                            : "");

            string _onlineVersion = null;
            try
            {
                var _regex = new Regex("(?<=<span style=\"font-size: 18px\">v)[.0-9]+(?=<)");
                var _html = new KOMWebClient().DownloadString("https://www.thebuddyforum.com/threads/plugin-keeponmoving-unstuck-handling-and-more-in-development.249513/");
                var _result = _regex.Matches(_html);
                _onlineVersion = _result[0].ToString();
            }
            catch (Exception)
            {
                //
            }

            var _updateInformation = (_onlineVersion == null)
                                            ? ") [WebClient Timeout]"
                                            : (Version.ToString() != _onlineVersion)
                                                        ? $" != v{_onlineVersion})"
                                                        : ")";
            _logString += $"----- KOM-LOG (v{Version}{_updateInformation} -----\n\n";

            try
            {
                Settings.GetSettings();
            }
            catch (Exception _ex)
            {
                Log("Restoring default values as settings seem to be corrupted");
                File.Delete(KOMSettings._settingsFile);
            }

            var _changed = false;
            var _settings = Settings.GetSettings().OrderBy(s => s.Key);
            string _lastCategory = null;
            foreach (var _setting in _settings)
            {
                var _category = _setting.Key.Substring(0, _setting.Key.IndexOf("_"));
                if (_category != _lastCategory)
                {
                    _lastCategory = _category;
                    _logString += $"[{_category.ToUpper()}]\n";
                }

                if (!Settings._settingsDict.ContainsKey(_setting.Key))
                    Settings._settingsDict.Add(_setting.Key, _setting.Value);

                var _settingCaption = _setting.Key.Substring(_setting.Key.IndexOf("_") + 1);
                if (Settings._settingsDict[_setting.Key].ToString() == _setting.Value.ToString())
                    _logString += $"     {_settingCaption}: {_setting.Value}\n";
                else
                {
                    _changed = true;
                    _logString += $"  !  {_settingCaption}: {Settings._settingsDict[_setting.Key]} > {_setting.Value}\n";
                }

                Settings._settingsDict[_setting.Key] = _setting.Value;
            }

            _logString += "\n----- KOM-LOG -----\n\n";

            if (_enable || _changed || _imported)
                DevLog(_logString);

            if (_enable)
            {
                if (_onlineVersion != null && Version.ToString() != _onlineVersion)
                {
                    Log("----- New version available: v{0} -----", _onlineVersion);

                    if (Settings.General_UpdateSound)
                    {
                        _soundPlayer.SoundLocation = $"{Application.StartupPath}\\..\\Plugins\\KeepOnMoving\\update.wav";
                        _soundPlayer.Play();
                    }
                }
            }

            if(_enable)
                PrintRankInfo();
        }

        /*
        private int GetTurnInNpcId(int _questId, string _mode = "PFW")
        {
            if (_mode.Contains("P"))        // last (P)rofile
            {
                try
                {
                    var _doc = new XmlDocument();
                    _doc.Load(_lastProfile);
                    var _node = _doc.SelectSingleNode($"//TurnIn[@QuestId='{_questId}']");
                    return Convert.ToInt32(_node.Attributes["TurnInId"].Value);
                }
                catch (Exception)
                {
                    //
                }
            }

            if (_mode.Contains("F"))        // questing (F)olders
            {
                var _questingFolder = $"{Application.StartupPath}\\Default Profiles\\Questing Profile Pack\\";
                var _files = Directory.GetFiles(_questingFolder, "*.xml", SearchOption.AllDirectories).ToList();
                foreach (var _file in _files)
                {
                    try
                    {
                        var _doc = new XmlDocument();
                        _doc.Load(_file);
                        var _node = _doc.SelectSingleNode($"//TurnIn[@QuestId='{_questId}']");
                        return Convert.ToInt32(_node.Attributes["TurnInId"].Value);
                    }
                    catch (Exception)
                    {
                        //
                    }
                }
            }

            if (_mode.Contains("W"))        // (W)owhead
            {
                try
                {
                    var _url = $"http://www.wowhead.com/quest={_questId}";
                    var _html = new KOMWebClient().DownloadString(_url);

                    var _regex = new Regex("(?<=myMapper(.+)\"end\"(.+)\"id\":)\\d+");
                    var _result = _regex.Matches(_html);
                    return Convert.ToInt32(_result[0].ToString());
                }
                catch (Exception)
                {
                    //
                }
            }

            return -1;
        }
        */

        private object QueryOnline(string _mode, int _id, string _filter, string _returnType = "int")
        {
            string _html = null;
            var _wowhead = false;
            try
            {
                var _url = $"http://www.wowdb.com/{_mode}s/{_id}";
                _html = new KOMWebClient().DownloadString(_url);
            }
            catch(Exception)
            {
                try
                {
                    var _url = $"http://www.wowhead.com/{_mode}={_id}";
                    _html = new KOMWebClient().DownloadString(_url);
                    _wowhead = true;
                }
                catch(Exception)
                {
                    return null;
                }
            }

            var _regex = new Regex("");
            if (_wowhead)
            {
                switch (_filter)
                {
                    case "TurnInNpcId":
                        _regex = new Regex("(?<=myMapper(.+)\"end\"(.+)\"id\":)\\d+");
                        break;
                }
            }
            else
            {
                switch (_filter)
                {
                    case "TurnInNpcId":
                        _regex = new Regex("(?<=(.+)quest-end(.+))\\d+");
                        break;
                    case "FactionId":
                        _regex = new Regex("(?<=Faction:(.+))\\d+");
                        break;
                }
            }

            var _result = _regex.Matches(_html);
            try
            {
                if (_returnType == "int")
                    return Convert.ToInt32(_result[0].ToString());
            }
            catch(Exception)
            {
                //
            }

            return null;
        }

        private void PrintRankInfo()
        {
            try
            {
                if (Me.Level < 98)
                    return;

                var _professions = Settings.GatheringProfessions_Professions.ToString();
                if (_professions == "None")
                    return;

                if (!Settings.GatheringProfessions_Ranks)
                    return;

                string _logString = "\n\n\n----- KOM-LOG (RankInfo) -----\n\n";

                var _miningValue = GetSkillValue(SkillLine.Mining, true);
                var _herbalismValue = GetSkillValue(SkillLine.Herbalism, true);
                var _miningString = (_miningValue > 0)
                                        ? $"Mining: {_miningValue}\n"
                                        : "";
                var _herbalismBonus = (Me.Race == WoWRace.Tauren)
                                            ? " (+15)"
                                            : "";
                var _herbalismString = (_herbalismValue > 0)
                                                ? $"Herbalism: {_herbalismValue}{_herbalismBonus}\n"
                                                : "";
                _logString += _miningString + _herbalismString;
                _logString += (_miningString != "" || _herbalismString != "")
                                    ? "\n"
                                    : "";

                string _lastSkill = null;
                string _lastItem = null;
                string _lastRank = null;
                foreach (var _entry in _legionRankQuestData)
                {
                    var _skill = (string)_entry[1];
                    if (!_professions.Contains(_skill))
                        continue;

                    var _info = (string)_entry[0];
                    var _item = _info.Substring(0, _info.Length - 2);
                    var _rank = _info.Substring(_info.Length - 1, 1);
                    var _questId = (int)_entry[4];

                    var _isDone = IsQuestDone(_questId);
                    var _isCompleted = (_isDone || IsQuestCompleted(_questId));
                    var _hasQuest = (_isCompleted || HasQuest(_questId));
                    var _canAccept = (_hasQuest || CanAcceptQuest(_entry));

                    if (_skill != _lastSkill)
                    {
                        _lastSkill = _skill;
                        _logString += $"[{_skill.ToUpper()}]\n";
                    }
                    if (_item != _lastItem)
                    {
                        _lastItem = _item;
                        _logString += $"     {_item}\n";
                    }
                    if (_rank != _lastRank)
                    {
                        _lastRank = _rank;
                        _logString += $"          < {_rank} >\n";
                    }

                    _logString += String.Format("               {0}: {1}{2}{3}{4}\n",
                                         _questId,
                                         _canAccept,
                                         _hasQuest,
                                         _isCompleted,
                                         _isDone);
                }
                _logString = _logString
                                  .Replace("True", "[#]")
                                  .Replace("False", " . ");

                _logString += "\n----- KOM-LOG (RankInfo) -----\n\n";

                DevLog(_logString);
            }
            catch (Exception _ex)
            {
                DevLog("Exception: {0}", _ex.ToString());
            }

        }

        private List<uint> _avoidNpcIds = new List<uint>()
        {
            95937,
            99791,
            99792,
            102886,
            104220,
            107850
        };

        private void AddAvoids()
        {
            AvoidanceManager.AddAvoidObject<WoWUnit>(
                    () => true,
                    15,
                    o => _avoidNpcIds.Contains(o.Entry));
            DevLog("Avoids added");
        }

        public override void OnEnable()
        {
            Log("Enabled");

            try
            {
                PrintSettings(true);
                AddAvoids();

                if (_trainerData.Count() == 0)
                    CreateTrainerData();

                BotEvents.OnBotStarted += BotEvents_OnOnBotStarted;
                BotEvents.OnBotStopped += BotEvents_OnOnBotStopped;
                BotEvents.OnBotPaused += BotEvents_OnOnBotPaused;
                BotEvents.OnBotResumed += BotEvents_OnOnBotResumed;
                Logging.OnLogMessage += Logging_OnLogMessage;
                BotEvents.Profile.OnNewOuterProfileLoaded += Profile_OnNewOuterProfileLoaded;
                Targeting.Instance.WeighTargetsFilter += Targeting_WeighTargetsFilter;

                TreeHooks.Instance.InsertHook("InGame_Check", 0, _KOM ?? (_KOM = new ActionRunCoroutine(ctx => KOMloop())));

                Lua.Events.AttachEvent("PLAYER_LEVEL_UP", LuaEvents_PlayerLevelUp);
                Lua.Events.AttachEvent("PLAYER_DEAD", LuaEvents_PlayerDead);
                Lua.Events.AttachEvent("PLAYER_UNGHOST", LuaEvents_PlayerUnghost);

                Lua.Events.AttachEvent("QUEST_WATCH_UPDATE", LuaEvents_QuestWatchUpdate);
                Lua.Events.AttachEvent("QUEST_TURNED_IN", LuaEvents_QuestTurnedIn);
                Lua.Events.AttachEvent("QUEST_ACCEPTED", LuaEvents_QuestAccepted);
                Lua.Events.AttachEvent("QUEST_REMOVED", LuaEvents_QuestRemoved);

                Lua.Events.AttachEvent("BAG_UPDATE_DELAYED", LuaEvents_BagUpdateDelayed);
                Lua.Events.AttachEvent("ENCOUNTER_LOOT_RECEIVED", LuaEvents_EncounterLootReceived);

                Lua.Events.AttachEvent("DUEL_REQUESTED", LuaEvents_DuelRequested);
                Lua.Events.AttachEvent("PARTY_INVITE_REQUEST", LuaEvents_PartyInviteRequest);

                Lua.Events.AttachEvent("UI_ERROR_MESSAGE", LuaEvents_UiErrorMessage);

                Lua.Events.AttachEvent("UNIT_SPELLCAST_INTERRUPTED", LuaEvents_UnitSpellcastInterrupted);
                Lua.Events.AttachEvent("UNIT_SPELLCAST_SUCCEEDED", LuaEvents_UnitSpellcastSucceeded);

                Lua.Events.AttachEvent("TAXIMAP_OPENED", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("GOSSIP_SHOW", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("MAIL_SHOW", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("TRAINER_SHOW", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("LOOT_OPENED", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("CINEMATIC_START", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("QUEST_DETAIL", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("QUEST_PROGRESS", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("QUEST_COMPLETE", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("QUEST_GREETING", LuaEvents_DialogOpened);
                Lua.Events.AttachEvent("MERCHANT_SHOW", LuaEvents_DialogOpened);

                Lua.Events.AttachEvent("LOOT_CLOSED", LuaEvents_DialogClosed);

                Lua.Events.AttachEvent("UNIT_ENTERING_VEHICLE", LuaEvents_UnitEnteringVehicle);
                Lua.Events.AttachEvent("UNIT_EXITED_VEHICLE", LuaEvents_UnitExitedVehicle);

                Lua.Events.AttachEvent("LFG_PROPOSAL_SHOW", LuaEvents_LfgProposalShow);
                Lua.Events.AttachEvent("LFG_PROPOSAL_SUCCEEDED", LuaEvents_LfgProposalSucceeded);
                Lua.Events.AttachEvent("LFG_PROPOSAL_FAILED", LuaEvents_LfgProposalFailed);

                Lua.Events.AttachEvent("UPDATE_PENDING_MAIL", LuaEvents_UpdatePendingMail);

                Lua.Events.AttachEvent("ARTIFACT_XP_UPDATE", LuaEvents_ArtifactXpUpdateArtifactUpdate);
                Lua.Events.AttachEvent("ARTIFACT_UPDATE", LuaEvents_ArtifactXpUpdateArtifactUpdate);

                _bagItems = ToBagItem(Me.BagItems);
            }
            catch (Exception _ex)
            {
                DevLog("Exception: {0}", _ex);
            }
        }

        private void LuaEvents_ArtifactXpUpdateArtifactUpdate(object _sender, LuaEventArgs _args)
        {
            DevLog("Artifact(XP)Update");
            _checkedTraits = false;
        }

        private void LuaEvents_UpdatePendingMail(object _sender, LuaEventArgs _args)
        {
            DevLog("UpdatePendingMail");
            _checkedMails = false;
        }

        private void LuaEvents_LfgProposalSucceeded(object _sender, LuaEventArgs _args)
        {
            DungeonState = Dungeon.Succeeded;
            DevLog("LFG succeeded");
        }

        private void LuaEvents_LfgProposalFailed(object _sender, LuaEventArgs _args)
        {
            DungeonState = Dungeon.InQueue;
            Log("Proposal failed");
            _init = false;
        }

        private void LuaEvents_LfgProposalShow(object _sender, LuaEventArgs _args)
        {
            DungeonState = Dungeon.Proposal;

            ThreadPool.QueueUserWorkItem(t =>
            {
                Thread.Sleep(1000);

                Log("Accepting proposal");
                Lua.DoString("AcceptProposal()");
            });
        }

        private void LuaEvents_UnitExitedVehicle(object _sender, LuaEventArgs _args)
        {
            if ((string)_args.Args[0] == "player")
                _timeExitVehicle = DateTime.Now;
        }

        private void LuaEvents_UnitEnteringVehicle(object _sender, LuaEventArgs _args)
        {
            if ((string)_args.Args[0] == "player")
                _timeEnterVehicle = DateTime.Now;
        }

        private void LuaEvents_DialogClosed(object _sender, LuaEventArgs _args)
        {
            _timeDialogClosed = DateTime.Now;
            var _eventName = _args.EventName;
            DevLog("DialogClosed ({0})", _args.EventName);
        }

        private void LuaEvents_DialogOpened(object _sender, LuaEventArgs _args)
        {
            _timeDialogOpened = DateTime.Now;
            DevLog("DialogOpened ({0})", _args.EventName);
        }

        private double GetGatheringSpellId(SkillLine _skill)
        {
            var _skillLineAbilities = SkillLineAbility.GetAbilities()
                .Where(a => a.SkillLine == _skill &&
                            SpellManager.HasSpell(a.SpellId));

            var _gatheringSpell = _skillLineAbilities
                .FirstOrDefault(a => WoWSpell.FromId(a.SpellId).SpellEffects.Count() > 0 &&
                                     WoWSpell.FromId(a.SpellId).SpellEffects.Any(se => se.EffectType == WoWSpellEffectType.OpenLock));

            if (_gatheringSpell == null)
                return -1;

            return _gatheringSpell.SpellId;
        }

        private bool IsGatheringSpell(double _spellId)
        {
            var _gatheringSpellIds = new List<double>()
            {
                GetGatheringSpellId(SkillLine.Herbalism),
                GetGatheringSpellId(SkillLine.Mining),
                191910                                              // PickQuest
            };

            return _gatheringSpellIds.Contains(_spellId);
        }

        private void LuaEvents_UnitSpellcastSucceeded(object _sender, LuaEventArgs _args)
        {
            var _spellId = (double)_args.Args[4];
            if (IsGatheringSpell(_spellId))
                _timeSucceeded = DateTime.Now;
        }

        private void LuaEvents_UnitSpellcastInterrupted(object _sender, LuaEventArgs _args)
        {
            /*
            var _spellId = (double)_args.Args[4];
            if (IsGatheringSpell(_spellId))
                _timeInterrupted = DateTime.Now;
            */
            _timeInterrupted = DateTime.Now;
        }

        private void LuaEvents_UiErrorMessage(object _sender, LuaEventArgs _args)
        {
            if ((string)_args.Args[1] == _mountErrorMessage)
                _timeMountError = DateTime.Now;
        }

        private void LuaEvents_DuelRequested(object _sender, LuaEventArgs _args)
        {
            DialogState = Dialog.Duel;
        }

        private void LuaEvents_PartyInviteRequest(object _sender, LuaEventArgs _args)
        {
            DialogState = Dialog.Party;
        }

        private void LuaEvents_PlayerDead(object _sender, LuaEventArgs _args)
        {
            _timeDied = DateTime.Now;
            DevLog("Died at {0}", DateTime.Now);

            if (DeltaTime(_timeStartRareAttack, "s") < 120)
            {
                DevLog("Blacklist RareNpc for 1 hour");
                DevLog("     {0} (Id: {1})", _lastRareName, _lastRareId);

                if (!_ignoredGeneralGuids.ContainsKey(_lastRareGuid))
                    _ignoredGeneralGuids.Add(_lastRareGuid, DateTime.Now);
                else
                    _ignoredGeneralGuids[_lastRareGuid] = DateTime.Now;
            }

            if (_timeUnghost == DateTime.MinValue)
                return;

            var _deltaAlive = DeltaTime(_timeUnghost, "s");
            if (_deltaAlive < 60)
            {
                DevLog("*{0}  >  +{1} ({2}s)", _timeUnghost, _timeDied, Math.Round(_deltaAlive));
                _failedReviveCount++;
                DevLog("FailedReviveCount++ ({0})", _failedReviveCount);
            }
            else if (_failedReviveCount > 0)
            {
                _failedReviveCount = 0;
                DevLog("FailedReviveCount Reset");
            }
        }

        private void LuaEvents_PlayerUnghost(object _sender, LuaEventArgs _args)
        {
            _timeUnghost = DateTime.Now;
            Reset();
            DevLog("Unghost at {0}", DateTime.Now);
        }

        private void UpdateQuests(uint _questId = 0)
        {
            DevLog("UpdateQuests");

            ThreadPool.QueueUserWorkItem(t =>
            {
                while(true)
                {
                    var _timeElapsed = DeltaTime(_timeStartUpdateQuests, "ms");
                    if (_timeElapsed >= 7000)
                        break;

                    var _state = TreeRoot.State;
                    if ((int)_state == 0 || (int)_state == 4)
                    {
                        DevLog("     TreeRoot {0}", _state);
                        break;
                    }

                    Thread.Sleep(50);
                }
                _timeStartUpdateQuests = DateTime.Now;

                var _preCompletedCount = _completedQuests.Count();
                var _preAllCount = _allQuests.Count();

                while (true)
                {
                    try
                    {
                        var _timeElapsed = Math.Round(DeltaTime(_timeStartUpdateQuests, "ms"));
                        if (_timeElapsed >= 7000)
                            break;

                        if (_questId != 0 && !_completedQuests.Contains(_questId))
                            _completedQuests.Add(_questId);

                        if (StyxWoW.IsInWorld)
                        {
                            var _tempCompletedQuests = new List<uint>(QuestLog.Instance.GetCompletedQuests());
                            _completedQuests = _completedQuests.Union(_tempCompletedQuests).ToList();
                            _allQuests = QuestLog.Instance.GetAllQuests();
                        }

                        if (_preCompletedCount != _completedQuests.Count())
                        {
                            DevLog("     Completed: {0} > {1} ({2}ms)",
                                        _preCompletedCount,
                                        _completedQuests.Count(),
                                        _timeElapsed);
                            _preCompletedCount = _completedQuests.Count();
                        }

                        if (_preAllCount != _allQuests.Count())
                        {
                            DevLog("     All: {0} > {1} ({2}ms)",
                                _preAllCount,
                                _allQuests.Count(),
                                _timeElapsed);
                            _preAllCount = _allQuests.Count();
                        }

                        var _state = TreeRoot.State;
                        if ((int)_state == 0 || (int)_state == 4)
                        {
                            DevLog("     TreeRoot {0}", _state);
                            break;
                        }
                    }
                    catch (Exception _ex)
                    {
                        DevLog("Exception: {0}", _ex.ToString());
                    }

                    Thread.Sleep(500);
                }

                _checkedQuests = false;
            });
        }

        private void LuaEvents_QuestWatchUpdate(object _sender, LuaEventArgs _args)
        {
            var _questId = Convert.ToInt32(_args.Args[0]);

            DevLog("QuestWatchUpdate - QuestId: {0}", _questId);
            UpdateQuests();
        }

        private void LuaEvents_QuestTurnedIn(object _sender, LuaEventArgs _args)
        {
            var _questId = Convert.ToInt32(_args.Args[0]);
            if (_questId == 31012 || _questId == 30987)
                CreateTrainerData();

            DevLog("QuestTurnedIn - QuestId: {0}", _questId);
            UpdateQuests((uint)_questId);
        }

        private void LuaEvents_QuestAccepted(object _sender, LuaEventArgs _args)
        {
            var _questId = (_args.Args.Count() > 1)
                                    ? Convert.ToInt32(_args.Args[1])
                                    : 0;
            DevLog("QuestAccepted - QuestId: {0}", _questId);
            UpdateQuests();
        }

        private void LuaEvents_QuestRemoved(object _sender, LuaEventArgs _args)
        {
            var _questId = Convert.ToInt32(_args.Args[0]);
            DevLog("QuestRemoved - QuestId: {0}", _questId);
            UpdateQuests();
        }

        private void LuaEvents_EncounterLootReceived(object _sender, LuaEventArgs _args)
        {
            return;

            ThreadPool.QueueUserWorkItem(t =>
            {
                Thread.Sleep(1000);

                var _itemId = Convert.ToInt32(_args.Args[1]);
                var _quantity = Convert.ToInt32(_args.Args[3]);
                WoWItem _item = null;

                try
                {
                    _item = GetItem(_itemId);
                    if (_item == null)
                        return;
                }
                catch (Exception _ex)
                {
                    DevLog("{0}", _ex.Message);
                    return;
                }

                if (IsArtifactRelic(_item))
                    _checkedRelics = false;

                var _string = String.Format("  +  {0} (Id: {1}) {2}",
                                                          _item.Name,
                                                          _item.Entry,
                                                          (_quantity > 1)
                                                                ? $"x{_quantity}"
                                                                : "");
                DevLog("{0}", _string);
            });
        }

        private class BagItem
        {
            public BagItem(string _name,
                           uint _id,
                           int _count)
            {
                Name = _name;
                Id = _id;
                Count = _count;
            }

            public string Name { get; }
            public uint Id { get; }
            public int Count { get; }
        }

        private List<BagItem> GetDeltaList(List<BagItem> _preList, List<BagItem> _postList)
        {
            var _bothList = _preList.Union(_postList).ToList();
            var _list = new List<BagItem>();
            var _ignoredItemIds = new List<uint>();

            foreach (var _bagItem in _bothList)
            {
                var _itemId = _bagItem.Id;
                if (_ignoredItemIds.Contains(_itemId))
                    continue;

                var _preItem = _preList.FirstOrDefault(bi => bi.Id == _itemId);
                var _preCount = (_preItem == null)
                                    ? 0
                                    : _preItem.Count;
                var _postItem = _postList.FirstOrDefault(bi => bi.Id == _itemId);
                var _postCount = (_postItem == null)
                                    ? 0
                                    : _postItem.Count;
                var _deltaCount = _postCount - _preCount;

                if (_deltaCount == 0)
                    continue;

                _list.Add(new BagItem(_bagItem.Name, _itemId, _deltaCount));
                _ignoredItemIds.Add(_itemId);
            }

            if (_list.Count == 0)
                return _list;

            return _list
                        .OrderByDescending(bi => bi.Count)
                        .ToList();
        }

        private List<BagItem> ToBagItem(List<WoWItem> _list)
        {
            var _bagItems = new List<BagItem>();
            var _ignoredItemIds = new List<uint>();

            foreach (var _item in _list)
            {
                var _itemId = _item.Entry;
                if (_ignoredItemIds.Contains(_itemId))
                    continue;

                var _count = GetItemCount((int)_itemId, _list);

                _bagItems.Add(new BagItem(_item.Name, _itemId, _count));
                _ignoredItemIds.Add(_itemId);
            }

            return _bagItems;
        }

        private void PrintDeltaItemsInfo()
        {
            var _preList = _bagItems;
            var _postList = ToBagItem(Me.BagItems);
            if (_preList.Count == 0 && _postList.Count == 0)
                return;

            var _deltaList = GetDeltaList(_preList, _postList);
            if (_deltaList.Count == 0)
                return;

            var _maxCount = _deltaList.Max(bi => Math.Abs(bi.Count));
            var _maxNumSpaces = (int)Math.Floor(Math.Log10(Math.Abs(_maxCount)));

            foreach (var _bagItem in _deltaList)
            {
                var _count = _bagItem.Count;
                var _digits = (int)Math.Floor(Math.Log10(Math.Abs(_count)) + 1);
                var _numSpaces = _maxNumSpaces - (_digits - 1);
                var _leadingSpaces = new String(' ', _numSpaces);
                var _sign = (_count > 0)
                                ? "+"
                                : "-";

                var _string = String.Format("  {0}  {1}{2} x {3} (Id: {4})",
                                    _sign,
                                    _leadingSpaces,
                                    Math.Abs(_count),
                                    _bagItem.Name,
                                    _bagItem.Id);

                DevLog(_string);
            }

            _bagItems = _postList;
        }

        private void LuaEvents_BagUpdateDelayed(object _sender, LuaEventArgs _args)
        {
            _checkedItems = false;
            _timeCheckedItems = DateTime.Now;

            return;

            PrintDeltaItemsInfo();
            DevLog("{0}", DeltaTime(_timeCheckedItems, "ms"));
        }

        private void LuaEvents_PlayerLevelUp(object _sender, LuaEventArgs _args)
        {
            var _profile = GetAutoLoader();
            if (Me.Level == 98 || !_profile.Contains("1-110 Autoloader"))
                _reloadProfile = true;

            _checkedItems = false;
            _checkedQuests = false;
            _checkedMails = false;
            _checkedGatheringSkills = false;
        }

        public override void Pulse()
        {
        }

        private void BotEvents_OnOnBotPaused(object _sender, EventArgs e)
        {
            Lua.DoString("SetCVar('AutoInteract', 0)");
            if (!Settings.Performance_LowGraphicMode)
                return;

            Log("Restoring Graphic Mode");
            LowGraphicMode(false);
        }

        private void BotEvents_OnOnBotResumed(object _sender, EventArgs e)
        {
            if (!Settings.Performance_LowGraphicMode)
                return;

            Log("Enabling Low Graphic Mode");
            LowGraphicMode();
        }

        private void BotEvents_OnOnBotStarted(EventArgs args)
        {
            Reset(true);

            DisableAddons();

            string _profile = ProfileManager.XmlLocation;
            DevLog("Bot Start");
            DevLog("      Current Profile: {0}", (_profile == null)
                                                    ? "null"
                                                    : _profile);

            if (Settings.Quest_PandarenFaction.ToString() != "NotYetChosen" && Me.IsNeutralPandaren) //Me.Race == WoWRace.Pandaren && (!IsQuestCompleted(31012) && !IsQuestCompleted(30987)))
            {
                if (_profile != null && _profile.ToLower().Contains("questing profile pack") && _profile.ToLower().Replace("auto loader", "autoloader").Contains("autoloader"))
                {
                    if (Settings.Quest_QuestingPack.ToString() == "Old")
                    {
                        var _factionLetter = Settings.Quest_PandarenFaction.ToString().Substring(0, 1);
                        if (_profile.ToLower().Contains("store:///"))
                            _profile = $"store:///1-110 Questing Profile Pack/Classic/[{_factionLetter} - Quest] 1-58 [Kick]/[{_factionLetter} - Quest] 1-12 [Kick]/[{_factionLetter} - Quest] Panda 1-12 [Kick].xml";
                        else
                            _profile = $"{Application.StartupPath}\\..\\Default Profiles\\Questing Profile Pack\\Classic\\[{_factionLetter} - Quest] 1-58 [Kick]\\[{_factionLetter} - Quest] 1-12 [Kick]\\[{_factionLetter} - Quest] Panda 1-12 [Kick].xml";
                    }
                    else
                    {
                        var _faction = Settings.Quest_PandarenFaction.ToString();
                        if (_profile.ToLower().Contains("store:///"))
                            _profile = $"store:///1-110 Questing Profile Pack/{_faction}/Starters/Pandaren.xml";
                        else
                            _profile = $"{Application.StartupPath}\\..\\Default Profiles\\Questing Profile Pack\\{_faction}\\Starters\\Pandaren.xml";
                    }

                    DevLog("      New Profile: {0}", _profile);

                    try
                    {
                        ProfileManager.LoadNew(_profile);
                    }
                    catch (Exception)
                    {
                            //
                    }
                }
            }

            if(Me.InVehicle)
            {
                DevLog("BotStart > VehicleExit");
                Lua.DoString("VehicleExit()");
            }

            if (IsFrameVisible("TaxiFrame", true) || IsFrameVisible("FlightMapFrame", true))
                TaxiFrame.Instance.Hide();
        }

        private string GetAutoLoader()
        {
            RuntimeLog(">>>");

            var _profile = ProfileManager.XmlLocation;
            if (_profile == null || _profile == "")
                return _profile;

            RuntimeLog(">>>");

            var _customLoaderProfile = Settings.Quest_CustomLoader.ToString();
            if (_customLoaderProfile != "0")
                return _customLoaderProfile;

            RuntimeLog(">>>");

            if (Settings.Quest_AutoLoader.ToString() == "Disabled")
                return _profile;

            if (_profile.ToLower().Replace("auto loader", "autoloader")
                                  .Contains("autoloader"))
                return _profile;

            if (BotManager.Current.ToString() != "Questing")
                return _profile;

            if (Settings.Quest_QuestingPack.ToString() == "None")
                return _profile;

            if (Settings.Quest_QuestingPack.ToString() == "Old")
            {
                if (_profile.ToLower().Contains("store:///"))
                    return "store:///1-110 Questing Profile Pack/Auto Loader - v2.xml";
                else
                    return $"{Application.StartupPath}\\..\\Default Profiles\\Questing Profile Pack\\Auto Loader - v2.xml";
            }
            else if (Settings.Quest_QuestingPack.ToString() == "New")
                return $"{Application.StartupPath}\\..\\Default Profiles\\Questing Profile Pack\\1-110 Autoloader.xml";

            return _profile;
        }

        /*
        private void RestartBot()
        {
            Reset();
            TreeRoot.Stop();

            if (!Settings.General_InfiniteMode)
            {
                ThreadPool.QueueUserWorkItem(t =>
                {
                    TreeRoot.WaitForStop();
                    TreeRoot.Start();
                });
            }
        }
        */

        private void InfiniteMode()
        {
            if (!Settings.General_InfiniteMode)
            {
                Reset(true);
                return;
            }

            Reset();

            ThreadPool.QueueUserWorkItem(t =>
            {
                Thread.Sleep(2000);
                // DeltaTime(_timeExceptionThrown, "ms") <= 2000
                DevLog("TimeProfileLoaded: {0}", DeltaTime(_timeProfileLoaded, "s"));

                if (DeltaTime(_timeProfileLoaded, "s") > 5)
                {
                    Log("Enabling Infinite Mode");

                    var _profile = ProfileManager.XmlLocation;
                    DevLog("Bot Stop");
                    DevLog("      Current Profile: {0}", (_profile == null)
                                                            ? "null"
                                                            : _profile);

                    if (Me.Race != WoWRace.Pandaren || IsQuestDone(31012) || IsQuestDone(30987))
                        _profile = GetAutoLoader();

                    TreeRoot.WaitForStop();

                    if (StyxWoW.IsInWorld && Me.InVehicle)
                    {
                        DevLog("BotStart > VehicleExit");
                        Lua.DoString("VehicleExit()");
                    }

                    if (IsFrameVisible("QuestFrame"))
                        Log("Closing QuestFrame");
                    else if (IsFrameVisible("GossipFrame"))
                        Log("Closing GossipFrame");

                    while (IsFrameVisible("QuestFrame") || IsFrameVisible("GossipFrame"))
                    {
                        DevLog("WHILE: BotStop > QuestFrame/GossipFrame");
                        if (IsFrameVisible("QuestFrame"))
                            QuestFrame.Instance.Hide();
                        else if (IsFrameVisible("GossipFrame"))
                            GossipFrame.Instance.Hide();
                    }

                    DevLog("      New Profile: {0}", _profile);

                    try
                    {
                        ProfileManager.LoadNew(_profile);
                    }
                    catch (Exception)
                    {
                        //
                    }

                    TreeRoot.Start();
                }
            });
        }

        private void BotEvents_OnOnBotStopped(EventArgs _args)
        {
            if (Settings.Performance_LowGraphicMode)
            {
                Log("Restoring Graphic Mode");
                LowGraphicMode(false);
            }
            Lua.DoString("SetCVar('AutoInteract', 0)");

            var _action = Settings.Stop_Action;
            var _message = Settings.Stop_Message;
            if (_message != "0" && _action != "0")
            {
                ThreadPool.QueueUserWorkItem(t =>
                {
                    Thread.Sleep(1000);

                    if (_stopMessage)
                    {
                        DevLog("StopMessage {0}", _message);

                        if (Settings.Stop_HBRelog)
                            Bots.Professionbuddy.Dynamic.Helpers.HBRelog.StopProfile(Bots.Professionbuddy.Dynamic.Helpers.HBRelog.CurrentProfileName);

                        Bots.Professionbuddy.Dynamic.Helpers.HBRelog.StartProfile(Settings.Stop_Action);
                    }
                });
            }

            InfiniteMode();
        }
    }
}